@@ -1,5 +1,5 @@
-;;; acss-structure.el --- CSS -- Cascaded Speech Style structure
-;;; $Id: acss-structure.el 6915 2011-03-08 21:55:39Z tv.raman.tv $
+;;; acss-structure.el --- CSS -- Cascaded Speech Style structure  -*- lexical-binding: t; -*-
+;;; $Id$
 ;;; $Author: tv.raman.tv $
 ;;; Description: DTK Interface for Cascaded Speech Style Sheets
 ;;; Keywords:emacspeak, audio interface to emacs CSS
@@ -16,7 +16,7 @@
 ;;}}}
 ;;{{{  Copyright:
 
-;;;Copyright (C) 1995 -- 2011, T. V. Raman 
+;;;Copyright (C) 1995 -- 2018, T. V. Raman 
 ;;; Copyright (c) 1996 by T. V. Raman
 ;;; All Rights Reserved.
 ;;;
@@ -46,18 +46,17 @@
 ;;; A setting conforming to the CSS speech specification can be
 ;;; represented in elisp as a structure.
 
-;;; We will refer to this structure as a "speech style".
-;;; This structure needs to be mapped to device dependent codes to
-;;; produce the desired effect.
-;;; This module forms a bridge between User Agents e.g. Emacs-w3 that
-;;; wish to implement a speech style sheet
-;;; and Emacspeak's dectalk-voices module.
-;;; Emacspeak produces voice change effects by examining the value of
-;;; text-property 'personality.
+;;; We will refer to this structure as a "speech style".  This
+;;; structure needs to be mapped to device dependent codes to produce
+;;; the desired effect.  This module forms a bridge between emacs
+;;; packages    that wish to implement audio formatting 
+;;; and Emacspeak's TTS module.  Emacspeak produces voice
+;;; change effects by examining the value of text-property
+;;; 'personality.
 
 ;;; Think of a buffer of formatted text along with the text-property
 ;;; 'personality appropriately set as a "aural display list".
-;;; Applications like W3 that produce such formatted buffers  call function
+;;; Applications like EWW that produce such formatted buffers  call function
 ;;; acss-personality-from-speech-style  with a  "speech-style"
 ;;; --a structure as defined in this module and get back a symbol that
 ;;; they then assign to the value of property 'personality.
@@ -78,10 +77,17 @@
 ;;{{{  Required modules
 
 ;;; Code:
-(require 'cl)
-(declaim  (optimize  (safety 0) (speed 3)))
+(require 'cl-lib)
+(cl-declaim  (optimize  (safety 0) (speed 3)))
+(require 'tts)
+;;}}}
+;;{{{ Forward Declarations:
+
+(declare-function  tts-voice-defined-p  "dectalk-voices.el" (voice))
+(declare-function tts-define-voice-from-speech-style "dectalk-voices.el" (name style))
 
 ;;}}}
+
 ;;{{{ tts common vars 
 (defvar tts-default-voice 'paul 
   "Default voice used. ")
@@ -89,7 +95,7 @@
 ;;}}}
 ;;{{{  A speech style structure
 
-(defstruct  acss
+(cl-defstruct  acss
   family
   gain left-volume right-volume
   average-pitch
@@ -104,17 +110,22 @@
 
 ;;; may be redefined at runtime when alternative tts engine is
 ;;; configured.
-(defalias 'tts-voice-defined-p 'dectalk-voice-defined-p)
-(defalias 'tts-define-voice-from-speech-style
-  'dectalk-define-voice-from-speech-style)
+
+(declare-function dectalk-voice-defined-p "dectalk-voices.el" (voice))
+(declare-function dectalk-define-voice-from-speech-style "dectalk-voices.el" (name style))
+
+(unless (fboundp 'tts-voice-defined-p)
+  (fset  'tts-voice-defined-p 'dectalk-voice-defined-p))
+
+(unless (fboundp 'tts-define-voice-from-speech-style)
+  (fset  'tts-define-voice-from-speech-style #'dectalk-define-voice-from-speech-style))
 
 (defun acss-personality-from-speech-style (style)
   "First compute a symbol that will be name for this STYLE.
 Then see if a voice defined for it.
 Finally return the symbol"
   (cond
-   ((and (acss-gain style)
-         (= 0 (acss-gain style)))
+   ((and (acss-gain style) (= 0 (acss-gain style)))
     'inaudible)
    (t
     (let ((f (acss-family style))
@@ -127,36 +138,23 @@ Finally return the symbol"
       (setq name 
             (intern
              (format "acss%s%s%s%s%s%s"
-                     (if f
-                         (format "-%s" f)
-                       "")
-                     (if a
-                         (format "-a%s" a)
-                       "")
-                     (if p
-                         (format "-p%s" p)
-                       "")
-                     (if s
-                         (format "-s%s" s)
-                       "")
-                     (if r
-                         (format "-r%s" r)
-                       "")
-                     (if m
-                         (format "-%s" m)
-                       ""))))
+                     (if f (format "-%s" f) "")
+                     (if a (format "-a%s" a) "")
+                     (if p (format "-p%s" p) "")
+                     (if s (format "-s%s" s) "")
+                     (if r (format "-r%s" r) "")
+                     (if m (format "-%s" m) ""))))
       (unless (tts-voice-defined-p name)
         (tts-define-voice-from-speech-style name style))
       name))))
 
 ;;}}}
 (provide  'acss-structure)
-(provide 'dtk-css-speech) ; for legacy: W3
 ;;{{{  emacs local variables
 
 ;;; local variables:
 ;;; folded-file: t
-;;; byte-compile-dynamic: nil
+;;; byte-compile-dynamic: t
 ;;; end:
 
 ;;}}}
