@@ -1,5 +1,5 @@
-;;; dectalk-voices.el --- Define various device independent voices in terms of Dectalk codes.
-;;; $Id: dectalk-voices.el 7511 2012-02-13 21:55:37Z tv.raman.tv $
+;;; dectalk-voices.el --- Define various device independent voices in terms of Dectalk codes  -*- lexical-binding: t; -*-
+;;; $Id$
 ;;; $Author: tv.raman.tv $
 ;;; Description:  Module to set up Dectalk voices and personalities
 ;;; Keywords: Voice, Personality, Dectalk
@@ -15,7 +15,7 @@
 
 ;;}}}
 ;;{{{  Copyright:
-;;;Copyright (C) 1995 -- 2011, T. V. Raman
+;;;Copyright (C) 1995 -- 2018, T. V. Raman
 ;;; Copyright (c) 1994, 1995 by Digital Equipment Corporation.
 ;;; All Rights Reserved.
 ;;;
@@ -48,12 +48,27 @@
 ;;{{{ required modules
 
 ;;; Code:
-(require 'cl)
-(declaim  (optimize  (safety 0) (speed 3)))
+(require 'cl-lib)
+(cl-declaim  (optimize  (safety 0) (speed 3)))
 (require 'acss-structure)
+(require 'tts)
+
+;;}}}
+;;{{{ Customizations:
+;;;###autoload
+(defcustom dectalk-default-speech-rate 225
+  "*Default speech rate at which TTS is started. "
+  :group 'tts
+  :type 'integer
+  :set #'(lambda(sym val)
+           (set-default sym val)
+           (when (and (getenv "DTK_PROGRAM")
+                      (string-match "dtk" (getenv "DTK_PROGRAM")))
+             (setq-default dtk-speech-rate val))))
 
 ;;}}}
 ;;{{{  Top-level TTS  switcher
+
 ;;;### autoload
 (defun dectalk ()
   "Select Dectalk TTS server."
@@ -69,10 +84,10 @@
 
 ;;; From dtk-speak.el:
 
-(defvar tts-default-speech-rate )
-(defvar dectalk-default-speech-rate )
-(defvar dtk-speech-rate-step )
-(defvar dtk-speech-rate-base )
+(defvar tts-default-speech-rate)
+
+(defvar dtk-speech-rate-step)
+(defvar dtk-speech-rate-base)
 
 ;;}}}
 ;;{{{  voice table
@@ -84,26 +99,26 @@
   "Association between symbols and strings to set Dectalk voices.
 The string can set any Dectalk parameter.")
 
-(defsubst dectalk-define-voice (name command-string)
+(defun dectalk-define-voice (name command-string)
   "Define a Dectalk voice named NAME.
 This voice will be set   by sending the string
 COMMAND-STRING to the Dectalk."
-  (declare (special dectalk-voice-table ))
+  (cl-declare (special dectalk-voice-table))
   (puthash  name command-string  dectalk-voice-table))
 
-(defsubst dectalk-get-voice-command (name)
+(defun dectalk-get-voice-command (name)
   "Retrieve command string for  voice NAME."
-  (declare (special dectalk-voice-table ))
+  (cl-declare (special dectalk-voice-table))
   (cond
    ((listp name)
     (mapconcat #'dectalk-get-voice-command name " "))
    (t (or  (gethash name dectalk-voice-table)
            dectalk-default-voice-string))))
 
-(defsubst dectalk-voice-defined-p (name)
+(defun dectalk-voice-defined-p (name)
   "Check if there is a voice named NAME defined."
-  (declare (special dectalk-voice-table ))
-  (gethash name dectalk-voice-table ))
+  (cl-declare (special dectalk-voice-table))
+  (gethash name dectalk-voice-table))
 
 ;;}}}
 ;;{{{ voice definitions
@@ -133,20 +148,20 @@ COMMAND-STRING to the Dectalk."
 (defvar dectalk-family-table nil
   "Association list of Dectalk voice names and control codes.")
 
-(defsubst dectalk-set-family-code (name code)
+(defun dectalk-set-family-code (name code)
   "Set control code for voice family NAME  to CODE."
-  (declare (special dectalk-family-table))
+  (cl-declare (special dectalk-family-table))
   (when (stringp name)
     (setq name (intern name)))
   (setq dectalk-family-table
-        (cons (list name code )
+        (cons (list name code)
               dectalk-family-table)))
 
-(defsubst dectalk-get-family-code (name)
+(defun dectalk-get-family-code (name)
   "Get control code for voice family NAME."
-  (declare (special dectalk-family-table ))
+  (cl-declare (special dectalk-family-table))
   (when (stringp name)
-    (setq name (intern name )))
+    (setq name (intern name)))
   (or (cadr (assq  name dectalk-family-table))
       ""))
 
@@ -168,17 +183,17 @@ COMMAND-STRING to the Dectalk."
 Keys are symbols of the form <FamilyName-Dimension>.
 Values are vectors holding the control codes for the 10 settings.")
 
-(defsubst dectalk-css-set-code-table (family dimension table)
+(defun dectalk-css-set-code-table (family dimension table)
   "Set up voice FAMILY.
 Argument DIMENSION is the dimension being set,
 and TABLE gives the values along that dimension."
-  (declare (special dectalk-css-code-tables))
+  (cl-declare (special dectalk-css-code-tables))
   (let ((key (intern (format "%s-%s" family dimension))))
-    (puthash  key table dectalk-css-code-tables )))
+    (puthash  key table dectalk-css-code-tables)))
 
-(defsubst dectalk-css-get-code-table (family dimension)
+(defun dectalk-css-get-code-table (family dimension)
   "Retrieve table of values for specified FAMILY and DIMENSION."
-  (declare (special dectalk-css-code-tables))
+  (cl-declare (special dectalk-css-code-tables))
   (let ((key (intern (format "%s-%s" family dimension))))
     (gethash key dectalk-css-code-tables)))
 
@@ -202,84 +217,81 @@ and TABLE gives the values along that dimension."
 ;;{{{  paul average pitch
 
 (let ((table (make-vector 10 "")))
-  (mapcar
-   (function
-    (lambda (setting)
-      (aset table
-            (first setting)
-            (format " ap %s hs % s"
-                    (second setting)
-                    (third setting)))))
+  (mapc
+   #'(lambda (setting)
+       (aset table
+             (cl-first setting)
+             (format " ap %s hs % s"
+                     (cl-second setting)
+                     (cl-third setting))))
    '(
      (0 96 115)
      (1 101 112)
      (2 108 109)
      (3 112 106)
-     (4 118 103 )
+     (4 118 103)
      (5 122  100)
      (6 128 98)
      (7 134 96)
      (8 140 94)
      (9 147 91)
      ))
-  (dectalk-css-set-code-table 'paul 'average-pitch table ))
+  (dectalk-css-set-code-table 'paul 'average-pitch table))
 
 ;;}}}
 ;;{{{  harry average pitch
 ;;; Harry  has a big head --and a lower pitch for the middle setting
 
 (let ((table (make-vector 10 "")))
-  (mapcar
-   (function
-    (lambda (setting)
-      (aset table
-            (first setting)
-            (format " ap %s hs % s"
-                    (second setting)
-                    (third setting)))))
+  (mapc
+   #'(lambda (setting)
+       (aset table
+             (cl-first setting)
+             (format " ap %s hs % s"
+                     (cl-second setting)
+                     (cl-third setting))))
    '(
      (0 50 125)
      (1 59 123)
      (2 68 121)
      (3 77 120)
-     (4 83  118 )
+     (4 83  118)
      (5 89 115)
      (6 95 112)
      (7 110 105)
      (8 125 100)
      (9 140 95)
      ))
-  (dectalk-css-set-code-table 'harry 'average-pitch table ))
+  (dectalk-css-set-code-table 'harry 'average-pitch table))
 
 ;;}}}
 ;;{{{  betty average pitch
 
 (let ((table (make-vector 10 "")))
-  (mapcar
-   (function
-    (lambda (setting)
-      (aset table
-            (first setting)
-            (format " ap %s hs % s"
-                    (second setting)
-                    (third setting)))))
+  (mapc
+   #'(lambda (setting)
+       (aset table
+             (cl-first setting)
+             (format " ap %s hs % s"
+                     (cl-second setting)
+                     (cl-third setting))))
    '(
      (0 160 115)
      (1 170 112)
      (2 181 109)
      (3 192 106)
-     (4 200 103 )
+     (4 200 103)
      (5 208  100)
      (6 219 98)
      (7 225  96)
      (8 240 94)
      (9 260  91)
      ))
-  (dectalk-css-set-code-table 'betty 'average-pitch table ))
+  (dectalk-css-set-code-table 'betty 'average-pitch table))
 
 ;;}}}
 
-(defsubst dectalk-get-average-pitch-code (value family)
+(defun dectalk-get-average-pitch-code (value family)
   "Get  AVERAGE-PITCH for specified VALUE and  FAMILY."
   (or family (setq family 'paul))
   (if value
@@ -299,82 +311,79 @@ and TABLE gives the values along that dimension."
 ;;{{{  paul pitch range
 
 (let ((table (make-vector 10 "")))
-  (mapcar
-   (function
-    (lambda (setting)
-      (aset table
-            (first setting)
-            (format " pr %s as %s "
-                    (second setting)
-                    (third setting)))))
+  (mapc
+   #'(lambda (setting)
+       (aset table
+             (cl-first setting)
+             (format " pr %s as %s "
+                     (cl-second setting)
+                     (cl-third setting))))
    '(
      (0 0 0)
      (1 20 10)
      (2 40 20)
      (3 60 30)
-     (4 80 40 )
-     (5 100 50 )
+     (4 80 40)
+     (5 100 50)
      (6 137 60)
      (7 174 70)
      (8 211 80)
      (9 250 100)
      ))
-  (dectalk-css-set-code-table 'paul 'pitch-range table ))
+  (dectalk-css-set-code-table 'paul 'pitch-range table))
 
 ;;}}}
 ;;{{{  harry pitch range
 
 (let ((table (make-vector 10 "")))
-  (mapcar
-   (function
-    (lambda (setting)
-      (aset table
-            (first setting)
-            (format " pr %s as %s "
-                    (second setting)
-                    (third setting)))))
+  (mapc
+   #'(lambda (setting)
+       (aset table
+             (cl-first setting)
+             (format " pr %s as %s "
+                     (cl-second setting)
+                     (cl-third setting))))
    '(
      (0 0 0)
      (1 16 20)
      (2 32 40)
      (3 48 60)
-     (4 64 80 )
-     (5 80 100 )
+     (4 64 80)
+     (5 80 100)
      (6 137 100)
      (7 174 100)
      (8 211 100)
      (9 250 100)
      ))
-  (dectalk-css-set-code-table 'harry 'pitch-range table ))
+  (dectalk-css-set-code-table 'harry 'pitch-range table))
 
 ;;}}}
 ;;{{{  betty pitch range
 
 (let ((table (make-vector 10 "")))
-  (mapcar
-   (function
-    (lambda (setting)
-      (aset table
-            (first setting)
-            (format " pr %s as %s "
-                    (second setting)
-                    (third setting)))))
+  (mapc
+   #'(lambda (setting)
+       (aset table
+             (cl-first setting)
+             (format " pr %s as %s "
+                     (cl-second setting)
+                     (cl-third setting))))
    '(
      (0 0 0)
      (1 50 10)
      (2 80 20)
      (3 100 25)
-     (4 110 30 )
+     (4 110 30)
      (5 140 35)
      (6 165 57)
      (7 190 75)
      (8 220 87)
      (9 250 100)
      ))
-  (dectalk-css-set-code-table 'betty 'pitch-range table ))
+  (dectalk-css-set-code-table 'betty 'pitch-range table))
 
 ;;}}}
-(defsubst dectalk-get-pitch-range-code (value family)
+(defun dectalk-get-pitch-range-code (value family)
   "Get pitch-range code for specified VALUE and FAMILY."
   (or family (setq family 'paul))
   (if value
@@ -397,21 +406,20 @@ and TABLE gives the values along that dimension."
 ;;{{{  paul stress
 
 (let ((table (make-vector 10 "")))
-  (mapcar
-   (function
-    (lambda (setting)
-      (aset table
-            (first setting)
-            (format " hr %s sr %s qu %s bf %s "
-                    (second setting)
-                    (third setting)
-                    (fourth setting)
-                    (fifth setting)))))
+  (mapc
+   #'(lambda (setting)
+       (aset table
+             (cl-first setting)
+             (format " hr %s sr %s qu %s bf %s "
+                     (cl-second setting)
+                     (cl-third setting)
+                     (cl-fourth setting)
+                     (cl-fifth setting))))
    '(
      (0  0 0 0 0)
      (1 3 6  20 3)
      (2 6 12  40 6)
-     (3 9 18  60 9 )
+     (3 9 18  60 9)
      (4 12 24 80 14)
      (5 18 32  100 18)
      (6 34 50 100 20)
@@ -425,22 +433,21 @@ and TABLE gives the values along that dimension."
 ;;{{{  harry stress
 
 (let ((table (make-vector 10 "")))
-  (mapcar
-   (function
-    (lambda (setting)
-      (aset table
-            (first setting)
-            (format " hr %s sr %s qu %s bf %s "
-                    (second setting)
-                    (third setting)
-                    (fourth setting)
-                    (fifth setting)))))
+  (mapc
+   #'(lambda (setting)
+       (aset table
+             (cl-first setting)
+             (format " hr %s sr %s qu %s bf %s "
+                     (cl-second setting)
+                     (cl-third setting)
+                     (cl-fourth setting)
+                     (cl-fifth setting))))
    '(
      (0  0 0 0 0)
-     (1 4 6 2 2 )
-     (2 8 12 4 4 )
-     (3 12 18 6 6 )
-     (4 16 24 8 8 )
+     (1 4 6 2 2)
+     (2 8 12 4 4)
+     (3 12 18 6 6)
+     (4 16 24 8 8)
      (5 20 30 10 9)
      (6 40  48 32 16)
      (7 60 66 54 22)
@@ -453,21 +460,20 @@ and TABLE gives the values along that dimension."
 ;;{{{  betty stress
 
 (let ((table (make-vector 10 "")))
-  (mapcar
-   (function
-    (lambda (setting)
-      (aset table
-            (first setting)
-            (format " hr %s sr %s qu %s bf %s "
-                    (second setting)
-                    (third setting)
-                    (fourth setting)
-                    (fifth setting)))))
+  (mapc
+   #'(lambda (setting)
+       (aset table
+             (cl-first setting)
+             (format " hr %s sr %s qu %s bf %s "
+                     (cl-second setting)
+                     (cl-third setting)
+                     (cl-fourth setting)
+                     (cl-fifth setting))))
    '(
      (0  1 1 0 0)
      (1 3 4 11 0)
      (2 5 8 22 0)
-     (3 8 12 33 0 )
+     (3 8 12 33 0)
      (4 11  16 44 0)
      (5 14 20 55 0)
      (6 35 40 65 10)
@@ -478,8 +484,8 @@ and TABLE gives the values along that dimension."
   (dectalk-css-set-code-table 'betty 'stress table))
 
 ;;}}}
-(defsubst dectalk-get-stress-code (value family)
-  (or family (setq family 'paul ))
+(defun dectalk-get-stress-code (value family)
+  (or family (setq family 'paul))
   (if value
       (aref (dectalk-css-get-code-table family 'stress)
             value)
@@ -494,21 +500,20 @@ and TABLE gives the values along that dimension."
 ;;{{{  paul richness
 
 (let ((table (make-vector 10 "")))
-  (mapcar
-   (function
-    (lambda (setting)
-      (aset table (first setting)
-            (format " ri %s sm %s "
-                    (second setting)
-                    (third setting)))))
+  (mapc
+   #'(lambda (setting)
+       (aset table (cl-first setting)
+             (format " ri %s sm %s "
+                     (cl-second setting)
+                     (cl-third setting))))
    '(
      (0 0 100)
      (1 14 80)
      (2 28 60)
-     (3 42 40 )
+     (3 42 40)
      (4 56 20)
-     (5 70  3 )
-     (6 60 24 )
+     (5 70  3)
+     (6 60 24)
      (7 70 16)
      (8 80 8 20)
      (9 100  0)
@@ -519,13 +524,12 @@ and TABLE gives the values along that dimension."
 ;;{{{  harry richness
 
 (let ((table (make-vector 10 "")))
-  (mapcar
-   (function
-    (lambda (setting)
-      (aset table (first setting)
-            (format " ri %s sm %s "
-                    (second setting)
-                    (third setting)))))
+  (mapc
+   #'(lambda (setting)
+       (aset table (cl-first setting)
+             (format " ri %s sm %s "
+                     (cl-second setting)
+                     (cl-third setting))))
    '(
      (0 100 0)
      (1 96 3)
@@ -533,7 +537,7 @@ and TABLE gives the values along that dimension."
      (3 90 9)
      (4 88 11)
      (5 86 12)
-     (6 60 24 )
+     (6 60 24)
      (7 40 44)
      (8 20 65)
      (9 0 70)
@@ -544,13 +548,12 @@ and TABLE gives the values along that dimension."
 ;;{{{  betty richness
 
 (let ((table (make-vector 10 "")))
-  (mapcar
-   (function
-    (lambda (setting)
-      (aset table (first setting)
-            (format " ri %s sm %s "
-                    (second setting)
-                    (third setting)))))
+  (mapc
+   #'(lambda (setting)
+       (aset table (cl-first setting)
+             (format " ri %s sm %s "
+                     (cl-second setting)
+                     (cl-third setting))))
    '(
      (0 0 100)
      (1 8 76)
@@ -558,7 +561,7 @@ and TABLE gives the values along that dimension."
      (3 24  28)
      (4 32 10)
      (5 40 4)
-     (6 50 3 )
+     (6 50 3)
      (7 65 3)
      (8 80 8 2)
      (9 100  0)
@@ -567,7 +570,7 @@ and TABLE gives the values along that dimension."
 
 ;;}}}
 
-(defsubst dectalk-get-richness-code (value family)
+(defun dectalk-get-richness-code (value family)
   (or family (setq family 'paul))
   (if value
       (aref (dectalk-css-get-code-table family 'richness)
@@ -577,7 +580,7 @@ and TABLE gives the values along that dimension."
 ;;}}}
 ;;{{{  punctuations
 
-(defsubst dectalk-get-punctuations-code (value)
+(defun dectalk-get-punctuations-code (value)
   "Return string needed to set specified punctuations mode."
   (if value
       (format " :pu %s " value)
@@ -597,12 +600,12 @@ and TABLE gives the values along that dimension."
                   (dectalk-get-punctuations-code (acss-punctuations style))
                   (when (or (acss-average-pitch style)
                             (acss-pitch-range style)
-                            (acss-stress style )
+                            (acss-stress style)
                             (acss-richness style))
                     (concat " :dv "
                             (dectalk-get-average-pitch-code (acss-average-pitch style) family)
                             (dectalk-get-pitch-range-code (acss-pitch-range style) family)
-                            (dectalk-get-stress-code (acss-stress style ) family)
+                            (dectalk-get-stress-code (acss-stress style) family)
                             (dectalk-get-richness-code (acss-richness style) family)))
                   "]")))
     (dectalk-define-voice name command)))
@@ -612,18 +615,17 @@ and TABLE gives the values along that dimension."
 
 (defun dectalk-list-voices ()
   "List defined voices."
-  (declare (special dectalk-voice-table))
-  (loop for k being the hash-keys of dectalk-voice-table
-        collect   k))
+  (cl-declare (special dectalk-voice-table))
+  (cl-loop for k being the hash-keys of dectalk-voice-table
+           collect   k))
 
 ;;}}}
 ;;{{{ configurater
 ;;;###autoload
 (defun dectalk-configure-tts ()
   "Configures TTS environment to use Dectalk family of synthesizers."
-  (declare (special  dectalk-default-speech-rate
-                     tts-default-speech-rate
-                     tts-default-voice))
+  (cl-declare (special  dectalk-default-speech-rate
+                        tts-default-speech-rate tts-default-voice))
   (setq tts-default-voice 'paul)
   (fset 'tts-list-voices 'dectalk-list-voices)
   (fset 'tts-voice-defined-p 'dectalk-voice-defined-p)
@@ -637,6 +639,25 @@ and TABLE gives the values along that dimension."
   (setq-default dtk-speech-rate-step 50
                 dtk-speech-rate-base 150))
 
+;;}}}
+;;{{{  tts-env for Dectalk:
+
+;;;###autoload
+(defun dectalk-make-tts-env  ()
+  "Constructs a TTS environment for Dectalk."
+  (cl-declare (special dectalk-default-speech-rate))
+  (make-tts-env
+   :name :dectalk
+   :default-voice 'paul
+   :default-speech-rate dectalk-default-speech-rate
+   :list-voices #'dectalk-list-voices
+   :acss-voice-defined-p #'dectalk-voice-defined-p
+   :get-acss-voice-command #'dectalk-get-voice-command
+   :define-voice-from-acss #'dectalk-define-voice-from-speech-style
+   :speech-rate-base 150 :speech-rate-step 50))
+
+(tts-env-set :dectalk  (dectalk-make-tts-env))
+
 ;;}}}
 (provide 'dectalk-voices)
 ;;{{{  emacs local variables
