@@ -1,4 +1,4 @@
-;;; dtk-unicode.el --- Pronounce more characters correctly
+;;; dtk-unicode.el --- Pronounce Unicode characters correctly  -*- lexical-binding: t; -*-
 ;;{{{ Header: Lukas
 
 ;; Copyright 2007, 2011 Lukas Loehrer
@@ -6,7 +6,7 @@
 ;;; Using patch from Lukas.
 ;;
 ;; Author: Lukas Loehrer <loehrerl |at| gmx.net>
-;; Version: $Id: dtk-unicode.el 8146 2013-02-09 20:05:08Z tv.raman.tv $
+;; Version: $Id$
 ;; Keywords:  TTS, Unicode
 
 ;;}}}
@@ -22,7 +22,7 @@
 
 ;;}}}
 ;;{{{  Copyright:
-;;;Copyright (C) 1995 -- 2011, T. V. Raman
+;;;Copyright (C) 1995 -- 2018, T. V. Raman
 ;;; Copyright (c) 1994, 1995 by Digital Equipment Corporation.
 ;;; All Rights Reserved.
 ;;;
@@ -47,7 +47,7 @@
 
 ;;; Commentary:
 
-;; 
+;;
 ;;; This  Provides Unicode support to the speech layer.
 
 ;;; Code:
@@ -55,10 +55,9 @@
 ;;}}}
 ;;{{{ Preamble
 
-(require 'cl)
-(declaim  (optimize  (safety 0) (speed 3)))
+(require 'cl-lib)
+(cl-declaim (optimize (safety 0) (speed 3)))
 (require 'descr-text)
-
 ;;}}}
 ;;{{{ Customizations
 
@@ -70,38 +69,39 @@
 ;;;###autoload
 (defcustom dtk-unicode-character-replacement-alist
   '(
-    (? . "-")                   ; START OF GUARDED AREA
-    (?━ .  "-")                     ; horiz bars
-    (?┃ . "|")                      ; vertical block
-    (?° . " degrees ")              ; degree sign
-    (?℃ . "Degree C")               ; celsius
-    (?℉ . "Degree F ")              ; Fahrenheit
-    (?“ . "\"")                     ;LEFT DOUBLE QUOTATION MARK
-    (?” . "\"")                     ; RIGHT DOUBLE QUOTATION MARK
-    (?⋆ . "*")                      ; STAR OPERATOR
-    (?‘ . " backquote  ")           ; LEFT SINGLE QUOTATION MARK
-    (?’ . "'")                      ; right SINGLE QUOTATION MARK
-    (?‐ . "-")                      ; hyphen
-    (?– . "--")                     ; n-dash
-    (?— . "---")                    ; m-dash
-    (?― . "----")                   ; horizontal bar 
-    (?‖ . "||")                     ; vertical bar
-    (?… . "...")                    ; ellipses
-    (?• . " bullet ")               ; bullet
-    (? . " ... ")               ; message-waiting
-    (?™ . "TM")                     ; trademark
-    (?ﬀ . "ff")                     ; latin small ligature ff
-    (?ﬁ . "fi")                     ; latin small ligature fi
-    (?ﬂ . "fl")                     ; latin small ligature fl
-    (?ﬃ . "ffi")                    ; latin small ligature ffi
-    (?ﬄ . "Ffl")                    ; latin small ligature ffl
+    (? . "-")                       ; START OF GUARDED AREA
+    (?━ . "-")                          ; horiz bars
+    (?┃ . "|")                          ; vertical block
+    (?° . " degrees ")                  ; degree sign
+    (?℃ . "Degree C")                   ; celsius
+    (?℉ . "Degree F ")                  ; Fahrenheit
+    (?“ . "\"")                         ;LEFT DOUBLE QUOTATION MARK
+    (?” . "\"")                         ; RIGHT DOUBLE QUOTATION MARK
+    (?⋆ . "*")                          ; STAR OPERATOR
+    (?­ . "-")                          ; soft-hyphen
+    (?‘ . " backquote  ")               ; LEFT SINGLE QUOTATION MARK
+    (?’ . "'")                          ; right SINGLE QUOTATION MARK
+    (?‐ . "hyphen")                     ; hyphenm
+    (?– . " -- ")                       ; n-dash
+    (?— . " --- ")                      ; m-dash
+    (?  . " ")                          ; hair space
+    (?― . "----")                       ; horizontal bar
+    (?‖ . "||")                         ; vertical bar
+    (?… . "...")                        ; ellipses
+    (?• . " bullet ")                   ; bullet
+    (? . " ... ")                   ; message-waiting
+    (?™ . "TM")                         ; trademark
+    (?ﬀ . "ff")                         ; latin small ligature ff
+    (?ﬁ . "fi")                         ; latin small ligature fi
+    (?ﬂ . "fl")                         ; latin small ligature fl
+    (?ﬃ . "ffi")                        ; latin small ligature ffi
+    (?ﬄ . "Ffl")                        ; latin small ligature ffl
     )
   "Explicit replacements for some characters."
   :group 'dtk-unicode
   :type '(alist
           :key-type (character :tag "character")
-          :value-type (string :tag "replacement"))
-  )
+          :value-type (string :tag "replacement")))
 
 ;;;###autoload
 (defcustom dtk-unicode-process-utf8 t
@@ -112,16 +112,20 @@ themselves, e.g., when using an Asian language."
 
 (defcustom dtk-unicode-name-transformation-rules-alist
   '(
-    ("^greek\\( small\\| capital\\)? letter \\(.*\\)$" .  (lambda (s) (match-string 2 s)))
-    ("^latin\\( small\\| capital\\)? letter \\(.*\\)$" .  (lambda (s) (match-string 2 s)))
-    ("\\(.*\\) sign$" . (lambda (s) (match-string 1 s)))
+    ("BOX DRAWING" . (lambda (s) "."))
+    ("^greek\\( small\\| capital\\)? letter \\(.*\\)$" . (lambda (s) (match-string 2 s)))
+    ("^latin\\( small\\| capital\\)? letter \\(.*\\)$" . (lambda (s)
+                                                           (match-string 2 s)))
+    ("^DEVANAGARI \\(sign\\|vowel sign\\|letter\\)? \\(.*\\)$" . (lambda (s) (match-string 2 s)))
+                                        ;("\\(.*\\) sign$" . (lambda (s) (match-string 1 s)))
     )
   "Alist of character name transformation rules."
   :group 'dtk-unicode
-  :type '(repeat (cons :value ("." . identity)
-                       (regexp :tag "pattern")
-                       (function :tag "transformation")))
-  )
+  :type
+  '(repeat
+    (cons :value ("." . identity)
+          (regexp :tag "pattern")
+          (function :tag "transformation"))))
 
 ;;}}}
 ;;{{{ Variables
@@ -150,8 +154,7 @@ A handler returns a non-nil value if the   replacement was successful, nil other
    ((eq charset 'eight-bit-graphic)
     (list 160 255))
    (t
-    (let* ((dim (charset-dimension charset))
-           (chars (charset-chars charset))
+    (let* ((chars (charset-chars charset))
            min max)
       (if (eq chars 96)
           (setq min 32 max 127)
@@ -161,9 +164,9 @@ A handler returns a non-nil value if the   replacement was successful, nil other
 (defun dtk-unicode-build-skip-regexp (charsets)
   "Construct regexp to match all but the characters in dtk-unicode-untouched-charsets."
   (format "[^%s]"
-          (loop for charset in charsets
-                when (charsetp charset)
-                concat (apply 'format "%c-%c" (dtk-unicode-charset-limits charset)))))
+          (cl-loop for charset in charsets
+                   when (charsetp charset)
+                   concat (apply 'format "%c-%c" (dtk-unicode-charset-limits charset)))))
 
 (defvar dtk-unicode-charset-filter-regexp
   (dtk-unicode-build-skip-regexp dtk-unicode-untouched-charsets)
@@ -175,30 +178,24 @@ A handler returns a non-nil value if the   replacement was successful, nil other
   (setq dtk-unicode-charset-filter-regexp (dtk-unicode-build-skip-regexp dtk-unicode-untouched-charsets)))
 
 (eval-and-compile
-  (if (> emacs-major-version 22)
-      (progn
-        (defmacro with-charset-priority (charsets &rest body)
-          "Execute BODY like `progn' with CHARSETS at the front of priority list.
+  (defmacro dtk--with-charset-priority (charsets &rest body)
+    "Execute BODY like `progn' with CHARSETS at the front of priority list.
 CHARSETS is a list of charsets.  See
 `set-charset-priority'.  This affects the implicit sorting of lists of
 charsets returned by operations such as `find-charset-region'."
-          (let ((current (make-symbol "current")))
-            `(let ((,current (charset-priority-list)))
-               (apply #'set-charset-priority ,charsets)
-               (unwind-protect
-                   (progn ,@body)
-                 (apply #'set-charset-priority ,current)))))
+    (declare (indent 1) (debug t))
+    (let ((current (make-symbol "current")))
+      `(let ((,current (charset-priority-list)))
+         (apply #'set-charset-priority ,charsets)
+         (unwind-protect
+             (progn ,@body)
+           (apply #'set-charset-priority ,current)))))
+;;; Now use it:
+  (defun dtk-unicode-char-in-charsets-p (char charsets)
+    "Return t if CHAR is a member of one in the charsets in CHARSETS."
+    (dtk--with-charset-priority charsets (memq (char-charset char) charsets))))
 
-        (defun dtk-unicode-char-in-charsets-p  (char charsets)
-          "Return t if CHAR is a member of one in the charsets in CHARSETS."
-          (with-charset-priority charsets
-                                 (memq (char-charset char) charsets))))
-    ;; emacs-major-version <= 22
-    (defun dtk-unicode-char-in-charsets-p (char charsets)
-      "Return t if CHAR is a member of one in the charsets in CHARSETS."
-      (memq (char-charset char) charsets))))
-
-(defsubst dtk-unicode-char-untouched-p (char)
+(defun dtk-unicode-char-untouched-p (char)
   "Return t if char is a member of one of the charsets in dtk-unicode-untouched-charsets."
   (dtk-unicode-char-in-charsets-p char dtk-unicode-untouched-charsets))
 
@@ -215,37 +212,34 @@ charsets returned by operations such as `find-charset-region'."
           (puthash char ad-return-value dtk-unicode-cache))
       (setq ad-return-value result))))
 
-(defsubst dtk-unicode-char-properties (char)
-  "Return unicode properties for CHAR.
-
-Converts char to unicode if necessary (for emacs 22)."
-  (let ((unicode (encode-char char 'ucs)))
-    (and unicode (condition-case nil
-                     (let ((emacspeak-speak-errors nil)
-                           (emacspeak-speak-messages nil))
-                       (describe-char-unicode-data unicode))
-                   (error nil)))))
-
-(defsubst dtk-unicode-char-property (char prop-name)
-  "Get character property by name."
-  (second (assoc prop-name (dtk-unicode-char-properties char))))
-
-(defsubst dtk-unicode-name-for-char (char)
+(defun dtk-unicode-name-for-char (char)
   "Return unicode name for character CHAR.
 nil if CHAR is not in Unicode."
-  (downcase
-   (or (car (rassoc char (ucs-names)))
-       (cadar (describe-char-unicode-data char))
-       "")))
+  (cond
+   ((= char 128) "")
+   (t
+    (downcase
+     (or
+      (get-char-code-property char 'name)
+      (get-char-code-property char 'old-name)
+      (format "%c" char))))))
+(defun dtk-unicode-char-properties (char)
+  "Return unicode properties for CHAR."
+  (let ((unicode (encode-char char 'ucs)))
+    (when unicode (describe-char-unicode-data unicode))))
 
-(defsubst dtk-unicode-char-punctuation-p (char)
+(defun dtk-unicode-char-property (char prop-name)
+  "Get character property by name."
+  (cl-second (assoc prop-name (dtk-unicode-char-properties char))))
+
+(defun dtk-unicode-char-punctuation-p (char)
   "Use unicode properties to determine whether CHAR is a ppunctuation character."
   (let ((category (dtk-unicode-char-property char "Category"))
         (case-fold-search t))
     (when (stringp category)
       (string-match "punctuation" category))))
 
-(defsubst dtk-unicode-apply-name-transformation-rules (name)
+(defun dtk-unicode-apply-name-transformation-rules (name)
   "Apply transformation rules in dtk-unicode-name-transformation-rules-alist to NAME."
   (funcall
    (or (assoc-default name dtk-unicode-name-transformation-rules-alist 'string-match)
@@ -258,8 +252,8 @@ nil if CHAR is not in Unicode."
 When called interactively, CHAR defaults to the character after point."
   (interactive (list (following-char)))
   (setq dtk-unicode-character-replacement-alist
-        (loop for elem in dtk-unicode-character-replacement-alist
-              unless (eq (car elem) char) collect elem)))
+        (cl-loop for elem in dtk-unicode-character-replacement-alist
+                 unless (eq (car elem) char) collect elem)))
 
 (defun dtk-unicode-customize-char (char replacement)
   "Add a custom replacement string for CHAR.
@@ -275,15 +269,15 @@ When called interactively, CHAR defaults to the character after point."
 ;;}}}
 ;;{{{ Character replacement handlers
 
-(defsubst dtk-unicode-user-table-handler (char)
+(defun dtk-unicode-user-table-handler (char)
   "Return user defined replacement character if it exists."
   (cdr (assq char dtk-unicode-character-replacement-alist)))
 
-(defsubst dtk-unicode-full-table-handler (char)
+(defun dtk-unicode-full-table-handler (char)
   "Uses the unicode data file to find the name of CHAR."
   (let ((char-desc (dtk-unicode-name-for-char char)))
     (when char-desc
-      (format  " %s " (dtk-unicode-apply-name-transformation-rules char-desc)))))
+      (format " %s " (dtk-unicode-apply-name-transformation-rules char-desc)))))
 
 ;;}}}
 ;;{{{ External interface
@@ -293,7 +287,6 @@ When called interactively, CHAR defaults to the character after point."
 
 This is meant to be used in places where the user asks for a detailed description of CHAR."
   (dtk-unicode-name-for-char char))
-
 (defun dtk-unicode-short-name-for-char (char)
   "Return name of CHAR.
 
@@ -308,15 +301,15 @@ This is meant to be used in places where the user asks for a short description o
 This is the main entry point for this module.
 The argument MODE specifies the current punctuation mode.
 Does nothing for unibyte buffers."
-  (declare (special dtk-unicode-process-utf8))
-  (when  dtk-unicode-process-utf8
+  (cl-declare (special dtk-unicode-process-utf8))
+  (when dtk-unicode-process-utf8
     (let ((inhibit-read-only t))
       (goto-char (point-min))
-      (while (re-search-forward dtk-unicode-charset-filter-regexp  nil t)
+      (while (re-search-forward dtk-unicode-charset-filter-regexp nil t)
         (let* ((pos (match-beginning 0))
                (char (char-after pos))
                (replacement
-                (save-match-data  
+                (save-match-data
                   (if (and (eq mode 'none) (dtk-unicode-char-punctuation-p char))
                       " "
                     (run-hook-with-args-until-success 'dtk-unicode-handlers char)))))
@@ -324,7 +317,10 @@ Does nothing for unibyte buffers."
             (let ((props (text-properties-at pos)))
               (replace-match replacement t t nil)
               (when props
-                (set-text-properties pos (point) props)))))))))
+                (set-text-properties pos (point) props))
+              (add-text-properties
+               pos (point)
+               '(personality voice-animate)))))))))
 
 ;;}}}
 (provide 'dtk-unicode)
@@ -334,7 +330,7 @@ Does nothing for unibyte buffers."
 ;;; local variables:
 ;;; coding: utf-8
 ;;; folded-file: t
-;;; byte-compile-dynamic: nil
+;;; byte-compile-dynamic: t
 ;;; end:
 
 ;;}}}
