@@ -1,4 +1,4 @@
-;;; emacspeak-apt-sources.el --- speech-enable APT's sources.list file editor
+;;; emacspeak-apt-sources.el --- speech-enable APT's sources.list file editor  -*- lexical-binding: t; -*-
 ;;; Description:  Emacspeak extension to speech-enable editing of sources.list
 ;;; Keywords: Emacspeak, apt, sources.list
 ;;{{{  LCD Archive entry:
@@ -38,8 +38,10 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 ;;{{{ required modules
-
+(require 'cl-lib)
+(cl-declaim  (optimize  (safety 0) (speed 3)))
 (require 'emacspeak-preamble)
+(require 'apt-sources nil 'no-error)
 ;;}}}
 ;;{{{  Introduction:
 
@@ -52,8 +54,8 @@
 
 ;;}}}
 ;;{{{  Required modules
+(cl-declaim  (optimize  (safety 0) (speed 3)))
 (require 'emacspeak-preamble)
-(require 'arc-mode)
 ;;}}}
 ;;{{{ Advice interactive commands to speak.
 
@@ -63,38 +65,37 @@
 
 (defadvice apt-sources-previous-source-line (after emacspeak pre act comp)
   "Provide auditory feedback."
-  (when (ems-interactive-p )
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'select-object)
     (emacspeak-speak-line)))
 
 (defadvice apt-sources-next-source-line (after emacspeak pre act comp)
   "Provide auditory feedback."
-  (when (ems-interactive-p )
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'select-object)
     (emacspeak-speak-line)))
 
 (defadvice apt-sources-deb-or-src-replicate (after emacspeak pre act comp)
   "Provide auditory feedback."
-  (when (ems-interactive-p )
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'yank-object)))
 
 (defadvice apt-sources-insert-local-vars (after emacspeak pre act comp)
   "Provide auditory feedback."
-  (when (ems-interactive-p )
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'yank-object)))
 
 (defadvice apt-sources-new-source (after emacspeak pre act comp)
   "Provide auditory feedback."
-  (when (ems-interactive-p )
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'yank-object)))
 
 (defadvice apt-sources-around-lines (after emacspeak pre act comp)
   "Provide auditory feedback."
-  (when (ems-interactive-p )
-    (emacspeak-auditory-icon
-     (if apt-sources-around-lines
-         'on
-       'off))))
+  (when (ems-interactive-p)
+    (emacspeak-auditory-icon (if apt-sources-around-lines
+                                 'on
+                               'off))))
 
 ;;}}}
 (provide 'emacspeak-apt-sources)
@@ -102,7 +103,7 @@
 
 ;;; local variables:
 ;;; folded-file: t
-;;; byte-compile-dynamic: nil
+;;; byte-compile-dynamic: t
 ;;; end: 
 
 ;;}}}
