@@ -1,5 +1,5 @@
-;;; emacspeak-arc.el --- Speech enable archive-mode -- a  Emacs interface to zip and friends
-;;; $Id: emacspeak-arc.el 6708 2011-01-04 02:27:29Z tv.raman.tv $
+;;; emacspeak-arc.el --- Speech enable archive-mode -- a  Emacs interface to zip and friends  -*- lexical-binding: t; -*-
+;;; $Id$
 ;;; $Author: tv.raman.tv $ 
 ;;; Description: Auditory interface to archive mode
 ;;; Keywords: Emacspeak, Speak, Spoken Output, archive
@@ -16,7 +16,7 @@
 ;;}}}
 ;;{{{  Copyright:
 
-;;; Copyright (c) 1995 -- 2011, T. V. Raman
+;;; Copyright (c) 1995 -- 2018, T. V. Raman
 ;;; All Rights Reserved. 
 ;;;
 ;;; This file is not part of GNU Emacs, but the same permissions apply.
@@ -39,13 +39,15 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 ;;{{{  Required modules
+(cl-declaim  (optimize  (safety 0) (speed 3)))
 (require 'emacspeak-preamble)
 (require 'arc-mode)
 ;;}}}
 ;;{{{  Introduction 
-
+;;; Commentary:
 ;;;Auditory interface to archive mode
-
+;;; This lets Emacs manipulate package files such as .zip and .jar files.
+;;; Code:
 ;;}}}
 ;;{{{ Helpers
 
@@ -66,52 +68,52 @@
 
 (defadvice archive-mark (after emacspeak pre act comp)
   "Provide auditory feedback"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'mark-object)
     (emacspeak-archive-speak-line)))
 
 (defadvice archive-next-line (after emacspeak pre act comp)
   "Provide spoken feedback"
-  (when (interactive-p )
+  (when (ems-interactive-p)
     (emacspeak-archive-speak-line)))
 
 (defadvice archive-previous-line (after emacspeak pre act comp)
   "Provide spoken feedback"
-  (when (interactive-p )
+  (when (ems-interactive-p)
     (emacspeak-archive-speak-line)))
 
 (defadvice archive-flag-deleted (after emacspeak pre act comp)
   "Provide auditory feedback"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'delete-object)
     (emacspeak-archive-speak-line)))
 
 (defadvice archive-unflag (after emacspeak pre act comp)
   "Provide auditory feedback"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'yank-object)
     (emacspeak-archive-speak-line)))
 (defadvice archive-unflag-backwards (after emacspeak pre act comp)
   "Provide auditory feedback"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'yank-object)
     (emacspeak-archive-speak-line)))
 
 (defadvice archive-extract (after emacspeak pre act comp)
   "Provide auditory feedback"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'open-object)
     (emacspeak-speak-mode-line)))
 
 (defadvice archive-extract-other-window (after emacspeak pre act comp)
   "Provide auditory feedback"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'open-object)
     (emacspeak-speak-mode-line)))
 
 (defadvice archive-view (after emacspeak pre act comp)
   "Provide auditory feedback"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'open-object)
     (emacspeak-speak-mode-line)))
 
@@ -121,34 +123,32 @@
 (defvar emacspeak-arc-header-list-format nil
   "Field names in the header line")
 
-(defsubst emacspeak-arc-get-header-line-format ()
+(defun emacspeak-arc-get-header-line-format ()
   "Return  header line format vector, after
 first initializing it if necessary."
-  (declare (special emacspeak-arc-header-list-format))
+  (cl-declare (special emacspeak-arc-header-list-format))
   (unless emacspeak-arc-header-list-format
     (let ((line nil)
           (fields nil))
       (save-excursion
         (goto-char (point-min))
-        (setq line (thing-at-point 'line)))
+        (setq line (ems-this-line)))
       (setq fields (split-string line))
-      (loop for f in fields 
-            and i from 0
-            do
-            (setq emacspeak-arc-header-list-format
-                  (cons
-                   (list f i )
-                   emacspeak-arc-header-list-format)))))
+      (cl-loop for f in fields 
+               and i from 0
+               do
+               (setq emacspeak-arc-header-list-format
+                     (cons
+                      (list f i)
+                      emacspeak-arc-header-list-format)))))
   emacspeak-arc-header-list-format)
-(defsubst emacspeak-arc-get-field-index (field)
-  (let ((marked-p (save-excursion
-                    (beginning-of-line)
-                    (= (following-char) ?\ )))
-        (position (cadr (assoc field
-                               (emacspeak-arc-get-header-line-format)))))
-    (if marked-p
-        (1- position)
-      position )))
+(defun emacspeak-arc-get-field-index (field)
+  (let ((marked-p
+         (save-excursion
+           (beginning-of-line)
+           (= ?\  (following-char))))
+        (pos (cadr (assoc field (emacspeak-arc-get-header-line-format)))))
+    (if marked-p (1- pos) pos)))
 
 (defun emacspeak-arc-speak-file-name ()
   "Speak the name of the file on current line"
@@ -162,7 +162,7 @@ first initializing it if necessary."
      (t
       (message "File: %s"
                (nth  (emacspeak-arc-get-field-index "File")
-                     (split-string (thing-at-point 'line))))))))
+                     (split-string (ems-this-line))))))))
 
 (defun emacspeak-arc-speak-file-size ()
   "Speak the size of the file on current line"
@@ -176,7 +176,7 @@ first initializing it if necessary."
      (t
       (message "Size: %s"
                (nth  (emacspeak-arc-get-field-index "Length")
-                     (split-string (thing-at-point 'line))))))))
+                     (split-string (ems-this-line))))))))
 
 (defun emacspeak-arc-speak-file-modification-time ()
   "Speak modification time of the file on current line"
@@ -188,7 +188,7 @@ first initializing it if necessary."
      ((null entry)
       (message "No file on this line"))
      (t
-      (let* ((fields (split-string (thing-at-point 'line)))
+      (let* ((fields (split-string (ems-this-line)))
              (date (nth  (emacspeak-arc-get-field-index "Date")
                          fields))
              (time (nth  (emacspeak-arc-get-field-index "Time")
@@ -214,7 +214,7 @@ first initializing it if necessary."
                 mode)))))
 (defun emacspeak-arc-setup-keys ()
   "Setup emacspeak keys for arc mode"
-  (declare (special archive-mode-map))
+  (cl-declare (special archive-mode-map))
   (define-key archive-mode-map "." 'emacspeak-arc-speak-file-name)
   (define-key archive-mode-map "c" 'emacspeak-arc-speak-file-modification-time)
   (define-key archive-mode-map "z" 'emacspeak-arc-speak-file-size)
@@ -222,7 +222,7 @@ first initializing it if necessary."
     'emacspeak-arc-speak-file-permissions)
   )
 
-(eval-when (load)
+(cl-eval-when (load)
   (emacspeak-arc-setup-keys))
 ;;}}}
 (provide 'emacspeak-arc)
