@@ -1,5 +1,5 @@
-;;; emacspeak-auctex.el --- Speech enable AucTeX -- a powerful TeX/LaTeX authoring environment
-;;; $Id: emacspeak-auctex.el 6708 2011-01-04 02:27:29Z tv.raman.tv $
+;;; emacspeak-auctex.el --- Speech enable AucTeX -- a powerful TeX/LaTeX authoring environment  -*- lexical-binding: t; -*-
+;;; $Id$
 ;;; $Author: tv.raman.tv $
 ;;; DescriptionEmacspeak extensions for auctex-mode
 ;;; Keywords:emacspeak, audio interface to emacs AUCTEX
@@ -16,7 +16,7 @@
 ;;}}}
 ;;{{{  Copyright:
 
-;;;Copyright (C) 1995 -- 2011, T. V. Raman 
+;;;Copyright (C) 1995 -- 2018, T. V. Raman 
 ;;; Copyright (c) 1994, 1995 by Digital Equipment Corporation.
 ;;; All Rights Reserved. 
 ;;;
@@ -38,12 +38,14 @@
 
 ;;}}}
 ;;{{{ Required modules 
+(cl-declaim  (optimize  (safety 0) (speed 3)))
 (require 'emacspeak-preamble)
 ;;}}}
 ;;{{{  Introduction:
-
-;;; Provide additional advice to auctex
-
+;;; Commentary:
+;;; Speech-enables the AucTeX package.
+;;; AucTeX, now available from ELPA, has been my authoring environment of choice for writing LaTeX since 1991.
+;;; Code:
 ;;}}}
 ;;{{{ voice locking:
 
@@ -59,10 +61,6 @@
    (font-latex-string-face voice-lighten)
    (font-latex-subscript-face voice-smoothen)
    (font-latex-superscript-face voice-brighten)
-   (font-latex-title-1-face voice-bolden-extra)
-   (font-latex-title-2-face voice-bolden-medium)
-   (font-latex-title-3-face voice-bolden)
-   (font-latex-title-4-face voice-smoothen-extra)
    (font-latex-verbatim-face voice-monotone)
    (font-latex-warning-face voice-bolden-and-animate)
    ))
@@ -72,38 +70,38 @@
 
 (defadvice LaTeX-fill-paragraph (after emacspeak pre act  comp)
   "Provide auditory feedback."
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'fill-object)))
 
 (defadvice LaTeX-mark-section (after emacspeak pre act)
   "Speak the first line. 
 Also provide an auditory icon. "
-  (when (interactive-p) 
+  (when (ems-interactive-p) 
     (emacspeak-speak-line)
     (emacspeak-auditory-icon 'mark-object)))
 
 (defadvice LaTeX-mark-environment (after emacspeak pre act)
   "Speak the first line. 
 Also provide an auditory icon. "
-  (when (interactive-p) 
+  (when (ems-interactive-p) 
     (emacspeak-speak-line)
     (emacspeak-auditory-icon 'mark-object)))
 
-(defadvice LaTeX-format-paragraph (after emacspeak pre act )
+(defadvice LaTeX-format-paragraph (after emacspeak pre act)
   "Provide auditory feedback"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'fill-object)
     (message "Filled current paragraph")))
-(defadvice LaTeX-format-region (around emacspeak pre act )
+(defadvice LaTeX-format-region (around emacspeak pre act)
   "Ask for confirmation.
 Provide auditory feedback after formatting region"
   (cond
-   ((and (interactive-p)
+   ((and (ems-interactive-p)
          (y-or-n-p "Really format region? "))
     ad-do-it
     (emacspeak-auditory-icon 'fill-object)
     (message "Reformatted region"))
-   ((not (interactive-p)) ad-do-it))
+   ((not (ems-interactive-p)) ad-do-it))
   ad-return-value)
 
 ;;}}}
@@ -111,62 +109,65 @@ Provide auditory feedback after formatting region"
 
 (defadvice LaTeX-find-matching-begin (after emacspeak pre act)
   "Provide auditory feedback. "
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-speak-line)))
 
 (defadvice LaTeX-find-matching-end (after emacspeak pre act)
   "Provide auditory feedback. "
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-speak-line)))
 
 (defadvice LaTeX-close-environment (after emacspeak pre act)
   "Speak the inserted line. "
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'close-object)
     (emacspeak-read-previous-line)))
 
 (defadvice TeX-insert-quote(around emacspeak pre act com)
   "Speak quotes that were inserted."
   (cond
-   ((interactive-p)
+   ((ems-interactive-p)
     (let ((orig (point)))
       ad-do-it
       (emacspeak-speak-region orig (point))))
    (t ad-do-it))
   ad-return-value)
-(loop for f in
-      '(TeX-insert-dollar TeX-insert-backslash
-                          LaTeX-babel-insert-hyphen)
-      do
-      (eval
-       `(defadvice ,f (after emacspeak pre act comp)
-          "Speak what you inserted"
-          (when (interactive-p)
-            (emacspeak-speak-this-char  (preceding-char ))))))
+(unless (and (boundp 'post-self-insert-hook)
+             post-self-insert-hook
+             (memq 'emacspeak-post-self-insert-hook post-self-insert-hook))
+  (cl-loop for f in
+           '(TeX-insert-dollar TeX-insert-backslash
+                               LaTeX-babel-insert-hyphen)
+           do
+           (eval
+            `(defadvice ,f (after emacspeak pre act comp)
+               "Speak what you inserted"
+               (when (ems-interactive-p)
+                 (emacspeak-speak-this-char  (preceding-char)))))))
 
 ;;}}}
 ;;{{{  Inserting structures
 
 (defadvice TeX-newline (after emacspeak pre act comp)
   "Provide auditory feedback to indicate indentation."
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-speak-line)))
 
 (defadvice LaTeX-insert-item (after emacspeak pre act)
   "Provide auditory feedback. "
-  (when (interactive-p)
-    (emacspeak-speak-line )))
+  (when (ems-interactive-p)
+    (emacspeak-speak-line)))
 
 (defadvice LaTeX-environment (after emacspeak pre act)
   "Provide auditory feedback, by speaking
 the opening line of the newly inserted environment. "
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'open-object)
     (emacspeak-read-previous-line)))
 
 (defadvice TeX-insert-macro (around  emacspeak pre act)
   "Provide spoken feedback."
-  (let ((opoint (point )))
+  (let ((opoint (point)))
     ad-do-it
     (emacspeak-speak-region opoint (point))))
 
@@ -175,25 +176,25 @@ the opening line of the newly inserted environment. "
 
 (defadvice TeX-comment-region (after emacspeak pre act)
   "Provide spoken and auditory feedback. "
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-speak-line)
     (emacspeak-auditory-icon 'select-object)))
 
 (defadvice TeX-un-comment (after emacspeak pre act)
   "Provide spoken and auditory feedback. "
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-speak-line)
     (emacspeak-auditory-icon 'select-object)))
 
 (defadvice TeX-un-comment-region (after emacspeak pre act)
   "Provide spoken and auditory feedback. "
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-speak-line)
     (emacspeak-auditory-icon 'select-object)))
 
 (defadvice TeX-comment-paragraph (after emacspeak pre act)
   "Provide spoken and auditory feedback. "
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-speak-line)
     (emacspeak-auditory-icon 'select-object)))
 
@@ -202,9 +203,9 @@ the opening line of the newly inserted environment. "
 
 (defadvice TeX-next-error (after emacspeak pre act)
   "Speak the error line. "
-  (when (interactive-p)
-    (emacspeak-auditory-icon 'select-object)
-    (emacspeak-speak-line )))
+  (when (ems-interactive-p)
+    (emacspeak-auditory-icon 'item)
+    (emacspeak-speak-line)))
 
 ;;}}}
 ;;{{{  Hooks
@@ -214,8 +215,8 @@ the opening line of the newly inserted environment. "
 (add-hook  'LaTeX-mode-hook
            (function
             (lambda ()
-              (declare (special imenu-generic-expression
-                                imenu-create-index-function))
+              (cl-declare (special imenu-generic-expression
+                                   imenu-create-index-function))
               (require 'imenu)
               (setq imenu-create-index-function 'imenu-default-create-index-function)
               (setq imenu-generic-expression
@@ -230,7 +231,7 @@ the opening line of the newly inserted environment. "
 (defadvice TeX-font (around emacspeak pre act comp)
   "Speak the font we inserted"
   (cond 
-   ((interactive-p)
+   ((ems-interactive-p)
     (let ((orig (point)))
       ad-do-it
       (if (ad-get-arg 0)
