@@ -1,6 +1,6 @@
-;;; emacspeak-bbdb.el --- Speech enable BBDB -- a powerful address manager
+;;; emacspeak-bbdb.el --- Speech enable BBDB -- a powerful address manager  -*- lexical-binding: t; -*-
 
-;;; $Id: emacspeak-bbdb.el 6708 2011-01-04 02:27:29Z tv.raman.tv $
+;;; $Id$
 ;;; $Author: tv.raman.tv $ 
 ;;; DescriptionEmacspeak extensions for bbdb 
 ;;; Keywords:emacspeak, audio interface to emacs bbdb 
@@ -16,7 +16,7 @@
 
 ;;}}}
 ;;{{{  Copyright:
-;;;Copyright (C) 1995 -- 2011, T. V. Raman 
+;;;Copyright (C) 1995 -- 2018, T. V. Raman 
 ;;; Copyright (c) 1994, 1995 by Digital Equipment Corporation.
 ;;; All Rights Reserved. 
 ;;;
@@ -39,120 +39,121 @@
 ;;}}}
 
 ;;{{{  Required libraries
+(cl-declaim  (optimize  (safety 0) (speed 3)))
 (require 'emacspeak-preamble)
 ;;}}}
 ;;{{{  Introduction:
-
-;;; Provide additional advice to bbdb
-
+;;; Commentary:
+;;; Speech-enables BBDB.
+;;; I have used BBDB to manage email address and contact information since 1991.
+;;; Code:
 ;;}}}
 ;;{{{ personalities 
 
 (voice-setup-add-map
  '(
-   (bbdb-company voice-brighten)
-   (bbdb-field-name voice-smoothen)
-   (bbdb-field-value voice-animate)
-   (bbdb-name voice-bolden)))
+   (bbdb-field-name voice-monotone)
+   (bbdb-name voice-bolden)
+   (bbdb-organization voice-lighten)))
+
+;;}}}
 ;;{{{  Variable settings:
 
 ;;; Emacspeak will not work if bbdb is in electric mode
-(declaim (special bbdb-electric-p))
+(cl-declaim (special bbdb-electric-p))
 (setq bbdb-electric-p nil)
-(declaim (special bbdb-mode-map))
+(cl-declaim (special bbdb-mode-map))
 
-(add-hook 'bbdb-mode-hook
-          (function (lambda ()
-                      (define-key  bbdb-mode-map "b" 'bbdb)
-                      (define-key bbdb-mode-map "N" 'bbdb-name)
-                      (define-key bbdb-mode-map "c" 'bbdb-create)
-                      )))
-
-;;}}}
+(add-hook
+ 'bbdb-mode-hook
+ #'(lambda ()
+     (define-key  bbdb-mode-map "b" 'bbdb)
+     (define-key bbdb-mode-map "N" 'bbdb-name)
+     (define-key bbdb-mode-map "c" 'bbdb-create)
+     ))
 
 ;;}}}
 ;;{{{ Advice:
 
-(defadvice              bbdb-delete-current-field-or-record (after emacspeak pre act )
+(defadvice              bbdb-delete-current-field-or-record (after emacspeak pre act)
   "Provide auditory feedback"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'delete-object)
     (save-excursion
       (when (looking-at  "?")
         (forward-line 1))
       (emacspeak-speak-line))))
 
-(defadvice bbdb-edit-current-field (before emacspeak pre act )
+(defadvice bbdb-edit-current-field (before emacspeak pre act)
   "Provide auditory feedback"
-  (when (interactive-p)
-    (emacspeak-auditory-icon 'open-object )))
+  (when (ems-interactive-p)
+    (emacspeak-auditory-icon 'open-object)))
 
-(defadvice bbdb-send-mail (before emacspeak pre act )
+(defadvice bbdb-send-mail (before emacspeak pre act)
   "Provide auditory feedback"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (let ((to (if (consp (ad-get-arg 0))
                   (bbdb-dwim-net-address
                    (car (ad-get-arg 0)))
                 (bbdb-dwim-net-address
-                 (ad-get-arg 0 ))))
+                 (ad-get-arg 0))))
           (subject  (ad-get-arg 1)))
-      (emacspeak-auditory-icon 'open-object )
+      (emacspeak-auditory-icon 'open-object)
       (message "Starting an email message  %s to %s %s "
                (if subject  (format "about %s" subject) "")
                to
                (if  (consp (ad-get-arg 0))
                    " and others " " ")))))
 
-(defadvice bbdb-next-record (after emacspeak pre act )
+(defadvice bbdb-next-record (after emacspeak pre act)
   "Provide auditory feedback"
-  (when (interactive-p)
-    (emacspeak-auditory-icon 'large-movement )
+  (when (ems-interactive-p)
+    (emacspeak-auditory-icon 'large-movement)
     (save-excursion
       (when (looking-at  "?")
         (forward-line 1))
-      (emacspeak-speak-line ))))
+      (emacspeak-speak-line))))
 
-(defadvice bbdb-prev-record (after emacspeak pre act )
+(defadvice bbdb-prev-record (after emacspeak pre act)
   "Provide auditory feedback"
-  (when (interactive-p)
-    (emacspeak-auditory-icon 'large-movement )
+  (when (ems-interactive-p)
+    (emacspeak-auditory-icon 'large-movement)
     (save-excursion
       (when (looking-at  "?")
         (forward-line 1))
-      (emacspeak-speak-line ))))
+      (emacspeak-speak-line))))
 
-(defadvice bbdb-omit-record (after emacspeak pre act )
+(defadvice bbdb-omit-record (after emacspeak pre act)
   "Provide auditory feedback"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'close-object)
     (emacspeak-speak-line)))
 
-(defadvice bbdb-bury-buffer (after emacspeak pre act )
+(defadvice bbdb-bury-buffer (after emacspeak pre act)
   "Provide auditory feedback"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'close-object)
-    (emacspeak-speak-mode-line )))
+    (emacspeak-speak-mode-line)))
 
-(defadvice bbdb-elide-record (after emacspeak pre act )
+(defadvice bbdb-elide-record (after emacspeak pre act)
   "Provide auditory feedback"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (message "Toggled  record display")))
 
-(defadvice bbdb-transpose-fields (after emacspeak pre act )
+(defadvice bbdb-transpose-fields (after emacspeak pre act)
   "Provide auditory feedback"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'large-movement)
-    (emacspeak-speak-line )))
+    (emacspeak-speak-line)))
 
-(defadvice bbdb-complete-name (around emacspeak pre act )
+(defadvice bbdb-complete-name (around emacspeak pre act)
   "Provide spoken feedback"
-  (declare (special completion-reference-buffer))
+  (cl-declare (special completion-reference-buffer))
   (cond
-   ((interactive-p)
+   ((ems-interactive-p)
     (let ((prior (point))
           (completion-ignore-case t)
-          (completions nil )
-          (window (selected-window))
+          (completions nil)
           (buffer (current-buffer)))
       ad-do-it
       (cond
@@ -162,28 +163,28 @@
         (setq completion-reference-buffer buffer)
         (unless (get-text-property (point) 'mouse-face)
           (goto-char (next-single-property-change (point)
-                                                  'mouse-face )))
+                                                  'mouse-face)))
         (dtk-speak (emacspeak-get-current-completion)))
-       (t (dtk-speak (buffer-substring prior (point )))))))
-   (t ad-do-it ))
-  ad-return-value )
+       (t (dtk-speak (buffer-substring prior (point)))))))
+   (t ad-do-it))
+  ad-return-value)
 
 ;;}}}
 ;;{{{  Advice mail-ua  specific hooks
 
 (defadvice bbdb/vm-show-sender (after emacspeak pre act comp)
   "Provide spoken feedback"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-speak-other-window 1)))
 
 (defadvice bbdb/rmail-show-sender (after emacspeak pre act comp)
   "Provide spoken feedback"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-speak-other-window 1)))
 
 (defadvice bbdb/mh-show-sender (after emacspeak pre act comp)
   "Provide spoken feedback"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-speak-other-window 1)))
 
 ;;}}}
@@ -191,8 +192,8 @@
 
 (defadvice bbdb-update-records (around emacspeak pre act comp)
   "Silence messages."
-  (let ((emacspeak-speak-messages nil))
-    ad-do-it))
+  (ems-with-messages-silenced
+   ad-do-it))
 
 ;;}}}
 (provide  'emacspeak-bbdb)
