@@ -1,5 +1,5 @@
-;;; emacspeak-calc.el --- Speech enable the Emacs Calculator -- a powerful symbolic algebra system
-;;; $Id: emacspeak-calc.el 6708 2011-01-04 02:27:29Z tv.raman.tv $
+;;; emacspeak-calc.el --- Speech enable the Emacs Calculator -- a powerful symbolic algebra system  -*- lexical-binding: t; -*-
+;;; $Id$
 ;;; $Author: tv.raman.tv $ 
 ;;; Description: 
 ;;; Keywords:
@@ -15,7 +15,7 @@
 
 ;;}}}
 ;;{{{  Copyright:
-;;;Copyright (C) 1995 -- 2011, T. V. Raman 
+;;;Copyright (C) 1995 -- 2018, T. V. Raman 
 ;;; Copyright (c) 1994, 1995 by Digital Equipment Corporation.
 ;;; All Rights Reserved. 
 ;;;
@@ -38,27 +38,28 @@
 ;;}}}
 
 ;;{{{  Introduction:
-
+;;; Commentary:
 ;;; This module extends the Emacs Calculator.
 ;;; Extensions are minimal.
 ;;; We force a calc-load-everything,
 ;;; And use an after advice on this function
 ;;; To fix all of calc's interactive functions
-
+;;; Code:
 ;;}}}
 ;;{{{ required modules
+(cl-declaim  (optimize  (safety 0) (speed 3)))
 (require 'emacspeak-preamble)
 
 ;;}}}
 ;;{{{  advice calc interaction 
 (defadvice calc-dispatch (after emacspeak pre act comp)
   "Provide auditory feedback."
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'open-object)))
 
-(defadvice calc-quit (after emacspeak pre act )
+(defadvice calc-quit (after emacspeak pre act)
   "Announce the buffer that becomes current when calc is quit."
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'close-object)
     (emacspeak-speak-mode-line)))
 
@@ -68,24 +69,21 @@
 (defadvice calc-call-last-kbd-macro (around emacspeak pre act)
   "Provide spoken feedback."
   (cond
-   ((interactive-p)
-    (let ((emacspeak-speak-messages nil)
-          (dtk-quiet t)
-          (emacspeak-use-auditory-icons nil))
-      ad-do-it)
+   ((ems-interactive-p)
+    (ems-with-messages-silenced ad-do-it)
     (tts-with-punctuations 'all
                            (emacspeak-read-previous-line))
     (emacspeak-auditory-icon 'task-done))
    (t ad-do-it))
-  ad-return-value )
+  ad-return-value)
 
 (defadvice  calc-do (around emacspeak pre act comp)
   "Speak previous line of output."
-  (let ((emacspeak-speak-messages nil))
-    ad-do-it
-    (tts-with-punctuations 'all
-                           (emacspeak-read-previous-line))
-    (emacspeak-auditory-icon 'select-object))
+  (ems-with-messages-silenced ad-do-it)
+  (tts-with-punctuations
+   'all
+   (emacspeak-read-previous-line)
+   (emacspeak-auditory-icon 'select-object))
   ad-return-value)
 
 (defadvice  calc-trail-here (after emacspeak pre act comp)
