@@ -1,5 +1,5 @@
-;;; emacspeak-cedet.el --- Speech enable CEDET
-;;; $Id: emacspeak-cedet.el 6708 2011-01-04 02:27:29Z tv.raman.tv $
+;;; emacspeak-cedet.el --- Speech enable CEDET Development Environment -*- lexical-binding: t; -*-
+;;; $Id$
 ;;; $Author: tv.raman.tv $
 ;;; Description: Auditory interface to CEDET
 ;;; Keywords: Emacspeak, Speak, Spoken Output, CEDET
@@ -16,7 +16,7 @@
 ;;}}}
 ;;{{{  Copyright:
 
-;;; Copyright (c) 1995 -- 2011, T. V. Raman
+;;; Copyright (c) 1995 -- 2018, T. V. Raman
 ;;; All Rights Reserved.
 ;;;
 ;;; This file is not part of GNU Emacs, but the same permissions apply.
@@ -40,36 +40,38 @@
 
 ;;{{{  Required modules
 
+(cl-declaim  (optimize  (safety 0) (speed 3)))
 (require 'emacspeak-preamble)
 
 ;;}}}
 ;;{{{  Introduction
+;;; Commentary: 
 
 ;;; CEDET consists of speedbar semantic ede and friends.
 ;;; This module speech enables new functionality in semantic,
 ;;senator and friends
-
+;;; Code:
 ;;}}}
 ;;{{{ advice semantic completion
 
-(loop for f in
-      '(semantic-complete-complete-tab semantic-complete-complete-space)
-      do
-      (eval
-       `(defadvice ,f (around emacspeak pre act comp)
-          "Provide auditory feedback."
-          (let ((prior (point ))
-                (dtk-stop-immediately t))
-            (emacspeak-kill-buffer-carefully "*Completions*")
-            ad-do-it
-            (if (> (point) prior)
-                (tts-with-punctuations 'all
-                                       (emacspeak-speak-rest-of-buffer))
-              (emacspeak-speak-completions-if-available))
-            ad-return-value))))
+(cl-loop for f in
+         '(semantic-complete-symbol)
+         do
+         (eval
+          `(defadvice ,f (around emacspeak pre act comp)
+             "Provide auditory feedback."
+             (let ((prior (point))
+                   (dtk-stop-immediately t))
+               (emacspeak-kill-buffer-carefully "*Completions*")
+               ad-do-it
+               (if (> (point) prior)
+                   (tts-with-punctuations 'all
+                                          (emacspeak-speak-rest-of-buffer))
+                 (emacspeak-speak-completions-if-available))
+               ad-return-value))))
 
 ;;}}}
-(provide 'emacspeak-cedet )
+(provide 'emacspeak-cedet)
 ;;{{{ end of file
 
 ;;; local variables:
