@@ -1,4 +1,4 @@
-;;; emacspeak-compile.el --- Speech enable  navigation of  compile errors, grep matches
+;;; emacspeak-compile.el --- Speech enable  navigation of  compile errors, grep matches  -*- lexical-binding: t; -*-
 ;;; $Author: tv.raman.tv $ 
 ;;; Description:  Emacspeak extensions to  the compile package 
 ;;; Keywords: Emacspeak compile
@@ -14,7 +14,7 @@
 
 ;;}}}
 ;;{{{  Copyright:
-;;;Copyright (C) 1995 -- 2011, T. V. Raman 
+;;;Copyright (C) 1995 -- 2018, T. V. Raman 
 ;;; Copyright (c) 1994, 1995 by Digital Equipment Corporation.
 ;;; All Rights Reserved. 
 ;;;
@@ -47,6 +47,7 @@
 
 ;;}}}
 ;;{{{ Required modules
+(cl-declaim  (optimize  (safety 0) (speed 3)))
 (require 'emacspeak-preamble)
 
 ;;}}}
@@ -58,7 +59,9 @@
    (compilation-info voice-lighten)
    (compilation-error voice-animate-extra)
    (compilation-warning voice-animate)
-   ))
+   (compilation-mode-line-exit voice-animate)
+   (compilation-mode-line-fail voice-brighten)
+   (compilation-mode-line-run voice-annotate)))
 
 ;;}}}
 ;;{{{  functions
@@ -72,48 +75,49 @@
 
 ;;}}}
 ;;{{{  advice  interactive commands
-(loop for f in 
-      '(
-        next-error previous-error
-                   compilation-next-file compilation-previous-file
-                   compile-goto-error compile-mouse-goto-error
-                   )
-      do
-      (eval
-       `(defadvice ,f (after  emacspeak pre act )
-          "Speak the line containing the error. "
-          (when (ems-interactive-p )
-            (dtk-stop)
-            (emacspeak-compilation-speak-error)))))
+(cl-loop for f in 
+         '(
+           next-error previous-error
+           compilation-next-file compilation-previous-file
+           compile-goto-error compile-mouse-goto-error
+           )
+         do
+         (eval
+          `(defadvice ,f (after  emacspeak pre act)
+             "Speak the line containing the error. "
+             (when (ems-interactive-p)
+               (dtk-stop)
+               (emacspeak-auditory-icon 'large-movement)
+               (emacspeak-compilation-speak-error)))))
 
-(loop for f in 
-      '(
-        compilation-next-error
-        compilation-previous-error
-        next-error-no-select
-        previous-error-no-select)
-      do
-      (eval
-       `(defadvice ,f (after emacspeak pre act comp)
-          "Provide spoken feedback."
-          (when (ems-interactive-p )
-            (emacspeak-speak-line)
-            (emacspeak-auditory-icon 'select-object)))))
+(cl-loop for f in 
+         '(
+           compilation-next-error
+           compilation-previous-error
+           next-error-no-select
+           previous-error-no-select)
+         do
+         (eval
+          `(defadvice ,f (after emacspeak pre act comp)
+             "Provide spoken feedback."
+             (when (ems-interactive-p)
+               (emacspeak-speak-line)
+               (emacspeak-auditory-icon 'select-object)))))
 ;;}}}
 ;;{{{ advise process filter and sentinels
 
-(defadvice compile (after emacspeak pre act )
+(defadvice compile (after emacspeak pre act)
   "provide auditory confirmation"
-  (when (ems-interactive-p )
+  (when (ems-interactive-p)
     (message "Launched compilation")
-    (emacspeak-auditory-icon 'select-object)))
+    (emacspeak-auditory-icon 'task-done)))
 
-(defadvice  compilation-sentinel (after emacspeak pre act )
+(defadvice  compilation-sentinel (after emacspeak pre act)
   "Provide auditory feedback"
   (emacspeak-auditory-icon 'task-done)
   (message "process %s %s"
            (process-name  (ad-get-arg 0))
-           (ad-get-arg 1 )))
+           (ad-get-arg 1)))
 
 ;;}}}
 (provide 'emacspeak-compile)
@@ -122,7 +126,7 @@
 
 ;;; local variables:
 ;;; folded-file: t
-;;; byte-compile-dynamic: nil
+;;; byte-compile-dynamic: t
 ;;; end: 
 
 ;;}}}
