@@ -1,5 +1,5 @@
-;;; emacspeak-dired.el --- Speech enable Dired Mode -- A powerful File Manager
-;;; $Id: emacspeak-dired.el 8146 2013-02-09 20:05:08Z tv.raman.tv $
+;;; emacspeak-dired.el --- Speech enable Dired Mode -- A powerful File Manager  -*- lexical-binding: t; -*-
+;;; $Id$
 ;;; $Author: tv.raman.tv $
 ;;; Description:  Emacspeak extension to speech enable dired
 ;;; Keywords: Emacspeak, Dired, Spoken Output
@@ -15,7 +15,7 @@
 
 ;;}}}
 ;;{{{  Copyright:
-;;;Copyright (C) 1995 -- 2011, T. V. Raman
+;;;Copyright (C) 1995 -- 2018, T. V. Raman
 ;;; Copyright (c) 1994, 1995 by Digital Equipment Corporation.
 ;;; All Rights Reserved.
 ;;;
@@ -49,77 +49,51 @@
 ;;{{{  required packages
 
 ;;; Code:
+(require 'cl-lib)
+(cl-declaim  (optimize  (safety 0) (speed 3)))
 (require 'emacspeak-preamble)
 (require 'desktop)
 (require 'dired)
 ;;}}}
 ;;{{{ Define personalities
+
 (voice-setup-add-map
  '(
    (dired-header voice-smoothen)
    (dired-mark voice-lighten)
-   (dired-marked voice-bolden-and-animate)
+   (dired-perm-write voice-lighten-extra)
+   (dired-marked voice-lighten)
    (dired-warning voice-animate-extra)
    (dired-directory voice-bolden-medium)
    (dired-symlink voice-animate-extra)
    (dired-ignored voice-lighten-extra)
    (dired-flagged voice-animate-extra)
    ))
-;;}}}
-;;{{{  configure dired
-
-(declaim (special dired-listing-switches ))
-;;; ensure we have -al in the listing switches
-(if (and  dired-listing-switches
-          (not (string-match "^-al" dired-listing-switches)))
-    (setq dired-listing-switches
-          (concat "-al"
-                  (substring dired-listing-switches 1)))
-  (setq dired-listing-switches "-al"))
-
-(defvar emacspeak-dired-pronunciations-defined nil
-  "Internal variable used to ensure we define dired
-pronunciations only once.")
-
-(defun emacspeak-dired-define-pronunciations ()
-  "Define pronunciations specific to Dired buffers."
-  (declare (special emacspeak-dired-pronunciations-defined
-                    emacspeak-pronounce-pronunciation-table emacspeak-pronounce-dictionaries-loaded))
-  (unless emacspeak-dired-pronunciations-defined
-    (setq emacspeak-dired-pronunciations-defined t)
-    (emacspeak-pronounce-add-dictionary-entry 'dired-mode "Dired" " DirEd  "))
-  (when (or (not (boundp 'emacspeak-pronounce-pronunciation-table))
-            (not emacspeak-pronounce-pronunciation-table))
-    (emacspeak-pronounce-toggle-use-of-dictionaries)))
 
 ;;}}}
 ;;{{{  functions:
 
 (defun emacspeak-dired-speak-line ()
   "Speak the dired line intelligently."
-  (declare (special emacspeak-speak-last-spoken-word-position))
-  (let ((filename (dired-get-filename t t ))
-        (personality (get-text-property (point) 'personality)))
+  (cl-declare (special emacspeak-speak-last-spoken-word-position))
+  (let ((filename (dired-get-filename 'no-dir  t))
+        (personality (dtk-get-style)))
     (cond
      (filename
-      (put-text-property  0  (length filename)
-                          'personality personality filename )
-      (dtk-speak filename)
+      (dtk-speak (propertize filename 'personality personality))
       (setq emacspeak-speak-last-spoken-word-position (point)))
-     (t (emacspeak-speak-line )))))
+     (t (emacspeak-speak-line)))))
 
 ;;}}}
 ;;{{{  advice:
 
-(defadvice dired-sort-toggle-or-edit (around emacspeak pre
-                                             act comp)
+(defadvice dired-sort-toggle-or-edit (around emacspeak pre act comp)
   "Provide auditory feedback."
   (cond
-   ((ems-interactive-p )
-    (let ((emacspeak-speak-messages nil))
-      ad-do-it
-      (emacspeak-auditory-icon 'task-done)
-      (emacspeak-speak-mode-line)))
+   ((ems-interactive-p)
+    (ems-with-messages-silenced ad-do-it)
+    (emacspeak-auditory-icon 'task-done)
+    (emacspeak-speak-mode-line))
    (t ad-do-it))
   ad-return-value)
 
@@ -129,103 +103,67 @@ pronunciations only once.")
 
 (defadvice dired-quit (after emacspeak pre act comp)
   "Provide auditory feedback."
-  (when (ems-interactive-p )
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'close-object)
     (emacspeak-speak-mode-line)))
 
-(defadvice dired-up-directory (after emacspeak pre act)
-  "Produce an auditory icon."
-  (when (ems-interactive-p )
-    (let ((emacspeak-speak-messages nil))
-      (emacspeak-dired-label-fields)
-      (emacspeak-auditory-icon 'open-object )
-      (emacspeak-speak-mode-line))))
 (defun emacspeak-dired-initialize ()
   "Set up emacspeak dired."
   (emacspeak-dired-label-fields)
-  (emacspeak-auditory-icon 'open-object )
-  (emacspeak-speak-mode-line))
-
-(defadvice dired (after emacspeak pre act comp)
-  "Hook is not reliable."
-  (emacspeak-dired-initialize))
+  (emacspeak-dired-setup-keys))
+(cl-loop
+ for  f in
+ '(dired ido-dired
+         dired-other-window dired-other-frame)
+ do
+ (eval
+  `(defadvice ,f (after emacspeak pre act comp)
+     "Set up emacspeak."
+     (when (ems-interactive-p)
+       (emacspeak-dired-initialize)
+       (emacspeak-auditory-icon 'open-object)
+       (emacspeak-speak-mode-line)))))
 
 (defadvice dired-find-file  (around  emacspeak pre act)
   "Produce an auditory icon."
   (cond
-   ((ems-interactive-p )
-    (let ((directory-p (file-directory-p (dired-get-filename t t ))))
+   ((ems-interactive-p)
+    (let ((directory-p (file-directory-p (dired-get-filename t t))))
       ad-do-it
-      (when directory-p
-        (emacspeak-dired-label-fields))
-      (emacspeak-auditory-icon 'open-object )))
+      (when directory-p (emacspeak-dired-label-fields))
+      (emacspeak-speak-mode-line)
+      (emacspeak-auditory-icon 'open-object)))
    (t ad-do-it))
   ad-return-value)
 
-(defadvice dired-tree-up (after emacspeak pre act)
-  "Speak the filename."
-  (when (ems-interactive-p  )
-    (emacspeak-auditory-icon 'large-movement)
-    (emacspeak-dired-speak-line)))
+(cl-loop
+ for  f in
+ '(
+   dired-next-subdir dired-prev-subdir
+   dired-tree-up dired-tree-down dired-up-directory
+   dired-next-marked-file dired-prev-marked-file
+   dired-next-dirline dired-prev-dirline
+   dired-jump
+   )
+ do
+ (eval
+  `(defadvice ,f (after emacspeak pre act)
+     "Speak the filename."
+     (when (ems-interactive-p)
+       (emacspeak-auditory-icon 'large-movement)
+       (emacspeak-dired-speak-line)))))
 
-(defadvice dired-tree-down (after emacspeak pre act)
-  "Speak the filename."
-  (when (ems-interactive-p  )
-    (emacspeak-auditory-icon 'large-movement)
-    (emacspeak-dired-speak-line)))
-
-(defadvice dired-next-line (after emacspeak pre act)
-  "Speak the filename name."
-  (when (ems-interactive-p  )
-    (emacspeak-auditory-icon 'select-object)
-    (emacspeak-dired-speak-line)))
-
-(defadvice dired-previous-line (after emacspeak pre act)
-  "Speak the filename name."
-  (when (ems-interactive-p  )
-    (emacspeak-auditory-icon 'select-object)
-    (emacspeak-dired-speak-line)))
-
-(defadvice dired-next-marked-file (after emacspeak pre act)
-  "Speak the filename name."
-  (when (ems-interactive-p  )
-    (emacspeak-auditory-icon 'large-movement)
-    (emacspeak-dired-speak-line)))
-
-(defadvice dired-prev-marked-file  (after emacspeak pre act)
-  "Speak the filename name."
-  (when (ems-interactive-p  )
-    (emacspeak-auditory-icon 'large-movement)
-    (emacspeak-dired-speak-line)))
-
-(defadvice dired-prev-subdir (after emacspeak pre act)
-  "Speak the filename name."
-  (when (ems-interactive-p  )
-    (emacspeak-auditory-icon 'large-movement)
-    (emacspeak-dired-speak-line)))
-
-(defadvice dired-next-subdir (after emacspeak pre act)
-  "Speak the filename name."
-  (when (ems-interactive-p  )
-    (emacspeak-auditory-icon 'large-movement)
-    (emacspeak-dired-speak-line)))
-
-(defadvice dired-next-dirline (after emacspeak pre act)
-  "Speak the filename name."
-  (when (ems-interactive-p  )
-    (emacspeak-auditory-icon 'large-movement)
-    (emacspeak-dired-speak-line)))
-
-(defadvice dired-prev-dirline (after emacspeak pre act)
-  "Speak the filename name."
-  (when (ems-interactive-p  )
-    (emacspeak-auditory-icon 'large-movement)
-    (emacspeak-dired-speak-line)))
-
-(defadvice dired-unmark-backward (after emacspeak pre act)
-  "Speak the filename name."
-  (when (ems-interactive-p  )
-    (emacspeak-dired-speak-line)))
+(cl-loop
+ for f in
+ '(dired-next-line dired-previous-line
+                   dired-unmark-backward dired-maybe-insert-subdir)
+ do
+ (eval
+  `(defadvice ,f  (after emacspeak pre act)
+     "Speak the filename name."
+     (when (ems-interactive-p)
+       (emacspeak-auditory-icon 'select-object)
+       (emacspeak-dired-speak-line)))))
 
 ;;; Producing auditory icons:
 ;;; These dired commands do some action that causes a state change:
@@ -237,20 +175,20 @@ pronunciations only once.")
 
 (defadvice dired-mark (after emacspeak pre act)
   "Produce an auditory icon."
-  (when (ems-interactive-p )
-    (emacspeak-auditory-icon 'mark-object )
+  (when (ems-interactive-p)
+    (emacspeak-auditory-icon 'mark-object)
     (emacspeak-dired-speak-line)))
 
-(defadvice dired-flag-file-deletion (after emacspeak pre act )
+(defadvice dired-flag-file-deletion (after emacspeak pre act)
   "Produce an auditory icon indicating that a file was marked for deletion."
-  (when (ems-interactive-p  )
-    (emacspeak-auditory-icon 'delete-object )
-    (emacspeak-dired-speak-line )))
+  (when (ems-interactive-p)
+    (emacspeak-auditory-icon 'delete-object)
+    (emacspeak-dired-speak-line)))
 
 (defadvice dired-unmark (after emacspeak pre act)
   "Give speech feedback. Also provide an auditory icon."
-  (when (ems-interactive-p )
-    (emacspeak-auditory-icon 'deselect-object )
+  (when (ems-interactive-p)
+    (emacspeak-auditory-icon 'deselect-object)
     (emacspeak-dired-speak-line)))
 
 ;;}}}
@@ -258,7 +196,7 @@ pronunciations only once.")
 
 (defun emacspeak-dired-label-fields-on-current-line ()
   "Labels the fields on a dired line.
-Assumes that `dired-listing-switches' contains  -al"
+Assumes that `dired-listing-switches' contains  -l"
   (let ((start nil)
         (fields (list "permissions"
                       "links"
@@ -270,25 +208,26 @@ Assumes that `dired-listing-switches' contains  -al"
                       "modified at"
                       "name")))
     (save-excursion
-      (beginning-of-line)
+      (forward-line 0)
       (skip-syntax-forward " ")
       (while (and fields
                   (not (eolp)))
         (setq start (point))
         (skip-syntax-forward "^ ")
         (put-text-property start (point)
-                           'field-name (car fields ))
-        (setq fields (cdr fields ))
+                           'field-name (car fields))
+        (setq fields (cdr fields))
         (skip-syntax-forward " ")))))
 
 (defun emacspeak-dired-label-fields ()
   "Labels the fields of the listing in the dired buffer.
 Currently is a no-op  unless
-unless `dired-listing-switches' contains -al"
+unless `dired-listing-switches' contains -l"
   (interactive)
-  (declare (special dired-listing-switches))
-  (when (save-match-data
-          (string-match  "al" dired-listing-switches))
+  (cl-declare (special dired-listing-switches))
+  (when
+      (save-match-data
+        (string-match  "l" dired-listing-switches))
     (let ((read-only buffer-read-only))
       (unwind-protect
           (progn
@@ -297,12 +236,13 @@ unless `dired-listing-switches' contains -al"
               (goto-char (point-min))
               (dired-goto-next-nontrivial-file)
               (while (not (eobp))
-                (emacspeak-dired-label-fields-on-current-line )
-                (forward-line 1 ))))
-        (setq buffer-read-only read-only )))))
+                (emacspeak-dired-label-fields-on-current-line)
+                (forward-line 1))))
+        (setq buffer-read-only read-only)))))
 
 ;;}}}
 ;;{{{ Additional status speaking commands
+
 (defcustom emacspeak-dired-file-cmd-options "-b"
   "Options passed to Unix builtin `file' command."
   :type '(choice
@@ -315,7 +255,7 @@ unless `dired-listing-switches' contains -al"
 Like Emacs' built-in dired-show-file-type but allows user to customize
 options passed to command `file'."
   (interactive (list (dired-get-filename t) current-prefix-arg))
-  (declare (special emacspeak-dired-file-cmd-options))
+  (cl-declare (special emacspeak-dired-file-cmd-options))
   (with-temp-buffer
     (if deref-symlinks
         (call-process "file" nil t t  "-l"
@@ -329,7 +269,7 @@ options passed to command `file'."
 (defun emacspeak-dired-speak-header-line()
   "Speak the header line of the dired buffer. "
   (interactive)
-  (emacspeak-auditory-icon 'select-object)
+  (emacspeak-auditory-icon 'section)
   (save-excursion (goto-char (point-min))
                   (forward-line 2)
                   (emacspeak-speak-region (point-min) (point))))
@@ -344,15 +284,14 @@ On a directory line, run du -s on the directory to speak its size."
      ((and filename
            (file-directory-p filename))
       (emacspeak-auditory-icon 'progress)
-      (emacspeak-shell-command (format "du -s \'%s\'"
-                                       filename )))
+      (emacspeak-shell-command (format "du -s \"%s\"" filename)))
      (filename
-      (setq size (nth 7 (file-attributes filename )))
+      (setq size (nth 7 (file-attributes filename)))
                                         ; check for ange-ftp
       (when (= size -1)
         (setq size
               (nth  4
-                    (split-string (thing-at-point 'line)))))
+                    (split-string (ems-this-line)))))
       (emacspeak-auditory-icon 'select-object)
       (message "File size %s"
                size))
@@ -368,7 +307,7 @@ On a directory line, run du -s on the directory to speak its size."
       (message "Modified on : %s"
                (format-time-string
                 emacspeak-speak-time-format-string
-                (nth 5 (file-attributes filename )))))
+                (nth 5 (file-attributes filename)))))
      (t (message "No file on current line")))))
 
 (defun emacspeak-dired-speak-file-access-time ()
@@ -381,7 +320,7 @@ On a directory line, run du -s on the directory to speak its size."
       (message "Last accessed   on  %s"
                (format-time-string
                 emacspeak-speak-time-format-string
-                (nth 4 (file-attributes filename )))))
+                (nth 4 (file-attributes filename)))))
      (t (message "No file on current line")))))
 (defun emacspeak-dired-speak-symlink-target ()
   "Speaks the target of the symlink on the current line."
@@ -404,35 +343,169 @@ On a directory line, run du -s on the directory to speak its size."
      (filename
       (emacspeak-auditory-icon 'select-object)
       (message "Permissions %s"
-               (nth 8 (file-attributes filename ))))
+               (nth 8 (file-attributes filename))))
      (t (message "No file on current line")))))
 
 ;;}}}
 ;;{{{  keys
-(eval-when (load))
+(cl-eval-when (load))
 
 (defun emacspeak-dired-setup-keys ()
   "Add emacspeak keys to dired."
-  (declare (special dired-mode-map ))
+  (cl-declare (special dired-mode-map))
+  (define-key dired-mode-map "F" 'emacspeak-wizards-find-file-as-root)
+  (define-key dired-mode-map "E" 'emacspeak-dired-epub-eww)
+  (define-key dired-mode-map (kbd "C-j") 'emacspeak-dired-open-this-file)
+  (define-key dired-mode-map (kbd "C-RET") 'emacspeak-dired-open-this-file)
+  (define-key dired-mode-map [C-return] 'emacspeak-dired-open-this-file)
   (define-key dired-mode-map "'" 'emacspeak-dired-show-file-type)
   (define-key  dired-mode-map "/" 'emacspeak-dired-speak-file-permissions)
   (define-key  dired-mode-map ";" 'emacspeak-dired-speak-header-line)
   (define-key  dired-mode-map "a" 'emacspeak-dired-speak-file-access-time)
   (define-key dired-mode-map "c" 'emacspeak-dired-speak-file-modification-time)
   (define-key dired-mode-map "z" 'emacspeak-dired-speak-file-size)
-  (define-key dired-mode-map "\C-t" 'emacspeak-dired-speak-symlink-target)
+  (define-key dired-mode-map "\M-t" 'emacspeak-dired-speak-symlink-target)
   (define-key dired-mode-map "\C-i" 'emacspeak-speak-next-field)
   (define-key dired-mode-map  "," 'emacspeak-speak-previous-field))
-(add-hook 'dired-mode-hook 'emacspeak-dired-initialize 'append)
-(add-hook 'dired-mode-hook 'emacspeak-dired-setup-keys)
-(add-hook 'dired-mode-hook 'emacspeak-dired-define-pronunciations)
+(add-hook 'dired-mode-hook  'emacspeak-dired-initialize 'append)
+
+;;}}}
+;;{{{ Advice locate:
+(cl-loop
+ for f in
+ '(locate locate-with-filter)
+ do
+ (eval
+  `(defadvice ,f (after emacspeak pre act comp)
+     "Provide auditory feedback."
+     (when (ems-interactive-p)
+       (emacspeak-speak-line)
+       (emacspeak-auditory-icon 'open-object)))))
+(load "locate" t t)
+
+(cl-declaim (special locate-mode-map))
+(define-key locate-mode-map  [C-return] 'emacspeak-dired-open-this-file)
+;;}}}
+;;{{{ Context-sensitive openers:
+
+(defun emacspeak-dired-play-this-media ()
+  "Plays media on current line."
+  (emacspeak-m-player (dired-get-filename)))
+
+(defun emacspeak-dired-play-this-playlist ()
+  "Plays playlist on current line."
+  (emacspeak-m-player (dired-get-filename) 'playlist))
+
+(defconst emacspeak-dired-opener-table
+  `(("\\.epub$"  emacspeak-dired-epub-eww)
+    ("\\.xhtml" emacspeak-dired-eww-open)
+    ("\\.html" emacspeak-dired-eww-open)
+    ("\\.htm" emacspeak-dired-eww-open)
+    ("\\.pdf" emacspeak-dired-pdf-open)
+    ("\\.md" emacspeak-dired-md-open)
+    ("\\.csv" emacspeak-dired-csv-open)
+    (,emacspeak-media-extensions emacspeak-dired-play-this-media)
+    (,emacspeak-m-player-playlist-pattern emacspeak-dired-play-this-playlist))
+  "Association of filename extension patterns to Emacspeak handlers.")
+
+(defun emacspeak-dired-open-this-file  ()
+  "Smart dired opener. Invokes appropriate Emacspeak handler on  current file in DirEd."
+  (interactive)
+  (let* ((f (dired-get-filename nil t))
+         (ext (file-name-extension f))
+         (handler nil))
+    (unless f (error "No file here."))
+    (unless ext (error "This entry has no extension."))
+    (setq handler
+          (cl-second
+           (cl-find
+            (format ".%s" ext)
+            emacspeak-dired-opener-table
+            :key #'car                  ; extract pattern from entry 
+            :test #'(lambda (e pattern) (string-match  pattern e)))))
+    (cond
+     ((and handler (fboundp handler))
+      (emacspeak-auditory-icon 'task-done)
+      (funcall handler))
+     (t (call-interactively #'dired-find-file)))))
+
+(defun emacspeak-dired-eww-open ()
+  "Open HTML file on current dired line."
+  (interactive)
+  (eww-open-file (dired-get-filename)))
+(declare-function markdown-preview "markdown-mode" (&optional output))
+(defun emacspeak-dired-md-open ()
+  "Preview markdown  file on current dired line."
+  (interactive)
+  (let ((buffer (find-file-noselect  (dired-get-filename))))
+    (with-current-buffer buffer
+      (markdown-preview))))
+
+(defun emacspeak-dired-pdf-open ()
+  "Open PDF file on current dired line."
+  (interactive)
+  (emacspeak-wizards-pdf-open (dired-get-filename current-prefix-arg)))
+
+(defun emacspeak-dired-epub-eww ()
+  "Open epub on current line  in EWW"
+  (interactive)
+  (emacspeak-epub-eww (shell-quote-argument(dired-get-filename)))
+  (emacspeak-auditory-icon 'open-object))
+
+(defun emacspeak-dired-csv-open ()
+  "Open CSV file on current dired line."
+  (interactive)
+  (emacspeak-table-find-csv-file (dired-get-filename current-prefix-arg)))
+
+;;}}}
+;;{{{ Locate results as a play-list:
+
+;;;###autoload
+(defun emacspeak-locate-play-results-as-playlist (&optional shuffle)
+  "Treat locate results as a play-list.
+Optional interactive prefix arg shuffles playlist."
+  (interactive "P")
+  (cl-declare (special emacspeak-m-player-options))
+  (cl-assert (eq major-mode 'locate-mode) t "Not in a locate buffer")
+  (save-excursion
+    (goto-char (point-min))
+    (dired-next-line 3)
+    (let* ((m3u (make-temp-file "locate-playlist" nil ".m3u"))
+           (buff (find-file-noselect m3u))
+           (results nil)
+           (file (dired-file-name-at-point)))
+      (while file
+        (push file results)
+        (dired-next-line 1)
+        (setq file  (dired-file-name-at-point)))
+      (setq results (nreverse results))
+      (message "%s tracks matching " (length results))
+      (with-current-buffer buff
+        (cl-loop
+         for f in results do
+         (insert (format "%s\n" (expand-file-name f))))
+        (save-buffer))
+      (let ((emacspeak-m-player-options
+             (if shuffle
+                 (append emacspeak-m-player-options (list "-shuffle"))
+               emacspeak-m-player-options)))
+        (emacspeak-m-player  m3u 'play-list)))))
+
+;;}}}
+;;{{{ Open Downloads:
+;;;###autoload
+(defun emacspeak-dired-downloads ()
+  "Open Downloads directory."
+  (interactive)
+  (funcall-interactively 'dired (expand-file-name "~/Downloads") "-alt"))
+
 ;;}}}
 (provide 'emacspeak-dired)
 ;;{{{ emacs local variables
 
 ;;; local variables:
 ;;; folded-file: t
-;;; byte-compile-dynamic: nil
+;;; byte-compile-dynamic: t
 ;;; end:
 
 ;;}}}
