@@ -1,5 +1,5 @@
-;;; emacspeak-ediary.el --- Speech-enable ediary
-;;; $Id: emacspeak-ediary.el 7378 2011-11-10 17:34:32Z tv.raman.tv $
+;;; emacspeak-ediary.el --- Speech-enable ediary  -*- lexical-binding: t; -*-
+;;; $Id$
 ;;; $Author: tv.raman.tv $
 ;;; Description:  Emacspeak speech-enabler for ediary 
 ;;; Keywords: Emacspeak, diary
@@ -16,7 +16,7 @@
 ;;}}}
 ;;{{{  Copyright:
 
-;;; Copyright (C) 1999, 2011 T. V. Raman <raman@cs.cornell.edu>
+;;; Copyright (C) 1995 -- 2018, T. V. Raman
 ;;; All Rights Reserved.
 ;;;
 ;;; This file is not part of GNU Emacs, but the same permissions apply.
@@ -39,15 +39,16 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 ;;{{{  Introduction:
-
-;;; ediary is a special mode for editting your.diary file.
+;;; Commentary:
+;;; ediary is a special mode for editing your.diary file.
 ;;; This module speech-enables ediary
-
+;;; Code:
 ;;}}}
 ;;{{{ required modules
 
 ;;; Code:
 
+(cl-declaim  (optimize  (safety 0) (speed 3)))
 (require 'emacspeak-preamble)
 
 ;;}}}
@@ -56,22 +57,22 @@
 (defvar emacspeak-ediary-commands-that-speak-entry
   '(
     ediary-entry-earlier      ediary-modify-entry
-                              ediary-time-block-longer        ediary-previous-entry       ediary-boe
-                              ediary-point-on-entry-date-p   ediary-next-entry   ediary-block-longer
-                              ediary-time-block-earlier     ediary-eoe     ediary-time-block-shorter
-                              ediary-time-earlier        ediary-block-later        ediary-kill-entry
-                              ediary-block-earlier      ediary-entry-later      ediary-block-shorter
-                              ediary-time-later     ediary-time-block-later)
+    ediary-time-block-longer        ediary-previous-entry       ediary-boe
+    ediary-point-on-entry-date-p   ediary-next-entry   ediary-block-longer
+    ediary-time-block-earlier     ediary-eoe     ediary-time-block-shorter
+    ediary-time-earlier        ediary-block-later        ediary-kill-entry
+    ediary-block-earlier      ediary-entry-later      ediary-block-shorter
+    ediary-time-later     ediary-time-block-later)
   "Commands that should speak the entry when done.")
 
-(loop for f in emacspeak-ediary-commands-that-speak-entry
-      do
-      (eval
-       `(defadvice ,f (after emacspeak pre act comp)
-          "Speak the entry."
-          (when (interactive-p)
-            (emacspeak-speak-line)
-            (emacspeak-auditory-icon 'select-object)))))
+(cl-loop for f in emacspeak-ediary-commands-that-speak-entry
+         do
+         (eval
+          `(defadvice ,f (after emacspeak pre act comp)
+             "Speak the entry."
+             (when (ems-interactive-p)
+               (emacspeak-speak-line)
+               (emacspeak-auditory-icon 'select-object)))))
 
 ;;}}}
 (provide 'emacspeak-ediary)
