@@ -1,5 +1,5 @@
-;;; emacspeak-etable.el --- Speech enable table.el
-;;; $Id: emacspeak-etable.el 6862 2011-02-18 16:19:36Z tv.raman.tv $
+;;; emacspeak-etable.el --- Speech enable table.el  -*- lexical-binding: t; -*-
+;;; $Id$
 ;;; $Author: tv.raman.tv $ 
 ;;; DescriptionEmacspeak extensions for table.el
 ;;; Keywords:emacspeak, audio interface to emacs Tables
@@ -15,7 +15,7 @@
 
 ;;}}}
 ;;{{{  Copyright:
-;;;Copyright (C) 1995 -- 2011, T. V. Raman 
+;;;Copyright (C) 1995 -- 2018, T. V. Raman 
 ;;; Copyright (c) 1994, 1995 by Digital Equipment Corporation.
 ;;; All Rights Reserved. 
 ;;;
@@ -38,13 +38,14 @@
 ;;}}}
 
 ;;{{{  Introduction:
-
-;;; table.el provides rich table editting for emacs.
+;;; Commentary:
+;;; table.el provides rich table editing for emacs.
 ;;; this module speech-enables table.el
-
+;;; Code:
 ;;}}}
 ;;{{{ required modules 
 
+(cl-declaim  (optimize  (safety 0) (speed 3)))
 (require 'emacspeak-preamble)
 (eval-when-compile
   (condition-case nil
@@ -55,18 +56,18 @@
 ;;{{{ Update command remap list.
 (defadvice table--make-cell-map(after emacspeak pre act comp)
   "Set up emacspeak for table.el"
-  (declare (special table-cell-map))
+  (cl-declare (special table-cell-map))
   (when  table-cell-map
-    (loop for k in
-          (where-is-internal 'emacspeak-self-insert-command (list table-cell-map))
-          do
-          (define-key table-cell-map k '*table--cell-self-insert-command ))
-    (loop for k in
-          '(
-            ([(shift tab)] table-backward-cell)
-            ("\C-e." emacspeak-etable-speak-cell))
-          do
-          (emacspeak-keymap-update table-cell-map k))))
+    (cl-loop for k in
+             (where-is-internal 'emacspeak-self-insert-command (list table-cell-map))
+             do
+             (define-key table-cell-map k '*table--cell-self-insert-command))
+    (cl-loop for k in
+             '(
+               ("S-TAB" table-backward-cell)
+               ("\C-e." emacspeak-etable-speak-cell))
+             do
+             (emacspeak-keymap-update table-cell-map k))))
 
 ;;}}}
 ;;{{{ Advice edit commands
@@ -74,8 +75,8 @@
 (defadvice *table--cell-delete-char (around emacspeak pre act)
   "Speak character you're deleting."
   (cond
-   ((interactive-p )
-    (dtk-tone 500 30 'force)
+   ((ems-interactive-p)
+    (dtk-tone 500 100 'force)
     (emacspeak-speak-char t)
     ad-do-it)
    (t ad-do-it))
@@ -84,16 +85,16 @@
 (defadvice *table--cell-delete-backward-char (around emacspeak pre act)
   "Speak character you're deleting."
   (cond
-   ((interactive-p )
-    (dtk-tone 500 30 'force)
-    (emacspeak-speak-this-char (preceding-char ))
+   ((ems-interactive-p)
+    (dtk-tone 500 100 'force)
+    (emacspeak-speak-this-char (preceding-char))
     ad-do-it)
    (t ad-do-it))
   ad-return-value)
 
 (defadvice *table--cell-self-insert-command (after emacspeak pre act comp)
   "Provide spoken output."
-  (when  (interactive-p)
+  (when  (ems-interactive-p)
     (cond
      ((and (= 32 last-input-event)
            emacspeak-word-echo)
@@ -104,36 +105,36 @@
           (emacspeak-speak-region orig (point)))))
      (emacspeak-character-echo
       (dtk-stop)
-      (emacspeak-speak-this-char last-input-event )))))
+      (emacspeak-speak-this-char last-input-event)))))
 
-(defadvice *table--cell-quoted-insert  (after emacspeak pre act )
+(defadvice *table--cell-quoted-insert  (after emacspeak pre act)
   "Speak the character that was inserted."
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (table--finish-delayed-tasks)
-    (emacspeak-speak-this-char (preceding-char ))))
+    (emacspeak-speak-this-char (preceding-char))))
 
 (defadvice *table--cell-newline (before emacspeak pre act)
   "Speak the previous line if line echo is on.
 See command \\[emacspeak-toggle-line-echo].  Otherwise cue the user to
 the newly created blank line."
-  (declare (special emacspeak-line-echo ))
-  (when (interactive-p)
+  (cl-declare (special emacspeak-line-echo))
+  (when (ems-interactive-p)
     (table--finish-delayed-tasks)
     (cond
-     (emacspeak-line-echo (emacspeak-speak-line ))
+     (emacspeak-line-echo (emacspeak-speak-line))
      (t(if dtk-stop-immediately (dtk-stop))
-       (dtk-tone 225 120 'force   )))))
+       (dtk-tone 225 120 'force)))))
 
 (defadvice *table--cell-newline-and-indent (around emacspeak pre act)
   "Speak the previous line if line echo is on.
 See command \\[emacspeak-toggle-line-echo].
 Otherwise cue user to the line just created."
-  (declare (special emacspeak-line-echo ))
+  (cl-declare (special emacspeak-line-echo))
   (cond
-   ((interactive-p)
+   ((ems-interactive-p)
     (cond
      (emacspeak-line-echo
-      (emacspeak-speak-line )
+      (emacspeak-speak-line)
       ad-do-it)
      (t ad-do-it
         (dtk-speak-using-voice voice-annotate
@@ -144,14 +145,14 @@ Otherwise cue user to the line just created."
    (t ad-do-it))
   ad-return-value)
 
-(defadvice *table--cell-open-line (after emacspeak pre act )
+(defadvice *table--cell-open-line (after emacspeak pre act)
   "Provide auditory feedback."
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (let ((count (ad-get-arg 0)))
       (emacspeak-auditory-icon 'open-object)
       (message "Opened %s blank line%s"
                (if (= count 1) "a" count)
-               (if (= count 1 ) "" "s")))))
+               (if (= count 1) "" "s")))))
 
 ;;}}}
 
@@ -168,17 +169,17 @@ Otherwise cue user to the line just created."
        (cdr cell)))
      (t (error "Cant identify cell.")))))
 
-(loop for f in
-      '(table-forward-cell table-backward-cell)
-      do
-      (eval
-       `(defadvice ,f (after emacspeak pre act comp)
-          "Provide auditory feedback by speaking current cell
+(cl-loop for f in
+         '(table-forward-cell table-backward-cell)
+         do
+         (eval
+          `(defadvice ,f (after emacspeak pre act comp)
+             "Provide auditory feedback by speaking current cell
       contents."
-          (when (interactive-p)
-            (table--finish-delayed-tasks)
-            (emacspeak-auditory-icon 'select-object)
-            (emacspeak-etable-speak-cell)))))
+             (when (ems-interactive-p)
+               (table--finish-delayed-tasks)
+               (emacspeak-auditory-icon 'select-object)
+               (emacspeak-etable-speak-cell)))))
 
 ;;}}}
 (provide  'emacspeak-etable)
