@@ -1,5 +1,5 @@
-;;; emacspeak-outline.el --- Speech enable Outline --   Browsing  Structured Documents
-;;; $Id: emacspeak-outline.el 7277 2011-10-14 21:26:35Z tv.raman.tv $
+;;; emacspeak-outline.el --- Speech enable Outline --   Browsing  Structured Documents  -*- lexical-binding: t; -*-
+;;; $Id$
 ;;; $Author: tv.raman.tv $
 ;;; DescriptionEmacspeak extensions for outline-mode
 ;;; Keywords:emacspeak, audio interface to emacs Outlines
@@ -15,7 +15,7 @@
 
 ;;}}}
 ;;{{{  Copyright:
-;;;Copyright (C) 1995 -- 2011, T. V. Raman
+;;;Copyright (C) 1995 -- 2018, T. V. Raman
 ;;; Copyright (c) 1994, 1995 by Digital Equipment Corporation.
 ;;; All Rights Reserved.
 ;;;
@@ -47,6 +47,7 @@
 
 ;;}}}
 ;;{{{ requires
+(cl-declaim  (optimize  (safety 0) (speed 3)))
 (require 'emacspeak-preamble)
 (require 'outline)
 
@@ -55,115 +56,115 @@
 
 (defadvice outline-next-heading (after emacspeak pre act comp)
   "Speak the line."
-  (when (interactive-p)
-    (emacspeak-auditory-icon 'large-movement)
-    (emacspeak-speak-line )))
+  (when (ems-interactive-p)
+    (emacspeak-auditory-icon 'section)
+    (emacspeak-speak-line)))
 
 (defadvice outline-back-to-heading (after emacspeak pre act comp)
   "Speak the line."
-  (when (interactive-p)
-    (emacspeak-auditory-icon 'large-movement)
-    (emacspeak-speak-line )))
+  (when (ems-interactive-p)
+    (emacspeak-auditory-icon 'section)
+    (emacspeak-speak-line)))
 
 (defadvice outline-next-visible-heading (after emacspeak pre act comp)
   "Speak the line."
-  (when (interactive-p)
-    (emacspeak-auditory-icon 'large-movement)
+  (when (ems-interactive-p)
+    (emacspeak-auditory-icon 'section)
     (and (looking-at "^$")
          (skip-syntax-backward " "))
-    (emacspeak-speak-line )))
+    (emacspeak-speak-line)))
 
 (defadvice outline-previous-visible-heading (after emacspeak pre act comp)
   "Speak the line."
-  (when (interactive-p)
-    (emacspeak-auditory-icon 'large-movement)
-    (emacspeak-speak-line )))
+  (when (ems-interactive-p)
+    (emacspeak-auditory-icon 'section)
+    (emacspeak-speak-line)))
 
 (defadvice outline-up-heading (after emacspeak pre act comp)
   "Provide auditory feedback."
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'large-movement)
-    (emacspeak-speak-line )))
+    (emacspeak-speak-line)))
 
 (defadvice outline-forward-same-level (after emacspeak pre act comp)
   "Provide auditory feedback."
-  (when (interactive-p)
-    (emacspeak-auditory-icon 'large-movement)
-    (emacspeak-speak-line )))
+  (when (ems-interactive-p)
+    (emacspeak-auditory-icon 'section)
+    (emacspeak-speak-line)))
 
 (defadvice outline-backward-same-level (after emacspeak pre act comp)
   "Provide auditory feedback."
-  (when (interactive-p)
-    (emacspeak-auditory-icon 'large-movement)
-    (emacspeak-speak-line )))
+  (when (ems-interactive-p)
+    (emacspeak-auditory-icon 'section)
+    (emacspeak-speak-line)))
 
 ;;}}}
 ;;{{{  Hiding and showing subtrees
 
 (defadvice hide-entry (after emacspeak pre act comp)
   "Produce an auditory icon"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'close-object)
     (message "Hid the body directly following this heading")))
 
 (defadvice show-entry (after emacspeak pre act comp)
   "Produce an auditory icon"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'open-object)
     (message "Exposed body directly following current heading")))
 
 (defadvice hide-body (after emacspeak pre act comp)
   "Produce an auditory icon"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'close-object)
     (message "Hid all of the buffer except for header lines")))
 
 (defadvice show-all (after emacspeak pre act comp)
   "Produce an auditory icon"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'open-object)
     (message "Exposed all text in the buffer")))
 
 (defadvice hide-subtree (after emacspeak pre act comp)
   "Produce an auditory icon"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'close-object)
     (message "Hid everything at deeper levels from current heading")))
 
 (defadvice hide-leaves (after emacspeak pre act comp)
   "Produce an auditory icon"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'close-object)
     (message "Hid all of the body at deeper levels")))
 
 (defadvice show-subtree  (after emacspeak pre act comp)
   "Produce an auditory icon"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'open-object)
     (message "Exposed everything after current heading at deeper levels")))
 
 (defadvice hide-sublevels (after emacspeak pre act comp)
   "Produce an auditory icon"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'close-object)
     (message "Hid everything except the top  %s levels"
              (ad-get-arg 0))))
 
 (defadvice hide-other (after emacspeak pre act comp)
   "Produce an auditory icon"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'close-object)
     (message "Hid everything except current body and parent headings")))
 
 (defadvice show-branches (after emacspeak pre act comp)
   "Produce an auditory icon"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'open-object)
     (message "Exposed all subheadings while leaving their bodies hidden")))
 
 (defadvice show-children (after emacspeak pre act comp)
   "Produce an auditory icon"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'open-object)
     (message "Exposed subheadings below current level")))
 
@@ -179,7 +180,7 @@ sections."
 (defun emacspeak-outline-speak-heading (what direction)
   "Function used by all interactive section speaking
 commands. "
-  (declare (special emacspeak-outline-query-before-speaking))
+  (cl-declare (special emacspeak-outline-query-before-speaking))
   (let ((start nil)
         (end nil))
     (funcall what  direction)
@@ -194,38 +195,39 @@ commands. "
     (when (or  emacspeak-outline-dont-query-before-speaking
                (y-or-n-p
                 (format  "Speak %s lines from section %s"
-                         (count-lines start end )
-                         (thing-at-point 'line))))
-      (emacspeak-speak-region start end ))))
-
+                         (count-lines start end) (ems-this-line))))
+      (emacspeak-speak-region start end))))
+;;;###autoload
 (defun emacspeak-outline-speak-next-heading ()
   "Analogous to outline-next-visible-heading,
 except that the outline section is  spoken"
   (interactive)
-  (emacspeak-auditory-icon 'large-movement)
+  (emacspeak-auditory-icon 'section)
   (emacspeak-outline-speak-heading 'outline-next-visible-heading 1))
-
+;;;###autoload
 (defun emacspeak-outline-speak-previous-heading ()
   "Analogous to outline-previous-visible-heading,
 except that the outline section is  spoken"
   (interactive)
-  (emacspeak-auditory-icon 'large-movement)
+  (emacspeak-auditory-icon 'section)
   (emacspeak-outline-speak-heading 'outline-next-visible-heading -1))
-
+;;;###autoload
 (defun emacspeak-outline-speak-forward-heading ()
   "Analogous to outline-forward-same-level,
 except that the outline section is  spoken"
   (interactive)
-  (emacspeak-auditory-icon 'large-movement)
+  (emacspeak-auditory-icon 'section)
   (emacspeak-outline-speak-heading 'outline-forward-same-level 1))
-
+;;;###autoload
 (defun emacspeak-outline-speak-backward-heading ()
   "Analogous to outline-backward-same-level
 except that the outline section is  spoken"
   (interactive)
-  (emacspeak-auditory-icon 'large-movement)
+  (emacspeak-auditory-icon 'section)
   (forward-line -1)
   (emacspeak-outline-speak-heading 'outline-forward-same-level -1))
+
+;;;###autoload
 (defun emacspeak-outline-speak-this-heading ()
   "Speak current outline section starting from point"
   (interactive)
@@ -241,15 +243,14 @@ except that the outline section is  spoken"
      (or emacspeak-outline-dont-query-before-speaking
          (y-or-n-p
           (format "Speak %s lines from section %s"
-                  (count-lines start end)
-                  (thing-at-point 'line))))
-     (emacspeak-speak-region start end ))))
+                  (count-lines start end) (ems-this-line))))
+     (emacspeak-speak-region start end))))
 
 ;;{{{ bind these in outline mode
 
 (defun emacspeak-outline-setup-keys ()
   "Bind keys in outline minor mode map"
-  (declare (special outline-mode-prefix-map))
+  (cl-declare (special outline-mode-prefix-map))
   (define-key outline-mode-prefix-map "p"
     'emacspeak-outline-speak-previous-heading)
   (define-key outline-mode-prefix-map "n"
@@ -272,15 +273,15 @@ except that the outline section is  spoken"
      (require 'foldout))
 (defadvice foldout-zoom-subtree (after emacspeak pre act comp)
   "Provide auditory feedback about the child we zoomed into"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'open-object)
-    (message "Zoomed into outline %s containing %s lines"
-             (thing-at-point 'line)
-             (count-lines (point-min) (point-max)))))
+    (message
+     "Zoomed into outline %s containing %s lines"
+     (ems-this-line) (count-lines (point-min) (point-max)))))
 
 (defadvice foldout-exit-fold (after emacspeak pre act comp)
   "Provide auditory feedback when exiting a fold"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'close-object)
     (emacspeak-speak-line)))
 
