@@ -1,5 +1,5 @@
-;;; emacspeak-rpm-spec.el --- Speech enable rpm spec editor
-;;; $Id: emacspeak-rpm-spec.el 6708 2011-01-04 02:27:29Z tv.raman.tv $
+;;; emacspeak-rpm-spec.el --- Speech enable rpm spec editor  -*- lexical-binding: t; -*-
+;;; $Id$
 ;;; $Author: tv.raman.tv $
 ;;; Description: Controlling mplayer from emacs 
 ;;; Keywords: Emacspeak, rpm-spec streaming media 
@@ -16,7 +16,7 @@
 ;;}}}
 ;;{{{  Copyright:
 
-;;; Copyright (c) 1995 -- 2011, T. V. Raman
+;;; Copyright (c) 1995 -- 2018, T. V. Raman
 ;;; All Rights Reserved. 
 ;;;
 ;;; This file is not part of GNU Emacs, but the same permissions apply.
@@ -40,12 +40,13 @@
 
 ;;{{{  Required modules
 
+(cl-declaim  (optimize  (safety 0) (speed 3)))
 (require 'emacspeak-preamble)
 ;;}}}
 ;;{{{ Introduction:
-
+;;; Commentary:
 ;;; speech-enable rpm-spec-mode --part of Emacs 21 on RH 7.3
-
+;;; Code:
 ;;}}}
 ;;{{{ Advice insertion commands:
 
@@ -62,18 +63,18 @@
     rpm-insert-packager)
   "List of rpm-spec insertion commands to speech-enable.")
 
-(loop for f in emacspeak-rpm-spec-insertion-commands
-      do
-      (eval
-       `(defadvice ,f (after emacspeak pre act comp)
-          "Provide spoken feedback."
-          (when (interactive-p)
-            (let ((entry  (format "%s"
-                                  (quote ,f))))
-              (setq entry
-                    (car (last (split-string entry "-"))))
-              (message
-               (format "Inserted %s entry" entry)))))))
+(cl-loop for f in emacspeak-rpm-spec-insertion-commands
+         do
+         (eval
+          `(defadvice ,f (after emacspeak pre act comp)
+             "Provide spoken feedback."
+             (when (ems-interactive-p)
+               (let ((entry  (format "%s"
+                                     (quote ,f))))
+                 (setq entry
+                       (car (last (split-string entry "-"))))
+                 (message
+                  (format "Inserted %s entry" entry)))))))
 
 ;;}}}
 ;;{{{ Advice navigation 
@@ -81,16 +82,16 @@
   '(rpm-backward-section rpm-beginning-of-section 
                          rpm-forward-section 
                          rpm-end-of-section 
-                         rpm-goto-section )
+                         rpm-goto-section)
   "Navigation commands in rpm-spec to speech-enable.")
-(loop for f in emacspeak-rpm-spec-navigation-commands
-      do
-      (eval
-       `(defadvice ,f (after emacspeak pre act comp)
-          "Provide auditory feedback."
-          (when (interactive-p)
-            (emacspeak-auditory-icon 'large-movement)
-            (emacspeak-speak-line)))))
+(cl-loop for f in emacspeak-rpm-spec-navigation-commands
+         do
+         (eval
+          `(defadvice ,f (after emacspeak pre act comp)
+             "Provide auditory feedback."
+             (when (ems-interactive-p)
+               (emacspeak-auditory-icon 'large-movement)
+               (emacspeak-speak-line)))))
 
 ;;}}}
 ;;{{{ Advice build commands 
@@ -105,19 +106,19 @@
     rpm-build-ba)
   "Build commands from rpm-spec that are speech-enabled.")
 
-(loop for  f in emacspeak-rpm-spec-build-commands
-      do
-      (eval
-       `(defadvice ,f (after emacspeak pre act comp)
-          "Provide spoken feedback."
-          (when (interactive-p)
-            (let ((target  (format "%s"
-                                   (quote ,f))))
-              (setq target
-                    (car (last (split-string target "-"))))
-              (emacspeak-auditory-icon 'task-done)
-              (message
-               (format "Launched build %s " target)))))))
+(cl-loop for  f in emacspeak-rpm-spec-build-commands
+         do
+         (eval
+          `(defadvice ,f (after emacspeak pre act comp)
+             "Provide spoken feedback."
+             (when (ems-interactive-p)
+               (let ((target  (format "%s"
+                                      (quote ,f))))
+                 (setq target
+                       (car (last (split-string target "-"))))
+                 (emacspeak-auditory-icon 'task-done)
+                 (message
+                  (format "Launched build %s " target)))))))
 
 ;;}}}
 ;;{{{ advice toggles 
@@ -127,24 +128,23 @@
     rpm-toggle-clean 
     rpm-toggle-test 
     rpm-toggle-sign-gpg 
-    rpm-toggle-add-attr )
+    rpm-toggle-add-attr)
   "Toggle commands from rpm-spec that are speech-enabled.")
 
-(loop for f in emacspeak-rpm-spec-toggle-commands
-      do
-      (eval
-       `(defadvice ,f (after emacspeak pre act comp)
-          "Provide spoken feedback."
-          (when (interactive-p)
-            (let ((toggle  (format "%s" (quote ,f)))
-                  (switch nil))
-              (setq switch
-                    (intern
-                     (replace-regexp-in-string "toggle"
-                                               "spec"
-                                               toggle)))
-              (emacspeak-auditory-icon
-               (if (eval switch) 'on 'off)))))))
+(cl-loop for f in emacspeak-rpm-spec-toggle-commands
+         do
+         (eval
+          `(defadvice ,f (after emacspeak pre act comp)
+             "Provide spoken feedback."
+             (when (ems-interactive-p)
+               (let ((toggle  (format "%s" (quote ,f)))
+                     (switch nil))
+                 (setq switch
+                       (intern
+                        (replace-regexp-in-string "toggle"
+                                                  "spec"
+                                                  toggle)))
+                 (emacspeak-auditory-icon (if (eval switch) 'on 'off)))))))
 
 ;;}}}
 ;;{{{ voice locking 
