@@ -1,5 +1,5 @@
-;;; emacspeak-speedbar.el --- Speech enable speedbar -- Tool for context-sensitive navigation
-;;; $Id: emacspeak-speedbar.el 6708 2011-01-04 02:27:29Z tv.raman.tv $
+;;; emacspeak-speedbar.el --- Speech enable speedbar -- Tool for context-sensitive navigation  -*- lexical-binding: t; -*-
+;;; $Id$
 ;;; $Author: tv.raman.tv $ 
 ;;; Description: Auditory interface to speedbar
 ;;; Keywords: Emacspeak, Speedbar
@@ -16,7 +16,7 @@
 ;;}}}
 ;;{{{  Copyright:
 
-;;; Copyright (c) 1995 -- 2011, T. V. Raman
+;;; Copyright (c) 1995 -- 2018, T. V. Raman
 ;;; All Rights Reserved. 
 ;;;
 ;;; This file is not part of GNU Emacs, but the same permissions apply.
@@ -56,8 +56,9 @@
 ;;}}}
 ;;{{{  Required modules
 
+(cl-declaim  (optimize  (safety 0) (speed 3)))
 (require 'emacspeak-preamble)
-(require 'speedbar nil t)
+(require 'speedbar "speedbar" 'no-error)
 ;;}}}
 ;;{{{ custom
 
@@ -97,22 +98,21 @@
       (setq indent 
             (if (zerop indent) "" indent))
       (dtk-speak 
-       (concat indent 
-               (thing-at-point 'line))))))
+       (concat indent (ems-this-line))))))
 
 ;;}}}
 ;;{{{ Advice interactive commands:
 
 (defadvice speedbar-close-frame (after emacspeak pre act comp)
   "Cue buffer that becomes active"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'close-object)
     (emacspeak-speak-mode-line)))
 
 (defadvice speedbar-next (around emacspeak pre act comp)
   "Provide reasonable spoken feedback"
   (cond
-   ((interactive-p)
+   ((ems-interactive-p)
     (let ((emacspeak-speak-messages nil))
       ad-do-it
       (emacspeak-speedbar-speak-line)
@@ -122,7 +122,7 @@
 (defadvice speedbar-prev (around emacspeak pre act comp)
   "Provide reasonable spoken feedback"
   (cond
-   ((interactive-p)
+   ((ems-interactive-p)
     (let ((emacspeak-speak-messages nil))
       ad-do-it
       (emacspeak-speedbar-speak-line)
@@ -131,7 +131,7 @@
   ad-return-value)
 (defadvice speedbar-edit-line (after emacspeak pre act comp)
   "Speak line you jumped to"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'large-movement)))
 
 (defadvice speedbar-tag-find (after emacspeak pre act comp)
@@ -146,13 +146,13 @@
 (defadvice speedbar-expand-line (after emacspeak pre act
                                        comp)
   "Speak the line we just expanded"
-  (when (interactive-p) 
+  (when (ems-interactive-p) 
     (emacspeak-speedbar-speak-line)
     (emacspeak-auditory-icon 'open-object)))
 
 (defadvice speedbar-contract-line (after emacspeak pre act comp)
   "Speak the line we just contracted"
-  (when (interactive-p) 
+  (when (ems-interactive-p) 
     (emacspeak-speedbar-speak-line)
     (emacspeak-auditory-icon 'close-object)))
 
@@ -160,7 +160,7 @@
   " Auditory icon and speech feedback indicate result of the
 action"
   (cond
-   ((interactive-p)
+   ((ems-interactive-p)
     ad-do-it
     (emacspeak-auditory-icon 'large-movement)
     (emacspeak-speedbar-speak-line))
@@ -170,14 +170,14 @@ action"
 (defadvice speedbar-restricted-next (after emacspeak pre act
                                            comp)
   "Provide spoken feedback"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'large-movement)
     (emacspeak-speedbar-speak-line)))
 
 (defadvice speedbar-restricted-prev (after emacspeak pre act
                                            comp)
   "Provide spoken feedback"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'large-movement)
     (emacspeak-speedbar-speak-line)))
 
@@ -191,7 +191,7 @@ An automatically updating speedbar consumes resources.")
 (defun emacspeak-speedbar-goto-speedbar ()
   "Switch to the speedbar"
   (interactive)
-  (declare (special emacspeak-speedbar-disable-updates))
+  (cl-declare (special emacspeak-speedbar-disable-updates))
   (unless (get-buffer " SPEEDBAR")
     (speedbar-frame-mode))
   (pop-to-buffer (get-buffer " SPEEDBAR"))
@@ -228,7 +228,7 @@ An automatically updating speedbar consumes resources.")
                (setq action-char (following-char))
                (emacspeak-speedbar-speak-line)
                (emacspeak-auditory-icon
-                (case action-char
+                (cl-case action-char
                   (?+ 'open-object)
                   (?- 'close-object)
                   (t 'large-movement))))
@@ -236,20 +236,20 @@ An automatically updating speedbar consumes resources.")
 
 ;;}}}
 ;;{{{  hooks
-(declaim (special speedbar-key-map))
-(eval-when (load)
+(cl-declaim (special speedbar-mode-map))
+(cl-eval-when (load)
   )
 (defun emacspeak-speedbar-enter-hook ()
   "Actions taken when we enter the Speedbar"
-  (declare (special speedbar-key-map
-                    speedbar-hide-button-brackets-flag))
+  (cl-declare (special speedbar-mode-map
+                       speedbar-hide-button-brackets-flag))
   (dtk-set-punctuations 'all)
   (setq speedbar-hide-button-brackets-flag t)
-  (define-key speedbar-key-map "f"
+  (define-key speedbar-mode-map "f"
     'emacspeak-speedbar-click)
-                                        ;(define-key speedbar-key-map "\M-n"
+                                        ;(define-key speedbar-mode-map "\M-n"
                                         ;'emacspeak-speedbar-forward)
-                                        ;(define-key speedbar-key-map "\M-p"
+                                        ;(define-key speedbar-mode-map "\M-p"
                                         ;'emacspeak-speedbar-backward)
   )
 
@@ -303,18 +303,18 @@ An automatically updating speedbar consumes resources.")
         (personality nil))
     (setq personality
           (cond
-           ((eq face 'speedbar-button-face )
-            emacspeak-speedbar-button-personality  )
-           ((eq face 'speedbar-selected-face )
-            emacspeak-speedbar-selected-personality  )
-           ((eq face 'speedbar-directory-face )
-            emacspeak-speedbar-directory-personality )
-           ((eq face 'speedbar-file-face )
-            emacspeak-speedbar-file-personality  )
-           ((eq face 'speedbar-highlight-face )
-            emacspeak-speedbar-highlight-personality )
-           ((eq face 'speedbar-tag-face )
-            emacspeak-speedbar-tag-personality )
+           ((eq face 'speedbar-button-face)
+            emacspeak-speedbar-button-personality)
+           ((eq face 'speedbar-selected-face)
+            emacspeak-speedbar-selected-personality)
+           ((eq face 'speedbar-directory-face)
+            emacspeak-speedbar-directory-personality)
+           ((eq face 'speedbar-file-face)
+            emacspeak-speedbar-file-personality)
+           ((eq face 'speedbar-highlight-face)
+            emacspeak-speedbar-highlight-personality)
+           ((eq face 'speedbar-tag-face)
+            emacspeak-speedbar-tag-personality)
            (t 'emacspeak-speedbar-default-personality)))
     (put-text-property start end 'personality personality)
     (save-excursion
@@ -323,7 +323,7 @@ An automatically updating speedbar consumes resources.")
 
 ;;}}}
 ;;{{{ keys 
-(declaim (special emacspeak-keymap))
+(cl-declaim (special emacspeak-keymap))
 
                                         ;(define-key emacspeak-keymap '[insert] 'emacspeak-speedbar-goto-speedbar)
 
