@@ -1,5 +1,5 @@
-;;; emacspeak-sudoku.el --- Play SuDoku 
-;;; $Id: emacspeak-sudoku.el 7409 2011-11-16 03:22:20Z tv.raman.tv $
+;;; emacspeak-sudoku.el --- Play SuDoku  -*- lexical-binding: t; -*- 
+;;; $Id$
 ;;; $Author: tv.raman.tv $
 ;;; Description: Playing SuDoku ;;; Keywords: Emacspeak, sudoku
 ;;{{{  LCD Archive entry: 
@@ -15,7 +15,7 @@
 ;;}}}
 ;;{{{  Copyright:
 
-;;; Copyright (c) 1995 -- 2011, T. V. Raman
+;;; Copyright (c) 1995 -- 2018, T. V. Raman
 ;;; All Rights Reserved. 
 ;;;
 ;;; This file is not part of GNU Emacs, but the same permissions apply.
@@ -49,8 +49,25 @@
 ;;}}}
 ;;{{{  Required modules
 ;;; Code:
+(require 'cl-lib)
+(cl-declaim  (optimize  (safety 0) (speed 3)))
 (require 'emacspeak-preamble)
 (require 'stack-f)
+(require 'sudoku "sudoku" 'no-error)
+;;}}}
+;;{{{Forward Decl:
+
+(declare-function sudoku-column "sudoku" (board n))
+(declare-function sudoku-subsquare "sudoku" (board n))
+(declare-function sudoku-get-cell-from-point "sudoku" (num))
+(declare-function sudoku-row "sudoku" (board n))
+(declare-function sudoku-cell "sudoku" (board x y))
+(declare-function sudoku-cell-possibles "sudoku" (board x y))
+(declare-function sudoku-remaining-cells "sudoku" (board))
+(declare-function sudoku-goto-cell "sudoku" (coords))
+(declare-function sudoku-change-cell "sudoku" (board x y input))
+(declare-function sudoku-board-print "sudoku" (board message))
+
 ;;}}}
 ;;{{{ Define additional speak commands:
 
@@ -64,7 +81,7 @@ s   Sub-square Distribution.
 "
   (interactive)
   (let ((c (read-char "Summary: ")))
-    (case c
+    (cl-case c
       (?d (call-interactively 'emacspeak-sudoku-board-distribution-summarize))
       (?r (call-interactively 'emacspeak-sudoku-board-rows-summarize))
       (?c (call-interactively
@@ -76,51 +93,51 @@ s   Sub-square Distribution.
 (defun emacspeak-sudoku-board-distribution-summarize ()
   "Shows distribution of filled numbers."
   (interactive)
-  (declare (special current-board))
+  (cl-declare (special current-board))
   (let ((counts (make-vector 9 0)))
-    (loop for row in current-board
-          do
-          (loop for v in row
-                do
-                (if (> v 0)
-                    (incf (aref counts (1- v))))))
+    (cl-loop for row in current-board
+             do
+             (cl-loop for v in row
+                      do
+                      (if (> v 0)
+                          (cl-incf (aref counts (1- v))))))
     (dtk-speak-list
-     (loop for i across counts collect i)
+     (cl-loop for i across counts collect i)
      3)))
 
 (defun emacspeak-sudoku-board-rows-summarize ()
   "Summarize rows --- speaks number of remaining cells."
   (interactive)
-  (declare (special current-board))
+  (cl-declare (special current-board))
   (dtk-speak-list
-   (loop for r in current-board
-         collect  (count 0 r))
+   (cl-loop for r in current-board
+            collect  (cl-count 0 r))
    3))
 
 (defun emacspeak-sudoku-board-columns-summarize ()
   "Summarize columns --- speaks number of remaining cells."
   (interactive)
-  (declare (special current-board))
+  (cl-declare (special current-board))
   (dtk-speak-list
-   (loop for c from 0 to 8
-         collect  (count 0 (sudoku-column current-board c)))
+   (cl-loop for c from 0 to 8
+            collect  (cl-count 0 (sudoku-column current-board c)))
    3))
 
 (defun emacspeak-sudoku-board-sub-squares-summarize ()
   "Summarize sub-squares --- speaks number of remaining cells."
   (interactive)
-  (declare (special current-board))
+  (cl-declare (special current-board))
   (dtk-speak-list
-   (loop for s from 0 to 8
-         collect  (count 0 (sudoku-subsquare current-board s)))
+   (cl-loop for s from 0 to 8
+            collect  (count 0 (sudoku-subsquare current-board s)))
    3))
 
 (defun emacspeak-sudoku-speak-current-cell-coordinates ()
 
   "speak current cell coordinates."
   (interactive)
-  (let ((row (second (sudoku-get-cell-from-point (point))))
-        (column (first (sudoku-get-cell-from-point (point)))))
+  (let ((row (cl-second (sudoku-get-cell-from-point (point))))
+        (column (cl-first (sudoku-get-cell-from-point (point)))))
     (message
      (format "Row %s Column %s"
              row column))))
@@ -128,32 +145,32 @@ s   Sub-square Distribution.
 (defun emacspeak-sudoku-speak-current-row ()
   "Speak current row."
   (interactive)
-  (declare (special current-board))
+  (cl-declare (special current-board))
   (let ((cell (sudoku-get-cell-from-point (point))))
     (dtk-speak-list (sudoku-row current-board
-                                (second cell ))
+                                (cl-second cell))
                     3)))
 
 (defun emacspeak-sudoku-speak-current-column ()
   "Speak current column."
   (interactive)
-  (declare (special current-board))
+  (cl-declare (special current-board))
   (let ((cell (sudoku-get-cell-from-point (point))))
     (dtk-speak-list (sudoku-column  current-board
-                                    (first cell ))
+                                    (cl-first cell))
                     3)))
 
-(defsubst emacspeak-sudoku-cell-sub-square (cell)
+(defun emacspeak-sudoku-cell-sub-square (cell)
   "Return sub-square that this cell is in."
-  (let ((row (second cell))
-        (column (first cell)))
-    (+ ( * 3 (/ row 3))
+  (let ((row (cl-second cell))
+        (column (cl-first cell)))
+    (+ (* 3 (/ row 3))
        (/ column 3))))
 
 (defun emacspeak-sudoku-speak-current-sub-square ()
   "Speak current sub-square."
   (interactive)
-  (declare (special current-board))
+  (cl-declare (special current-board))
   (let ((cell (sudoku-get-cell-from-point (point))))
     (dtk-speak-list
      (sudoku-subsquare  current-board
@@ -163,20 +180,20 @@ s   Sub-square Distribution.
 (defun emacspeak-sudoku-speak-current-cell-value ()
   "Speak value in current cell."
   (interactive)
-  (declare (special current-board))
+  (cl-declare (special current-board))
   (let ((cell (sudoku-get-cell-from-point (point))))
     (dtk-speak
-     (sudoku-cell current-board (first cell) (second cell)))))
+     (sudoku-cell current-board (cl-first cell) (cl-second cell)))))
 
 (defun emacspeak-sudoku-hint ()
   "Provide hint for current cell."
   (interactive)
-  (declare (special current-board))
+  (cl-declare (special current-board))
   (let* ((cell (sudoku-get-cell-from-point (point)))
          (possibles (sudoku-cell-possibles
                      current-board
-                     (first cell)
-                     (second cell))))
+                     (cl-first cell)
+                     (cl-second cell))))
     (cond
      (possibles 
       (dtk-speak-list possibles))
@@ -185,25 +202,25 @@ s   Sub-square Distribution.
 (defun emacspeak-sudoku-speak-remaining-in-row ()
   "Speaks number of remaining cells in current row."
   (interactive)
-  (declare (special current-board))
+  (cl-declare (special current-board))
   (let ((cell (sudoku-get-cell-from-point (point))))
     (dtk-speak
      (count 0
-            (sudoku-row current-board (second cell))))))
+            (sudoku-row current-board (cl-second cell))))))
 
 (defun emacspeak-sudoku-speak-remaining-in-column ()
   "Speaks number of remaining cells in current column."
   (interactive)
-  (declare (special current-board))
+  (cl-declare (special current-board))
   (let ((cell (sudoku-get-cell-from-point (point))))
     (dtk-speak
      (count 0
-            (sudoku-column current-board  (first cell))))))
+            (sudoku-column current-board  (cl-first cell))))))
 
 (defun emacspeak-sudoku-speak-remaining-in-sub-square ()
   "Speaks number of remaining cells in current sub-square."
   (interactive)
-  (declare (special current-board))
+  (cl-declare (special current-board))
   (let ((cell (sudoku-get-cell-from-point (point))))
     (dtk-speak
      (count 0
@@ -212,7 +229,7 @@ s   Sub-square Distribution.
 (defun emacspeak-sudoku-how-many-remaining ()
   "Speak number of remaining squares to fill."
   (interactive)
-  (declare (special current-board))
+  (cl-declare (special current-board))
   (message
    "%s squares remain"
    (sudoku-remaining-cells current-board)))
@@ -259,15 +276,15 @@ s   Sub-square Distribution.
 
 (defun emacspeak-sudoku-erase-these-cells (cell-list)
   "Erase cells in cell-list taking account of original values."
-  (declare (special start-board current-board
-                    sudoku-onscreen-instructions))
+  (cl-declare (special start-board current-board
+                       sudoku-onscreen-instructions))
   (let ((original (sudoku-get-cell-from-point (point))))
-    (loop for cell in cell-list
-          do
-          (let ((x (car cell))
-                (y (cadr  cell)))
-            (when (= (sudoku-cell start-board x y) 0)
-              (setq current-board (sudoku-change-cell current-board x y 0)))))
+    (cl-loop for cell in cell-list
+             do
+             (let ((x (car cell))
+                   (y (cadr  cell)))
+               (when (= (sudoku-cell start-board x y) 0)
+                 (setq current-board (sudoku-change-cell current-board x y 0)))))
     (setq buffer-read-only nil)
     (erase-buffer)
     (sudoku-board-print current-board
@@ -278,34 +295,34 @@ s   Sub-square Distribution.
 (defun emacspeak-sudoku-erase-current-row ()
   "Erase current row."
   (interactive)
-  (declare (special current-board))
+  (cl-declare (special current-board))
   (let ((cell (sudoku-get-cell-from-point (point))))
     (emacspeak-sudoku-erase-these-cells
-     (loop for i from 0 to  8
-           collect  (list i (second cell)))))
-  (when (interactive-p)
+     (cl-loop for i from 0 to  8
+              collect  (list i (cl-second cell)))))
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'delete-object)))
 
 (defun emacspeak-sudoku-erase-current-column ()
   "Erase current column."
   (interactive)
-  (declare (special current-board))
+  (cl-declare (special current-board))
   (let ((cell (sudoku-get-cell-from-point (point))))
     (emacspeak-sudoku-erase-these-cells
-     (loop for i from 0 to  8
-           collect  (list (first cell) i))))
-  (when (interactive-p)
+     (cl-loop for i from 0 to  8
+              collect  (list (cl-first cell) i))))
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'delete-object)))
 
-(defsubst emacspeak-sudoku-sub-square-cells (square)
+(defun emacspeak-sudoku-sub-square-cells (square)
   "Return list of cells in sub-square."
   (let ((row-start (* (/ square 3)  3)) 
         (col-start (* (% square 3)  3)))
     
-    (loop for r from row-start to (+ 2 row-start)
-          nconc
-          (loop  for c from col-start to (+ 2 col-start)
-                 collect (list c r )))))
+    (cl-loop for r from row-start to (+ 2 row-start)
+             nconc
+             (cl-loop  for c from col-start to (+ 2 col-start)
+                       collect (list c r)))))
 
 (defun emacspeak-sudoku-erase-current-sub-square ()
   "Erase current sub-square."
@@ -315,31 +332,31 @@ s   Sub-square Distribution.
            (sudoku-get-cell-from-point (point))))
          (square-cells (emacspeak-sudoku-sub-square-cells square)))
     (emacspeak-sudoku-erase-these-cells square-cells))
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'delete-object)))
 
 ;;}}}
 ;;{{{ advice motion:
 
-(loop for f   in
-      '(
-        sudoku-move-point-left 
-        sudoku-move-point-leftmost 
-        sudoku-move-point-right 
-        sudoku-move-point-rightmost 
-        sudoku-move-point-up 
-        sudoku-move-point-upmost 
-        sudoku-move-point-down 
-        sudoku-move-point-downmost )
-      do
-      (eval
-       `(defadvice ,f (after emacspeak pre act comp)
-          "Produce auditory output."
-          (when (interactive-p)
-            (emacspeak-sudoku-speak-current-cell-value)
-            (if (eq (get-text-property  (point) 'face) 'bold)
-                (emacspeak-auditory-icon 'item)
-              (emacspeak-auditory-icon 'select-object))))))
+(cl-loop for f   in
+         '(
+           sudoku-move-point-left 
+           sudoku-move-point-leftmost 
+           sudoku-move-point-right 
+           sudoku-move-point-rightmost 
+           sudoku-move-point-up 
+           sudoku-move-point-upmost 
+           sudoku-move-point-down 
+           sudoku-move-point-downmost)
+         do
+         (eval
+          `(defadvice ,f (after emacspeak pre act comp)
+             "Produce auditory output."
+             (when (ems-interactive-p)
+               (emacspeak-sudoku-speak-current-cell-value)
+               (if (eq (get-text-property  (point) 'face) 'bold)
+                   (emacspeak-auditory-icon 'item)
+                 (emacspeak-auditory-icon 'select-object))))))
 
 ;;}}}
 ;;{{{ advice interaction:
@@ -349,7 +366,7 @@ s   Sub-square Distribution.
 See
   http://emacspeak.blogspot.com/2006/02/playing-sudoku-using-auditory-feedback.html
   for details."
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (dtk-set-punctuations 'some)
     (emacspeak-auditory-icon 'open-object)
     (emacspeak-sudoku-speak-current-cell-value)))
@@ -358,12 +375,12 @@ See
 
 (defadvice sudoku-new (after emacspeak pre act comp)
   "Reset history stack."
-  (declare (special emacspeak-sudoku-history-stack))
+  (cl-declare (special emacspeak-sudoku-history-stack))
   (setq emacspeak-sudoku-history-stack nil))
 
 (defadvice sudoku-restart (after emacspeak pre act comp)
   "Provide auditory feedback."
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'open-object)
     (emacspeak-sudoku-speak-current-cell-value)))
 
@@ -375,8 +392,8 @@ See
 (defun emacspeak-sudoku-history-push ()
   "Push current state on to history stack."
   (interactive)
-  (declare (special emacspeak-sudoku-history-stack
-                    current-board))
+  (cl-declare (special emacspeak-sudoku-history-stack
+                       current-board))
   (unless emacspeak-sudoku-history-stack
     (setq emacspeak-sudoku-history-stack
           (stack-create)))
@@ -388,10 +405,10 @@ See
 (defun emacspeak-sudoku-history-pop ()
   "Pop saved state off stack and redraw board."
   (interactive)
-  (declare (special emacspeak-sudoku-history-stack
-                    sudoku-onscreen-instructions
-                    start-board
-                    current-board))
+  (cl-declare (special emacspeak-sudoku-history-stack
+                       sudoku-onscreen-instructions
+                       start-board
+                       current-board))
   (let ((original (sudoku-get-cell-from-point (point))))
     (cond
      ((stack-empty emacspeak-sudoku-history-stack) ;start board
@@ -411,43 +428,43 @@ See
 ;;}}}
 ;;{{{ setup keymap:
 
-(declaim (special sudoku-mode-map))
-(loop for k in
-      '(
-        ("u" emacspeak-sudoku-up-sub-square)
-        ("d" emacspeak-sudoku-down-sub-square)
-        ("/" emacspeak-sudoku-how-many-remaining)
-        ("n" emacspeak-sudoku-next-sub-square)
-        ("p" emacspeak-sudoku-previous-sub-square)
-        ("h" sudoku-move-point-left)
-        ("l" sudoku-move-point-right)
-        ("j" sudoku-move-point-down)
-        ("k" sudoku-move-point-up)
-        ("R" emacspeak-sudoku-speak-remaining-in-row)
-        ("S" emacspeak-sudoku-speak-remaining-in-sub-square)
-        ("C" emacspeak-sudoku-speak-remaining-in-column)
-        ("?" emacspeak-sudoku-hint)
-        ([home] sudoku-move-point-leftmost)
-        ([end] sudoku-move-point-rightmost)
-        ("a" sudoku-move-point-leftmost)
-        ("e" sudoku-move-point-rightmost)
-        ("b" sudoku-move-point-downmost)
-        ("t" sudoku-move-point-upmost)
-        ("." emacspeak-sudoku-speak-current-cell-value )
-        ("=" emacspeak-sudoku-speak-current-cell-coordinates)
-        ("\C-e" emacspeak-prefix-command)
-        ("r" emacspeak-sudoku-speak-current-row)
-        ("c" emacspeak-sudoku-speak-current-column)
-        ("s" emacspeak-sudoku-speak-current-sub-square)
-        ("\M-s" emacspeak-sudoku-erase-current-sub-square)
-        ("\M-r" emacspeak-sudoku-erase-current-row)
-        ("\M-c" emacspeak-sudoku-erase-current-column)
-        (","  emacspeak-sudoku-board-summarizer)
-        ("m" emacspeak-sudoku-history-push)
-        ("M" emacspeak-sudoku-history-pop)
-        )
-      do
-      (define-key  sudoku-mode-map (first k) (second k)))
+(cl-declaim (special sudoku-mode-map))
+(cl-loop for k in
+         '(
+           ("u" emacspeak-sudoku-up-sub-square)
+           ("d" emacspeak-sudoku-down-sub-square)
+           ("/" emacspeak-sudoku-how-many-remaining)
+           ("n" emacspeak-sudoku-next-sub-square)
+           ("p" emacspeak-sudoku-previous-sub-square)
+           ("h" sudoku-move-point-left)
+           ("l" sudoku-move-point-right)
+           ("j" sudoku-move-point-down)
+           ("k" sudoku-move-point-up)
+           ("R" emacspeak-sudoku-speak-remaining-in-row)
+           ("S" emacspeak-sudoku-speak-remaining-in-sub-square)
+           ("C" emacspeak-sudoku-speak-remaining-in-column)
+           ("?" emacspeak-sudoku-hint)
+           ("<home>" sudoku-move-point-leftmost)
+           ("<end>" sudoku-move-point-rightmost)
+           ("a" sudoku-move-point-leftmost)
+           ("e" sudoku-move-point-rightmost)
+           ("b" sudoku-move-point-downmost)
+           ("t" sudoku-move-point-upmost)
+           ("." emacspeak-sudoku-speak-current-cell-value)
+           ("=" emacspeak-sudoku-speak-current-cell-coordinates)
+           ("\C-e" emacspeak-prefix-command)
+           ("r" emacspeak-sudoku-speak-current-row)
+           ("c" emacspeak-sudoku-speak-current-column)
+           ("s" emacspeak-sudoku-speak-current-sub-square)
+           ("\M-s" emacspeak-sudoku-erase-current-sub-square)
+           ("\M-r" emacspeak-sudoku-erase-current-row)
+           ("\M-c" emacspeak-sudoku-erase-current-column)
+           (","  emacspeak-sudoku-board-summarizer)
+           ("m" emacspeak-sudoku-history-push)
+           ("M" emacspeak-sudoku-history-pop)
+           )
+         do
+         (define-key  sudoku-mode-map (cl-first k) (cl-second k)))
 
 ;;}}}
 (provide 'emacspeak-sudoku)
