@@ -1,5 +1,5 @@
-;;; emacspeak-tetris.el --- Speech enable game of Tetris
-;;; $Id: emacspeak-tetris.el 7280 2011-10-16 16:04:28Z tv.raman.tv $
+;;; emacspeak-tetris.el --- Speech enable game of Tetris  -*- lexical-binding: t; -*-
+;;; $Id$
 ;;; $Author: tv.raman.tv $ 
 ;;; Description: Auditory interface to tetris
 ;;; Keywords: Emacspeak, Speak, Spoken Output, tetris
@@ -16,7 +16,7 @@
 ;;}}}
 ;;{{{  Copyright:
 
-;;; Copyright (c) 1995 -- 2011, T. V. Raman
+;;; Copyright (c) 1995 -- 2018, T. V. Raman
 ;;; All Rights Reserved. 
 ;;;
 ;;; This file is not part of GNU Emacs, but the same permissions apply.
@@ -38,9 +38,17 @@
 ;;}}}
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+;;{{{ Introduction:
+;;; Commentary:
+;;; Speech-enables tetris.
+;;; Code:
+
+;;}}}
 ;;{{{  Required modules
 
+(cl-declaim  (optimize  (safety 0) (speed 3)))
 (require 'emacspeak-preamble)
+(require 'tetris "tetris" 'no-error)
 ;;}}}
 ;;{{{  Introduction 
 
@@ -54,8 +62,8 @@
 
 (defun emacspeak-tetris-define-pronunciations ()
   "Define speech display for tetris"
-  (declare (special emacspeak-tetris-pronunciations-defined
-                    emacspeak-pronounce-pronunciation-table emacspeak-pronounce-dictionaries-loaded))
+  (cl-declare (special emacspeak-tetris-pronunciations-defined
+                       emacspeak-pronounce-pronunciation-table emacspeak-pronounce-dictionaries-loaded))
   (require 'emacspeak-pronounce)
   (unless emacspeak-tetris-pronunciations-defined
     (setq emacspeak-tetris-pronunciations-defined t)
@@ -85,17 +93,17 @@
 
 (aset emacspeak-tetris-shape-name-table 1 " block ")
 (aset emacspeak-tetris-shape-name-table 2 " right elbow ")
-(aset emacspeak-tetris-shape-name-table 3 " left elbow" )
+(aset emacspeak-tetris-shape-name-table 3 " left elbow")
 (aset emacspeak-tetris-shape-name-table 4 " z ")
 (aset emacspeak-tetris-shape-name-table 5 " s ")
 (aset emacspeak-tetris-shape-name-table 6 " inverted t ")
 (aset emacspeak-tetris-shape-name-table 7 " line ")
-(defsubst emacspeak-tetris-blank-row ()
-  (declare (special tetris-width))
+(defun emacspeak-tetris-blank-row ()
+  (cl-declare (special tetris-width))
   (make-string tetris-width  0))
 
-(defsubst emacspeak-tetris-shape-name (tile)
-  (declare (special emacspeak-tetris-shape-name-table))
+(defun emacspeak-tetris-shape-name (tile)
+  (cl-declare (special emacspeak-tetris-shape-name-table))
   (aref emacspeak-tetris-shape-name-table tile))
 
 ;;}}}
@@ -103,8 +111,8 @@
 ;;;we need this because in the new version a line is not a row (on the
 ;;;bottom)
 
-(defsubst emacspeak-tetris-get-current-row ()
-  (declare (special tetris-border))
+(defun emacspeak-tetris-get-current-row ()
+  (cl-declare (special tetris-border))
   (save-excursion
     (beginning-of-line)
     (let ((start nil))
@@ -114,7 +122,7 @@
       (while (not (= (following-char) tetris-border)) (forward-char 1))
       (buffer-substring start (point)))))
 
-(defsubst emacspeak-tetris-speak-row ()
+(defun emacspeak-tetris-speak-row ()
   "Speak current tetris row"
   (interactive)
   (dtk-speak (emacspeak-tetris-get-current-row)))
@@ -123,30 +131,30 @@
   "Speak where on the tetris board we are"
   (interactive)
   (emacspeak-speak-line-number))
-(defsubst emacspeak-tetris-speak-x-coordinate ()
+(defun emacspeak-tetris-speak-x-coordinate ()
   "Speak current position"
   (interactive)
-  (declare  (special tetris-pos-x
-                     tetris-shape))
+  (cl-declare  (special tetris-pos-x
+                        tetris-shape))
   (message "%s at %s"
            (emacspeak-tetris-shape-name (1+ tetris-shape))
            tetris-pos-x))
 
-(defsubst emacspeak-tetris-speak-coordinates ()
+(defun emacspeak-tetris-speak-coordinates ()
   "Speak current position"
   (interactive)
-  (declare  (special tetris-pos-x tetris-pos-y
-                     tetris-shape))
+  (cl-declare  (special tetris-pos-x tetris-pos-y
+                        tetris-shape))
   (message "%s at %s %s"
            (emacspeak-tetris-shape-name (1+ tetris-shape))
            tetris-pos-x tetris-pos-y))
 
-(defsubst emacspeak-tetris-speak-current-shape ()
+(defun emacspeak-tetris-speak-current-shape ()
   "Speak current shape"
   (interactive)
-  (declare (special tetris-shape tetris-rot
-                    tetris-next-shape
-                    tetris-n-rows emacspeak-tetris-filled-a-row))
+  (cl-declare (special tetris-shape tetris-rot
+                       tetris-next-shape
+                       tetris-n-rows emacspeak-tetris-filled-a-row))
   (dtk-speak
    (format "%s %s at rotation  %s next is %s"
            (if emacspeak-tetris-filled-a-row 
@@ -158,31 +166,31 @@
   (if emacspeak-tetris-filled-a-row
       (setq  emacspeak-tetris-filled-a-row nil)))
 
-(defsubst emacspeak-tetris-speak-next-shape ()
+(defun emacspeak-tetris-speak-next-shape ()
   "Speak next shape"
   (interactive)
-  (declare (special tetris-next-shape ))
+  (cl-declare (special tetris-next-shape))
   (dtk-speak
    (format "%s "
            (emacspeak-tetris-shape-name (1+ tetris-next-shape)))))
 
-(defsubst emacspeak-tetris-speak-current-shape-and-coordinates ()
+(defun emacspeak-tetris-speak-current-shape-and-coordinates ()
   "Speak shape orientation and coordinates"
   (interactive)
-  (declare (special  tetris-pos-x tetris-pos-y
-                     tetris-shape tetris-rot))
+  (cl-declare (special  tetris-pos-x tetris-pos-y
+                        tetris-shape tetris-rot))
   (message "%s at %s %s  at rotation %s"
            (emacspeak-tetris-shape-name (1+ tetris-shape))
            tetris-pos-x
            tetris-pos-y
            tetris-rot))
 
-(defsubst emacspeak-tetris-speak-score()
+(defun emacspeak-tetris-speak-score()
   "Speak the score"
   (interactive)
-  (declare (special tetris-n-shapes tetris-n-rows
-                    tetris-score
-                    tetris-width))
+  (cl-declare (special tetris-n-shapes tetris-n-rows
+                       tetris-score
+                       tetris-width))
   (dtk-speak
    (format "%s complete rows after %s tiles to score %s  for an average of %s"
            tetris-n-rows
@@ -200,8 +208,8 @@
   "*Set this to a convenient value so you get time to look at what is
 going on. Reduce it as you get better.")
 
-(defsubst emacspeak-tetris-tick-period ()
-  (declare (special emacspeak-tetris-tick-period))
+(defun emacspeak-tetris-tick-period ()
+  (cl-declare (special emacspeak-tetris-tick-period))
   emacspeak-tetris-tick-period)
 
 (defalias 'tetris-get-tick-period 'emacspeak-tetris-tick-period)
@@ -321,9 +329,9 @@ compensate for the difference between aural and visual
 interaction.  Despite the clock being slowed down, playing
 Tetris with speech feedback alone requires a lot of
 concentration and the game is a good mental challenge. "
-  (declare (special tetris-tick-period
-                    emacspeak-tetris-blank-row))
-  (when (interactive-p)
+  (cl-declare (special tetris-tick-period
+                       emacspeak-tetris-blank-row))
+  (when (ems-interactive-p)
     (setq tetris-tick-period emacspeak-tetris-tick-period)
     (setq tetris-width emacspeak-tetris-width)
     (setq emacspeak-tetris-blank-row (emacspeak-tetris-blank-row))
@@ -332,13 +340,14 @@ concentration and the game is a good mental challenge. "
     (emacspeak-tetris-define-pronunciations)
     (emacspeak-tetris-define-keys)
     (emacspeak-tetris-speak-current-shape)
-    (goto-line tetris-height)
+    (goto-char (point-min))
+    (forward-line tetris-height)
     (message "Welcome to tetris!")))
 
 (defadvice tetris-start-game (after emacspeak pre act comp)
   "Provide auditory feedback"
-  (declare (special emacspeak-tetris-blank-row))
-  (when (interactive-p)
+  (cl-declare (special emacspeak-tetris-blank-row))
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'open-object)
     (setq tetris-width emacspeak-tetris-width)
     (setq emacspeak-tetris-blank-row (emacspeak-tetris-blank-row))
@@ -346,7 +355,7 @@ concentration and the game is a good mental challenge. "
 
 (defadvice tetris-end-game (after emacspeak pre act comp)
   "Provide auditory feedback"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'close-object)
     (message "Ending current game")))
 
@@ -356,42 +365,42 @@ concentration and the game is a good mental challenge. "
 
 (defadvice tetris-rotate-next (after emacspeak pre act comp)
   "Provide auditory feedback"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-tetris-speak-current-shape)))
 
 (defadvice tetris-rotate-prev (after emacspeak pre act comp)
   "Provide auditory feedback"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-tetris-speak-current-shape)))
 
 (defadvice tetris-move-left-edge (after emacspeak pre act comp)
   "Speak coordinates"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-tetris-speak-column)))
 
 (defadvice tetris-move-right-edge (after emacspeak pre act comp)
   "Speak coordinates"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-tetris-speak-column)))
 
 (defadvice tetris-move-to-x-pos (after emacspeak pre act comp)
   "Speak coordinates"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-tetris-speak-column)))
 
 (defadvice tetris-move-left (after emacspeak pre act comp)
   "Speak coordinates"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-tetris-speak-column)))
 
 (defadvice tetris-move-right (after emacspeak pre act comp)
   "Speak coordinates"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-tetris-speak-column)))
 
 (defadvice tetris-move-bottom (after emacspeak pre act comp)
   "Provide auditory feedback as the tile falls"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'close-object)))
 (defvar emacspeak-tetris-filled-a-row nil
   "Temporary flag indicating if we just filled a row.")
@@ -404,7 +413,8 @@ concentration and the game is a good mental challenge. "
 
 (defadvice tetris-draw-score (around emacspeak pre act comp)
   "dont draw the score"
-  nil)
+  (when nil ad-do-it) ; to silence byte-compiler 
+  )
 ;;}}}
 ;;{{{  speak line 
 
@@ -413,16 +423,16 @@ concentration and the game is a good mental challenge. "
 
 (defun emacspeak-tetris-define-keys ()
   "Setup emacspeak tetris key bindings "
-  (declare (special tetris-mode-map))
+  (cl-declare (special tetris-mode-map))
   (define-key tetris-mode-map "v" 'emacspeak-tetris-speak-column)
   (define-key tetris-mode-map "n" 'emacspeak-self-insert-command)
   (define-key tetris-mode-map "N" 'tetris-start-game)
   (define-key  tetris-mode-map "c" 'emacspeak-tetris-speak-row)
-  (loop for i from 0 to 9
-        do
-        (define-key tetris-mode-map
-          (format "%s" i)
-          'tetris-move-to-x-pos))
+  (cl-loop for i from 0 to 9
+           do
+           (define-key tetris-mode-map
+             (format "%s" i)
+             'tetris-move-to-x-pos))
   (define-key tetris-mode-map "a" 'tetris-move-left-edge)
   (define-key tetris-mode-map "e" 'tetris-move-right-edge)
   (define-key tetris-mode-map "l" 'tetris-move-right)
@@ -445,9 +455,9 @@ concentration and the game is a good mental challenge. "
 (defun emacspeak-tetris-goto-bottom-row ()
   "Move to and speak bottom row"
   (interactive)
-  (declare (special tetris-height))
-  (goto-line
-   (1+ tetris-height))
+  (cl-declare (special tetris-height))
+  (ems-with-messages-silenced
+   (goto-line (1+ tetris-height)))
   (emacspeak-tetris-speak-row))
 
 (defvar emacspeak-tetris-blank-row
@@ -457,8 +467,8 @@ concentration and the game is a good mental challenge. "
 (defun emacspeak-tetris-goto-top-row ()
   "Move to and speak the top row"
   (interactive)
-  (declare (special tetris-height
-                    emacspeak-tetris-blank-row))
+  (cl-declare (special tetris-height
+                       emacspeak-tetris-blank-row))
   (goto-line tetris-height)
   (search-backward emacspeak-tetris-blank-row  nil t)
   (forward-line 1)
@@ -467,7 +477,7 @@ concentration and the game is a good mental challenge. "
 (defun tetris-move-left-edge ()
   "Moves the shape to the left edge  of the playing area"
   (interactive)
-  (declare (special tetris-pos-x tetris-width))
+  (cl-declare (special tetris-pos-x tetris-width))
   (let ((hit nil))
     (tetris-erase-shape)
     (while(and (> tetris-pos-x 0)
@@ -479,7 +489,7 @@ concentration and the game is a good mental challenge. "
 (defun tetris-move-right-edge ()
   "Moves the shape to the right edge  of the playing area"
   (interactive)
-  (declare (special tetris-pos-x tetris-width))
+  (cl-declare (special tetris-pos-x tetris-width))
   (let ((hit nil)
         (max(- tetris-width (tetris-shape-width))))
     (tetris-erase-shape)
@@ -492,31 +502,31 @@ concentration and the game is a good mental challenge. "
 (defun tetris-move-to-x-pos ()
   "Moves the shape to a specified x position if possible"
   (interactive)
-  (declare (special tetris-pos-x tetris-width))
+  (cl-declare (special tetris-pos-x tetris-width))
   (let ((hit nil)
         (x
          (condition-case nil
-             (read (format "%c" last-input-event ))
-           (error nil )))
+             (read (format "%c" last-input-event))
+           (error nil)))
         (max(- tetris-width (tetris-shape-width)))
         (diff nil))
     (setq diff (- x tetris-pos-x))
     (tetris-erase-shape)
     (cond
-     ((plusp diff)
+     ((cl-plusp diff)
       (while(and (< tetris-pos-x max)
                  (> diff 0)
                  (not hit))
         (setq tetris-pos-x (1+ tetris-pos-x))
         (setq hit (tetris-test-shape))
-        (decf diff)))
-     ((minusp diff)
+        (cl-decf diff)))
+     ((cl-minusp diff)
       (while(and (> tetris-pos-x 0)
-                 (minusp diff)
+                 (cl-minusp diff)
                  (not hit))
         (setq tetris-pos-x (1- tetris-pos-x))
         (setq hit (tetris-test-shape))
-        (incf diff))))
+        (cl-incf diff))))
     (tetris-draw-shape)))
 
 ;;}}}
@@ -524,23 +534,31 @@ concentration and the game is a good mental challenge. "
 
 (defun emacspeak-tetris-get-column-contents (x)
   "Return column contents as a string"
-  (declare (special tetris-height tetris-top-left-y
-                    tetris-top-left-x))
+  (cl-declare (special tetris-height tetris-top-left-y
+                       tetris-top-left-x))
   (let ((result nil))
     (setq x (+ tetris-top-left-x x))
     (setq result
-          (loop for y from tetris-top-left-y
-                to (+ tetris-height tetris-top-left-y)
-                collect (gamegrid-get-cell x y)))
+          (cl-loop for y from tetris-top-left-y
+                   to (+ tetris-height tetris-top-left-y)
+                   collect (gamegrid-get-cell x y)))
     (mapconcat 'char-to-string result "")))
 
 (defun emacspeak-tetris-speak-column (&optional x)
   "Speak  column --default is to speak current column"
   (interactive "p")
-  (declare (special tetris-pos-x))
+  (cl-declare (special tetris-pos-x))
   (or  x (setq x tetris-pos-x))
   (dtk-speak (emacspeak-tetris-get-column-contents x)))
 
+;;}}}
+;;{{{ Mode hook:
+
+(add-hook
+ 'tetris-mode-hook
+ #'(lambda ()
+     (voice-lock-mode -1)))
+
 ;;}}}
 (provide 'emacspeak-tetris)
 ;;{{{ end of file 
