@@ -1,5 +1,5 @@
-;;; emacspeak-view.el --- Speech enable View mode -- Efficient browsing of read-only content
-;;; $Id: emacspeak-view.el 7323 2011-10-26 00:50:39Z tv.raman.tv $
+;;; emacspeak-view.el --- Speech enable View mode -- Efficient browsing of read-only content  -*- lexical-binding: t; -*-
+;;; $Id$
 ;;; $Author: tv.raman.tv $ 
 ;;; DescriptionEmacspeak extensions for view
 ;;; Keywords:emacspeak, audio interface to emacs, view-mode
@@ -15,7 +15,7 @@
 
 ;;}}}
 ;;{{{  Copyright:
-;;;Copyright (C) 1995 -- 2011, T. V. Raman 
+;;;Copyright (C) 1995 -- 2018, T. V. Raman 
 ;;; Copyright (c) 1996 by T. V. Raman 
 ;;; All Rights Reserved. 
 ;;;
@@ -38,22 +38,24 @@
 ;;}}}
 
 ;;{{{  Introduction:
-
+;;; Commentary:
 ;;; Provide additional advice to view-mode
-
+;;; Code:
 ;;}}}
 ;;{{{ requires
+(cl-declaim  (optimize  (safety 0) (speed 3)))
 (require 'emacspeak-preamble)
 
 ;;}}}
 ;;{{{  Setup view mode to work with emacspeak
 
 ;;; restore emacspeak keybindings:
-(declaim (special emacspeak-prefix))
-(add-hook 'view-mode-hook
-          (function (lambda ()
-                      (local-unset-key emacspeak-prefix )
-                      (emacspeak-view-setup-keys))))
+(cl-declaim (special emacspeak-prefix))
+(add-hook
+ 'view-mode-hook
+ #'(lambda ()
+     (local-unset-key emacspeak-prefix)
+     (emacspeak-view-setup-keys)))
 ;;; Generate automatic advise:
 
 ;;}}}
@@ -75,7 +77,8 @@
 
 (defadvice view-mode (after emacspeak pre act comp)
   "Announce what happened"
-  (when (interactive-p)
+  (cl-declare (special view-mode-map))
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'open-object)
     (emacspeak-speak-load-directory-settings)
     (outline-minor-mode 1)
@@ -84,93 +87,99 @@
                  (key-description
                   (where-is-internal 'View-exit view-mode-map 'firstonly)))
       (message "Exited view mode"))))
-
-(defadvice View-quit (after emacspeak pre act comp)
-  "Provide auditory feedback."
-  (when (interactive-p)
-    (emacspeak-auditory-icon 'close-object)
-    (emacspeak-speak-mode-line)))
+(cl-loop
+ for f in
+ '(
+   View-exit-and-edit View-kill-and-leave
+   View-quit-all View-quit)
+ do
+ (eval
+  `(defadvice ,f (after emacspeak pre act comp)
+     "Provide auditory feedback."
+     (when (ems-interactive-p)
+       (emacspeak-auditory-icon 'close-object)
+       (emacspeak-speak-mode-line)))))
 
 (defadvice View-exit (after emacspeak pre act comp)
   "Provide auditory feedback."
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'close-object)
     (emacspeak-speak-mode-line)))
 
 (defadvice View-leave (after emacspeak pre act comp)
   "Provide auditory feedback."
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-speak-mode-line)))
 
 (defadvice View-search-regexp-forward (after emacspeak pre act comp)
   "Provide auditory feedback"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (let ((emacspeak-show-point t))
-      (emacspeak-speak-line ))
+      (emacspeak-speak-line))
     (emacspeak-auditory-icon 'search-hit)))
 
 (defadvice View-search-regexp-backward (after emacspeak pre act comp)
   "Provide auditory feedback"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (let ((emacspeak-show-point t))
-      (emacspeak-speak-line ))
+      (emacspeak-speak-line))
     (emacspeak-auditory-icon 'search-hit)))
 
 (defadvice View-search-last-regexp-forward (after emacspeak pre act comp)
   "Provide auditory feedback"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (let ((emacspeak-show-point t))
-      (emacspeak-speak-line ))
+      (emacspeak-speak-line))
     (emacspeak-auditory-icon 'search-hit)))
 
 (defadvice View-search-last-regexp-backward (after emacspeak pre act comp)
   "Provide auditory feedback"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (let ((emacspeak-show-point t))
-      (emacspeak-speak-line ))
+      (emacspeak-speak-line))
     (emacspeak-auditory-icon 'search-hit)))
 
 (defadvice view-exit (after emacspeak pre act comp)
   "Provide auditory feedback"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'close-object)
     (emacspeak-speak-mode-line)))
 
 (defadvice View-scroll-one-more-line (after emacspeak pre act comp)
   "Provide auditory feedback"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'scroll)
     (emacspeak-speak-line)))
 
 (defadvice View-scroll-line-forward (after emacspeak pre act comp)
   "Provide auditory feedback"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'scroll)
     (emacspeak-speak-line)))
 
 (defadvice View-scroll-line-backward (after emacspeak pre act comp)
   "Provide auditory feedback"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'scroll)
     (emacspeak-speak-line)))
 (defadvice View-scroll-page-forward-set-page-size (after emacspeak pre act comp)
   "Provide auditory feedback"
-  (when (interactive-p)
-    (let ((start (point )))
+  (when (ems-interactive-p)
+    (let ((start (point)))
       (emacspeak-auditory-icon 'scroll)
       (dtk-speak (emacspeak-get-window-contents)))))
 
 (defadvice View-scroll-page-backward-set-page-size (after emacspeak pre act comp)
   "Provide auditory feedback"
-  (when (interactive-p)
-    (let ((start (point )))
+  (when (ems-interactive-p)
+    (let ((start (point)))
       (emacspeak-auditory-icon 'scroll)
       (dtk-speak (emacspeak-get-window-contents)))))
 
 (defadvice View-scroll-half-page-forward (around emacspeak pre act comp)
   "Read newly scrolled contents"
   (cond
-   ((interactive-p)
+   ((ems-interactive-p)
     (let ((start (point)))
       ad-do-it
       (emacspeak-auditory-icon 'scroll)
@@ -184,7 +193,7 @@
 (defadvice View-scroll-half-page-backward (around emacspeak pre act comp)
   "Read newly scrolled contents"
   (cond
-   ((interactive-p)
+   ((ems-interactive-p)
     (let ((start (point)))
       ad-do-it
       (emacspeak-auditory-icon 'scroll)
@@ -198,23 +207,23 @@
 
 (defadvice View-scroll-lines-forward-set-scroll-size (after emacspeak pre act comp)
   "Provide auditory feedback"
-  (when (interactive-p)
-    (let ((start (point )))
+  (when (ems-interactive-p)
+    (let ((start (point)))
       (emacspeak-auditory-icon 'scroll)
       (save-excursion
         (forward-line (window-height))
-        (emacspeak-speak-region start (point ))))))
+        (emacspeak-speak-region start (point))))))
 
 (defadvice View-scroll-lines-forward (after emacspeak pre act comp)
   "Provide auditory feedback"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'scroll)
     (dtk-speak (emacspeak-get-window-contents))))
 
 (defadvice View-scroll-lines-backward (around  emacspeak pre act comp)
   "provide auditory feedback"
   (cond
-   ((interactive-p)
+   ((ems-interactive-p)
     (let ((buffer (current-buffer)))
       ad-do-it
       (cond
@@ -226,52 +235,51 @@
    (t ad-do-it))
   ad-return-value)
 
-(defadvice View-back-to-mark ( after emacspeak pre act comp)
+(defadvice View-back-to-mark (after emacspeak pre act comp)
   "Provide auditory feedback"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'large-movement)
     (let ((emacspeak-show-point t))
       (emacspeak-speak-line))))
 
 (defadvice View-goto-line (after emacspeak pre act comp)
   "Provide spoken feedback"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (let ((line-number
            (format "line %s"
-                   (ad-get-arg 0 ))))
+                   (ad-get-arg 0))))
       (put-text-property 0 (length line-number)
                          'personality voice-annotate line-number)
       (emacspeak-auditory-icon 'large-movement)
       (dtk-speak
-       (concat line-number
-               (thing-at-point 'line))))))
+       (concat line-number (ems-this-line))))))
 (defadvice View-scroll-to-buffer-end (after emacspeak pre act comp)
   "Provide auditory feedback"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-speak-line)
     (emacspeak-auditory-icon 'large-movement)))
 
 (defadvice View-goto-percent (after emacspeak pre act comp)
   "Provide auditory feedback"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'scroll)
     (dtk-speak (emacspeak-get-window-contents))))
 
 (defadvice View-revert-buffer-scroll-page-forward (after emacspeak pre act comp)
   "Provide auditory feedback"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'scroll)
     (dtk-speak (emacspeak-get-window-contents))))
 
 (defadvice View-scroll-page-forward (after emacspeak pre act comp)
   "Provide auditory feedback"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'scroll)
     (dtk-speak (emacspeak-get-window-contents))))
 
 (defadvice View-scroll-page-backward (after emacspeak pre act comp)
   "Provide auditory feedback"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'scroll)
     (dtk-speak (emacspeak-get-window-contents))))
 
@@ -288,32 +296,39 @@ keybindings for view mode")
         'delete-char
         'completion-kill-region
         )
-  "Editting commands that emacspeak should rebind in view mode")
+  "Editing commands that emacspeak should rebind in view mode")
 
 (defun emacspeak-view-optimize-view-keys()
   "optimize keybindings for emacspeak in view mode"
-  (declare (special emacspeak-view-edit-commands
-                    emacspeak-view-keys-optimized
-                    view-mode-map emacspeak-keymap))
+  (cl-declare (special emacspeak-view-edit-commands
+                       emacspeak-view-keys-optimized
+                       view-mode-map emacspeak-keymap))
   (unless emacspeak-view-keys-optimized
     (setq emacspeak-view-keys-optimized t)
-    (loop for edit-command in emacspeak-view-edit-commands
-          do
-          (let ((edit-keys (where-is-internal edit-command (list view-mode-map))))
-            (loop for key in edit-keys 
-                  do
-                  (let ((command (lookup-key emacspeak-keymap key)))
-                    (when command
-                      (define-key view-mode-map key command))))))))
+    (cl-loop for edit-command in emacspeak-view-edit-commands
+             do
+             (let ((edit-keys (where-is-internal edit-command (list view-mode-map))))
+               (cl-loop for key in edit-keys 
+                        do
+                        (let ((command (lookup-key emacspeak-keymap key)))
+                          (when command
+                            (define-key view-mode-map key command))))))
+    (cl-loop for k in
+             '(
+               ("[" backward-paragraph)
+               ("]" forward-paragraph)
+               )
+             do
+             (define-key view-mode-map (cl-first k) (cl-second k)))))
 
 (defun emacspeak-view-setup-keys()
   "Setup emacspeak convenience keys"
-  (declare (special view-mode-map))
-  (loop for i from 0 to 9
-        do
-        (define-key view-mode-map
-          (format "%s" i)
-          'emacspeak-speak-predefined-window))
+  (cl-declare (special view-mode-map))
+  (cl-loop for i from 0 to 9
+           do
+           (define-key view-mode-map
+             (format "%s" i)
+             'emacspeak-speak-predefined-window))
 ;;;convenience keys
   (define-key view-mode-map "\C-j"
     'emacspeak-hide-speak-block-sans-prefix)
@@ -323,7 +338,6 @@ keybindings for view mode")
   (define-key view-mode-map "\M-p" 'outline-previous-visible-heading)
   (define-key view-mode-map " " 'scroll-up)
   (define-key view-mode-map "\d" 'scroll-down)
-  (define-key view-mode-map "P" 'dtk-pause)
   (define-key view-mode-map "R" 'dtk-resume)
   (define-key view-mode-map "S" 'dtk-stop)
   (define-key view-mode-map "t" 'emacspeak-view-line-to-top)
