@@ -1,5 +1,6 @@
-;;; emacspeak-vm.el --- Speech enable VM -- A powerful mail agent (and the one I use)
-;;; $Id: emacspeak-vm.el 8146 2013-02-09 20:05:08Z tv.raman.tv $
+;;; emacspeak-vm.el --- Speech enable VMMail    -*- lexical-binding: t; -*-
+;;;(and the one I use)
+;;; $Id$
 ;;; $Author: tv.raman.tv $
 ;;; Description:  Emacspeak extension to speech enhance vm
 ;;; Keywords: Emacspeak, VM, Email, Spoken Output, Voice annotations
@@ -16,7 +17,7 @@
 ;;}}}
 ;;{{{  Copyright:
 
-;;;Copyright (C) 1995 -- 2011, T. V. Raman
+;;;Copyright (C) 1995 -- 2018, T. V. Raman
 ;;; Copyright (c) 1994, 1995 by Digital Equipment Corporation.
 ;;; All Rights Reserved.
 ;;;
@@ -39,12 +40,35 @@
 ;;}}}
 
 ;;{{{  Introduction:
+
+;;; Commentary:
 ;;; This module extends the mail reader vm.
 ;;; Uses voice locking for message headers and cited messages
-
+;;; Code:
 ;;}}}
 ;;{{{ requires
+(cl-declaim  (optimize  (safety 0) (speed 3)))
 (require 'emacspeak-preamble)
+(require  'vm "vm" 'no-error)
+(require 'browse-url)
+;;}}}
+;;{{{ Forward Decls:
+
+(declare-function vm-from-of "vm-message" (msg))
+(declare-function  vm-subject-of "vm-message" (msg))
+(declare-function  vm-to-of "vm-message" (msg))
+(declare-function  vm-su-full-name "vm-summary" (msg))
+(declare-function  vm-su-from "vm-summary" (msg))
+(declare-function vm-su-subject "vm-summary" (msg))
+(declare-function  vm-su-to-names "vm-summary" (msg))
+(declare-function  vm-su-to "vm-summary" (msg))
+(declare-function  vm-su-line-count "vm-summary" (msg))
+(declare-function vm-decode-mime-encoded-words-in-string "vm-mime" (s))
+(declare-function  vm-labels-of "vm-message" (msg))
+(declare-function  vm-goto-message "vm-message" (msg))
+(declare-function vm-delete-message "vm-message" (msg))
+(declare-function  u-vm-color-fontify-buffer "u-vm-color" nil)
+(declare-function  u-vm-color-summary-mode "u-vm-color" (&optional arg))
 
 ;;}}}
 ;;{{{ voice locking:
@@ -55,36 +79,32 @@
   :group 'vm
   :prefix "emacspeak-vm-")
 
-(defcustom emacspeak-vm-voice-lock-messages nil
+(defcustom emacspeak-vm-voice-loc nil
   "Set this to T if you want messages automatically voice locked.
 Note that some badly formed mime messages  cause trouble."
   :type 'boolean
   :group 'emacspeak-vm)
 
-(add-hook 'vm-mode-hook
-          'emacspeak-vm-mode-setup)
+(add-hook 'vm-mode-hook 'emacspeak-vm-mode-setup)
 
 (defun emacspeak-vm-mode-setup ()
   "Setup function placed on vm-mode-hook by Emacspeak."
-  (declare (special  dtk-punctuation-mode emacspeak-vm-voice-lock-messages
-                     dtk-allcaps-beep))
+  (cl-declare (special  dtk-punctuation-mode dtk-allcaps-beep))
   (setq dtk-punctuation-mode 'all)
   (when dtk-allcaps-beep
-    (dtk-toggle-allcaps-beep))
-  (emacspeak-dtk-sync)
-  (when emacspeak-vm-voice-lock-messages))
+    (dtk-toggle-allcaps-beep)))
 
 ;;}}}
 ;;{{{ inline helpers
 
-(defsubst emacspeak-vm-number-of (message) (aref (aref message 1) 0))
+(defun emacspeak-vm-number-of (message) (aref (aref message 1) 0))
 
 ;;}}}
 ;;{{{ Advice completions
 
 (defadvice vm-minibuffer-complete-word (around emacspeak pre act)
   "Say what you completed."
-  (let ((prior (point ))
+  (let ((prior (save-excursion (skip-syntax-backward "^ >") (point)))
         (dtk-stop-immediately t))
     (emacspeak-kill-buffer-carefully "*Completions*")
     ad-do-it
@@ -99,7 +119,7 @@ Note that some badly formed mime messages  cause trouble."
 
 (defadvice vm-minibuffer-complete-word-and-exit (around emacspeak pre act)
   "Say what you completed."
-  (let ((prior (point ))
+  (let ((prior (save-excursion (skip-syntax-backward "^ >") (point)))
         (dtk-stop-immediately t))
     (emacspeak-kill-buffer-carefully "*Completions*")
     ad-do-it
@@ -123,25 +143,27 @@ Note that some badly formed mime messages  cause trouble."
 (defun emacspeak-vm-yank-header ()
   "Yank specified header into kill ring."
   (interactive)
-  (declare (special vm-message-pointer))
+  (cl-declare (special vm-message-pointer))
   (cond
    (vm-message-pointer
     (dtk-stop)
-    (let*  ((message (car vm-message-pointer ))
-            (from (vm-from-of message ))
-            (subject (vm-subject-of  message ))
-            (to (vm-to-of message ))
+    (let*  ((message (car vm-message-pointer))
+            (from (vm-from-of message))
+            (subject (vm-subject-of  message))
+            (to (vm-to-of message))
+            (url  (browse-url-url-at-point))
             (header nil))
       (while (not header)
         (setq header
-              (case
-                  (read-char "f From s Subject t To")
+              (cl-case (read-char "f From s Subject t To u URL")
                 (?s subject)
                 (?f from)
+                (?u url)
                 (?t to))))
-      (kill-new header)
-      (message header)))
-   (t (error "No current message." ))))
+      (when header (kill-new header))
+      (message "%s" header)))
+   (t (error "No current message."))))
+
 (defcustom emacspeak-vm-headers-strip-octals t
   "Specify whether non-ascii chars should be stripped when
   speaking email headers."
@@ -160,63 +182,62 @@ Note that some badly formed mime messages  cause trouble."
 
 (defun emacspeak-vm-summarize-message ()
   "Summarize the current vm message. "
-  (declare (special vm-message-pointer
-                    smtpmail-local-domain
-                    emacspeak-vm-headers-strip-octals
-                    emacspeak-vm-user-full-name emacspeak-vm-user-login-name))
+  (cl-declare (special vm-message-pointer smtpmail-local-domain
+                       vm-presentation-buffer  emacspeak-vm-headers-strip-octals))
   (when vm-message-pointer
-    (let*  ((message (car vm-message-pointer ))
+    (let*  ((message (car vm-message-pointer))
             (number (emacspeak-vm-number-of  message))
-            (from
-             (or (vm-su-full-name message) (vm-su-from message )))
-            (subject (vm-su-subject message ))
-            (to
-             (or (vm-su-to-names message) (vm-su-to message )))
+            (from (or (vm-su-full-name message) (vm-su-from message)))
+            (subject (vm-su-subject message))
+            (to (or (vm-su-to-names message) (vm-su-to message)))
             (self-p
-             (or (string-match emacspeak-vm-user-full-name to)
-                 (string-match  (user-login-name) to)
-                 (string-match (format "%s@%s" (user-login-name)
-                                       smtpmail-local-domain)
-                               to)))
+             (or (string-match user-mail-address to)
+                 (string-match (user-full-name) to)
+                 (string-match  (user-login-name) to)))
             (lines (vm-su-line-count message)))
-      (dtk-speak
-       (vm-decode-mime-encoded-words-in-string
-        (format "%s %s %s   %s %s "
-                number
-                (or from "")
-                (if subject (format "on %s" subject) "")
-                (if (and to (< (length to) 80))
-                    (format "to %s" to) "")
-                (if lines (format "%s lines" lines) ""))))
+      (with-current-buffer vm-presentation-buffer
+        (dtk-speak
+         (vm-decode-mime-encoded-words-in-string
+          (concat
+           number
+           (if from
+               (propertize from   'personality voice-brighten)
+             "")
+           (if subject
+               (propertize subject 'personality voice-lighten)
+             " ")
+           (if (and to (< (length to) 80))
+               (concat
+                (propertize " to " 'personality voice-smoothen)
+                (propertize  to 'personality voice-annotate))
+             "")
+           (if lines (format "%s lines" lines) "")))))
       (goto-char (point-min))
       (search-forward  (format "%c%c" 10 10) nil)
       (cond
        ((and self-p
-             (= 0 self-p)                    ) ;mail to me and others
-        (emacspeak-auditory-icon 'item))
-       (self-p                       ;mail to others including me
+             (= 0 self-p)) ;mail to me and others
+        (emacspeak-auditory-icon 'select-object))
+       (self-p                          ;mail to others including me
         (emacspeak-auditory-icon 'mark-object))
-       (t                       ;got it because of a mailing list
-        (emacspeak-auditory-icon 'select-object ))))))
+       (t                            ;got it because of a mailing list
+        (emacspeak-auditory-icon 'item))))))
 
 (defun emacspeak-vm-speak-labels ()
   "Speak a message's labels"
   (interactive)
-  (declare (special vm-message-pointer))
+  (cl-declare (special vm-message-pointer))
   (when vm-message-pointer
     (message "Labels: %s"
-             (vm-labels-of (car vm-message-pointer )))))
+             (vm-labels-of (car vm-message-pointer)))))
 
 (defun emacspeak-vm-mode-line ()
   "VM mode line information. "
   (interactive)
-  (declare (special vm-ml-message-attributes-alist
-                    vm-ml-message-read vm-ml-message-unread
-                    vm-virtual-folder-definition
-                    vm-ml-message-new
-                    vm-ml-message-number vm-ml-highest-message-number ))
-  (when (buffer-modified-p )
-    (dtk-tone 700 70))
+  (cl-declare (special vm-ml-message-attributes-alist
+                       vm-ml-message-read vm-ml-message-unread
+                       vm-virtual-folder-definition vm-ml-message-new
+                       vm-ml-message-number vm-ml-highest-message-number))
   (cond
    (vm-virtual-folder-definition
     (dtk-speak
@@ -230,22 +251,21 @@ Note that some badly formed mime messages  cause trouble."
                (if vm-ml-message-unread "unread" "")
                (if vm-ml-message-read "read" "")
                (mapconcat
-                (function (lambda(item)
-                            (let ((var (car item))
-                                  (value (cadr item )))
-                              (cond
-                               ((and (boundp var) (eval var ))
-                                (if (symbolp value)
-                                    (eval value)
-                                  value))
-                               (t "")))))
+                #'(lambda(item)
+                    (let ((var (car item))
+                          (value (cadr item)))
+                      (cond
+                       ((and (boundp var) (eval var))
+                        (if (symbolp value)
+                            (eval value)
+                          value))
+                       (t ""))))
                 (cdr vm-ml-message-attributes-alist)   " "))))))
 
 ;;}}}
 ;;{{{  Moving between messages
 
-(add-hook 'vm-select-message-hook
-          'emacspeak-vm-summarize-message)
+(add-hook 'vm-select-message-hook 'emacspeak-vm-summarize-message)
 
 ;;}}}
 ;;{{{  Scrolling messages:
@@ -255,66 +275,52 @@ Note that some badly formed mime messages  cause trouble."
 Useful when you're reading a message
 that has been forwarded multiple times."
   (interactive)
-  (re-search-forward "^ *Subject:" nil t )
+  (re-search-forward "^ *Subject:" nil t)
   (emacspeak-speak-line))
 
 (defadvice vm-scroll-forward (after emacspeak pre act)
   "Produce auditory feedback.
 Then speak the screenful. "
-  (when (ems-interactive-p )
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'scroll)
     (save-excursion
-      (let ((start  (point ))
-            (window (get-buffer-window (current-buffer ))))
+      (let ((start  (point))
+            (window (get-buffer-window (current-buffer))))
         (forward-line (window-height window))
-        (emacspeak-speak-region start (point ))))))
+        (emacspeak-speak-region start (point))))))
 
 (defadvice vm-scroll-backward (after emacspeak pre act)
   "Produce auditory feedback.
 Then speak the screenful. "
-  (when (ems-interactive-p )
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'scroll)
     (save-excursion
-      (let ((start  (point ))
-            (window (get-buffer-window (current-buffer ))))
+      (let ((start  (point))
+            (window (get-buffer-window (current-buffer))))
         (forward-line(-  (window-height window)))
-        (emacspeak-speak-region start (point ))))))
+        (emacspeak-speak-region start (point))))))
 (defun emacspeak-vm-browse-message ()
   "Browse an email message --read it paragraph at a time. "
   (interactive)
-  (emacspeak-execute-repeatedly 'forward-paragraph ))
+  (emacspeak-execute-repeatedly 'forward-paragraph))
 
-(declaim (special vm-mode-map))
-(eval-when (load)
-  (load-library "vm-vars")
-  )
-
-(declaim (special vm-mode-map))
-(define-key vm-mode-map "\M-\C-m" 'widget-button-press)
-(define-key vm-mode-map "y" 'emacspeak-vm-yank-header)
-(define-key vm-mode-map "\M-\t" 'emacspeak-vm-next-button)
-(define-key vm-mode-map  "j" 'emacspeak-hide-or-expose-all-blocks)
-(define-key vm-mode-map  "\M-g" 'vm-goto-message)
-(define-key vm-mode-map "J" 'vm-discard-cached-data)
-(define-key vm-mode-map "." 'emacspeak-vm-browse-message)
-(define-key vm-mode-map "'" 'emacspeak-speak-rest-of-buffer)
 ;;}}}
 ;;{{{  deleting and killing
 
 (defadvice vm-delete-message (after emacspeak pre act)
   "Provide auditory feedback."
-  (when (ems-interactive-p )
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'delete-object)
     (message "Message discarded.")))
 
 (defadvice vm-undelete-message (after emacspeak pre act)
   "Provide auditory feedback."
-  (when (ems-interactive-p )
+  (when (ems-interactive-p)
     (message "Message recovered.")))
 
 (defadvice vm-kill-subject (after emacspeak pre act)
   "Provide auditory feedback. "
-  (when (ems-interactive-p )
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'delete-object)
     (call-interactively 'vm-next-message)))
 
@@ -323,61 +329,62 @@ Then speak the screenful. "
 
 (defadvice vm-forward-message (around emacspeak pre act)
   "Provide aural feedback."
-  (if (ems-interactive-p )
+  (if (ems-interactive-p)
       (let ((dtk-stop-immediately nil))
         (message "Forwarding message")
         ad-do-it
-        (emacspeak-speak-line ))
+        (emacspeak-speak-line))
     ad-do-it)
-  ad-return-value )
+  ad-return-value)
 
 (defadvice vm-reply (after emacspeak pre act)
   "Provide aural feedback."
-  (when (ems-interactive-p )
+  (when (ems-interactive-p)
     (emacspeak-speak-mode-line)))
 
 (defadvice vm-followup (after emacspeak pre act)
   "Provide aural feedback."
-  (when (ems-interactive-p )
+  (when (ems-interactive-p)
     (message "Folluwing up")
     (emacspeak-speak-mode-line)))
 
 (defadvice vm-reply-include-text (after emacspeak pre act)
   "Provide aural feedback."
-  (when (ems-interactive-p )
-    (emacspeak-speak-mode-line )))
+  (when (ems-interactive-p)
+    (emacspeak-speak-mode-line)))
 
 (defadvice vm-followup-include-text (after emacspeak pre act)
   "Provide aural feedback."
-  (when (ems-interactive-p )
+  (when (ems-interactive-p)
     (message "Following up")
-    (emacspeak-speak-mode-line )))
+    (emacspeak-speak-mode-line)))
 (defadvice vm-mail-send (after emacspeak pre act comp)
   "Provide auditory context"
-  (when  (ems-interactive-p )
+  (when  (ems-interactive-p)
     (emacspeak-speak-mode-line)
     (emacspeak-auditory-icon 'close-object)))
 
 (defadvice vm-mail-send-and-exit (after emacspeak pre act comp)
   "Provide auditory context"
-  (when  (ems-interactive-p )
+  (when  (ems-interactive-p)
     (emacspeak-auditory-icon 'close-object)))
 
 (defadvice vm-mail (after emacspeak pre act)
   "Provide aural feedback."
-  (when (ems-interactive-p )
+  (when (ems-interactive-p)
     (let ((dtk-stop-immediately nil))
       (message "Composing a message")
-      (emacspeak-speak-line ))))
+      (emacspeak-speak-line))))
 
 ;;}}}
 ;;{{{ quitting
 
-(defadvice vm-quit (after emacspeak pre act )
+(defadvice vm-quit (after emacspeak pre act)
   "Provide an auditory icon if requested"
-  (when (ems-interactive-p )
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'close-object)
-    (emacspeak-speak-mode-line)))
+    (with-current-buffer (window-buffer (selected-window))
+      (emacspeak-speak-mode-line))))
 
 ;;}}}
 ;;{{{ catching up on folders
@@ -385,7 +392,7 @@ Then speak the screenful. "
 (defun emacspeak-vm-catch-up-all-messages ()
   "Mark all messages in folder to be deleted. Use with caution."
   (interactive)
-  (declare (special vm-ml-highest-message-number))
+  (cl-declare (special vm-ml-highest-message-number))
   (vm-goto-message 1)
   (vm-delete-message
    (read vm-ml-highest-message-number))
@@ -394,24 +401,32 @@ Then speak the screenful. "
 
 ;;}}}
 ;;{{{  Keybindings:
-(declaim  (special vm-mode-map
-                   global-map
-                   emacspeak-prefix
-                   emacspeak-keymap))
-(define-key vm-mode-map "\M-c" 'emacspeak-vm-catch-up-all-messages)
-(define-key vm-mode-map "\M-j" 'emacspeak-vm-locate-subject-line)
-(define-key vm-mode-map "," 'emacspeak-vm-speak-message)
-(define-key vm-mode-map "\M-l" 'emacspeak-vm-speak-labels)
-(define-key vm-mode-map
-  (concat emacspeak-prefix "m")
-  'emacspeak-vm-mode-line)
+(when (boundp 'vm-mode-map)
+  (cl-declaim  (special
+                vm-mode-map
+                global-map emacspeak-prefix emacspeak-keymap))
+  (define-key vm-mode-map "\M-\C-m" 'widget-button-press)
+  (define-key vm-mode-map "y" 'emacspeak-vm-yank-header)
+  (define-key vm-mode-map  "j" 'emacspeak-hide-or-expose-all-blocks)
+  (define-key vm-mode-map  "\M-g" 'vm-goto-message)
+  (define-key vm-mode-map "J" 'vm-discard-cached-data)
+  (define-key vm-mode-map "." 'emacspeak-vm-browse-message)
+  (define-key vm-mode-map "'" 'emacspeak-speak-rest-of-buffer)
+  (define-key vm-mode-map "\M-c" 'emacspeak-vm-catch-up-all-messages)
+  (define-key vm-mode-map "\M-j" 'emacspeak-vm-locate-subject-line)
+  (define-key vm-mode-map "," 'emacspeak-vm-speak-message)
+  (define-key vm-mode-map "\M-l" 'emacspeak-vm-speak-labels)
+  (define-key vm-mode-map
+    (concat emacspeak-prefix "m")
+    'emacspeak-vm-mode-line)
+  )
 ;;}}}
 ;;{{{ advise searching:
 (defadvice vm-isearch-forward (around emacspeak pre act comp)
   "Provide auditory feedback"
-  (declare (special vm-message-pointer))
+  (cl-declare (special vm-message-pointer))
   (cond
-   ((ems-interactive-p )
+   ((ems-interactive-p)
     (let ((orig (point)))
       ad-do-it
       (cond
@@ -424,9 +439,9 @@ Then speak the screenful. "
 
 (defadvice vm-isearch-backward (around emacspeak pre act comp)
   "Provide auditory feedback"
-  (declare (special vm-message-pointer))
+  (cl-declare (special vm-message-pointer))
   (cond
-   ((ems-interactive-p )
+   ((ems-interactive-p)
     (let ((orig (point)))
       ad-do-it
       (cond
@@ -441,19 +456,19 @@ Then speak the screenful. "
 ;;{{{  silence mime parsing in vm 6.0 and above
 
 (defadvice vm-mime-parse-entity (around emacspeak pre act comp)
-  (let ((emacspeak-speak-messages nil))
-    ad-do-it))
+  (ems-with-messages-silenced
+   ad-do-it))
 
 (defadvice vm-decode-mime-message (around emacspeak pre act comp)
-  (let ((emacspeak-speak-messages nil))
-    ad-do-it))
+  (ems-with-messages-silenced
+   ad-do-it))
 
 (defadvice vm-mime-run-display-function-at-point (around emacspeak pre act comp)
   "Provide auditory feedback.
 Leave point at front of decoded attachment."
   (cond
-   ((ems-interactive-p )
-    (let ((orig (point )))
+   ((ems-interactive-p)
+    (let ((orig (point)))
       ad-do-it
       (goto-char orig)
       (message "Decoded attachment")))
@@ -465,8 +480,8 @@ Leave point at front of decoded attachment."
 
 (defadvice vm-emit-eom-blurb (around emacspeak pre act comp)
   "Stop chattering"
-  (let ((emacspeak-speak-messages nil))
-    ad-do-it))
+  (ems-with-messages-silenced
+   ad-do-it))
 
 ;;}}}
 ;;{{{ advice password prompt
@@ -486,42 +501,38 @@ Leave point at front of decoded attachment."
 (add-hook 'vm-presentation-mode-hook
           #'(lambda nil
               (emacspeak-pronounce-refresh-pronunciations)))
-(declaim (special emacspeak-pronounce-internet-smileys-pronunciations))
-(loop for hook in
-      (list 'mail-mode-hook
-            'vm-presentation-mode-hook)
-      do
-      (add-hook hook 'emacspeak-pronounce-refresh-pronunciations
-                'append ))
 
-(loop for mode in
-      '(vm-presentation-mode
-        mail-mode)
-      do
-      (emacspeak-pronounce-augment-pronunciations mode
-                                                  emacspeak-pronounce-internet-smileys-pronunciations)
-      (emacspeak-pronounce-add-dictionary-entry mode
-                                                emacspeak-speak-embedded-url-pattern
-                                                (cons 're-search-forward
-                                                      #'(lambda
-                                                          (url) "
-      Link ")))
-      (emacspeak-pronounce-add-dictionary-entry mode
-                                                emacspeak-speak-rfc-3339-datetime-pattern
-                                                (cons 're-search-forward
-                                                      'emacspeak-speak-decode-rfc-3339-datetime))
-      (emacspeak-pronounce-add-dictionary-entry mode
-                                                emacspeak-speak-iso-datetime-pattern
-                                                (cons 're-search-forward
-                                                      'emacspeak-speak-decode-iso-datetime)))
+(cl-declaim (special emacspeak-pronounce-internet-smileys-pronunciations))
+(cl-loop
+ for hook in
+ '(mail-mode-hook vm-presentation-mode-hook)
+ do
+ (add-hook hook 'emacspeak-pronounce-refresh-pronunciations 'append))
+
+(cl-loop
+ for mode in
+ '(vm-presentation-mode mail-mode)
+ do
+ (emacspeak-pronounce-augment-pronunciations
+  mode
+  emacspeak-pronounce-internet-smileys-pronunciations)
+ (emacspeak-pronounce-add-dictionary-entry
+  mode
+  emacspeak-speak-embedded-url-pattern
+  (cons
+   #'re-search-forward
+   #'(lambda (_url) " Link ")))
+ (emacspeak-pronounce-add-dictionary-entry
+  mode
+  emacspeak-speak-rfc-3339-datetime-pattern
+  (cons #'re-search-forward #'emacspeak-speak-decode-rfc-3339-datetime))
+ (emacspeak-pronounce-add-dictionary-entry
+  mode
+  emacspeak-speak-iso-datetime-pattern
+  (cons #'re-search-forward #'emacspeak-speak-decode-iso-datetime)))
 
 ;;}}}
 ;;{{{ advice button motion
-(defadvice vm-next-button (after emacspeak pre act comp)
-  "Provide auditory feedback"
-  (when (ems-interactive-p )
-    (emacspeak-auditory-icon 'large-movement)
-    (emacspeak-speak-text-range  'w3-hyperlink-info)))
 
 ;;}}}
 ;;{{{  misc
@@ -533,38 +544,38 @@ Leave point at front of decoded attachment."
 ;;}}}
 ;;{{{  button motion in vm
 
-(defun emacspeak-vm-next-button (n)
-  "Move point to N buttons forward.
-If N is negative, move backward instead."
-  (interactive "p")
-  (let ((function (if (< n 0) 'previous-single-property-change
-                    'next-single-property-change))
-        (inhibit-point-motion-hooks t)
-        (backward (< n 0))
-        (limit (if (< n 0) (point-min) (point-max))))
-    (setq n (abs n))
-    (while (and (not (= limit (point)))
-                (> n 0))
-      ;; Skip past the current button.
-      (when (get-text-property (point) 'w3-hyperlink-info)
-        (goto-char (funcall function (point) 'w3-hyperlink-info nil limit)))
-      ;; Go to the next (or previous) button.
-      (goto-char (funcall function (point) 'w3-hyperlink-info nil limit))
-      ;; Put point at the start of the button.
-      (when (and backward (not (get-text-property (point) 'w3-hyperlink-info)))
-        (goto-char (funcall function (point) 'w3-hyperlink-info nil limit)))
-      ;; Skip past intangible buttons.
-      (when (get-text-property (point) 'intangible)
-        (incf n))
-      (decf n))
-    (unless (zerop n)
-      (message  "No more buttons"))
-    n))
-
 ;;}}}
 ;;{{{ saving mime attachment under point
 
 ;;}}}
+;;{{{ Voice Lock:
+
+(when (locate-library "u-vm-color")
+  (require 'u-vm-color)
+  (voice-setup-add-map
+   '(
+     (u-vm-color-author-face voice-brighten)
+     (u-vm-color-citation-1-face voice-smoothen)
+     (u-vm-color-citation-2-face voice-smoothen-extra)
+     (u-vm-color-date-face voice-monotone-medium)
+     (u-vm-color-header-face voice-bolden-medium)
+     (u-vm-color-recipient-face voice-annotate)
+     (u-vm-color-signature-face voice-smoothen)
+     (u-vm-color-spamassassin-face voice-monotone)
+     (u-vm-color-subject-face voice-lighten)
+     (u-vm-color-time-face voice-monotone)
+     (u-vm-color-user-face voice-animate)
+
+     )
+   )
+  (add-hook 'vm-showing-message-hook #'u-vm-color-fontify-buffer)
+  (add-hook 'vm-presentation-mode-hook #'u-vm-color-fontify-buffer)
+  (add-hook 'vm-summary-mode-hook #'u-vm-color-summary-mode)
+  (add-hook 'vm-select-message-hook #'u-vm-color-fontify-buffer)
+  )
+
+;;}}}
+
 ;;{{{ configure and customize vm
 
 ;;; This is how I customize VM
@@ -573,30 +584,44 @@ If N is negative, move backward instead."
   "Should VM  use the customizations used by the author of Emacspeak."
   :type 'boolean
   :group 'emacspeak-vm)
+(defvar emacspeak-vm-demote-html-attachments
+  '(
+    favorite-internal  "text/plain" "text/enriched"
+    "text/html" "application/xml+xhtml")
+  "Setting that prefers text/plain alternatives over html/xhtml.")
+
+(defvar emacspeak-vm-promote-html-attachments
+  '(
+    favorite-internal   "text/html" "application/xml+xhtml"
+    "text/plain" "text/enriched")
+  "Setting that prefers  alternatives  html/xhtml over text/plain.")
 
 (defun emacspeak-vm-use-raman-settings ()
   "Customization settings for VM used by the author of
 Emacspeak."
-  (declare (special
-            vm-mime-charset-converter-alist
-            vm-mime-default-face-charsets
-            vm-frame-per-folder
-            vm-frame-per-composition
-            vm-frame-per-edit
-            vm-frame-per-help
-            vm-frame-per-summary
-            vm-index-file-suffix
-            vm-primary-inbox
-            vm-folder-directory
-            vm-forwarding-subject-format
-            vm-startup-with-summary
-            vm-inhibit-startup-message
-            vm-visible-headers
-            vm-delete-after-saving
-            vm-url-browser
-            vm-confirm-new-folders
-            vm-move-after-deleting
-            emacspeak-vm-voice-lock-messages))
+  (cl-declare (special emacspeak-vm-demote-html-attachments
+                       emacspeak-vm-promote-html-attachments
+                       vm-mime-charset-converter-alist
+                       vm-mime-default-face-charsets
+                       vm-frame-per-folder
+                       vm-frame-per-composition
+                       vm-frame-per-edit
+                       vm-frame-per-help
+                       vm-frame-per-summary
+                       vm-index-file-suffix
+                       vm-primary-inbox
+                       vm-folder-directory
+                       vm-forwarding-subject-format
+                       vm-startup-with-summary
+                       vm-inhibit-startup-message
+                       vm-visible-headers
+                       vm-delete-after-saving
+                       vm-url-browser
+                       vm-confirm-new-folders
+                       vm-mime-alternative-select-method
+                       vm-move-after-deleting))
+  (setq vm-mime-alternative-select-method
+        emacspeak-vm-demote-html-attachments)
   (setq vm-mime-charset-converter-alist
         '(
           ("utf-8" "iso-8859-1" "iconv -f utf-8 -t iso-8859-1")
@@ -619,10 +644,28 @@ Emacspeak."
         vm-delete-after-saving t
         vm-url-browser 'browse-url
         vm-confirm-new-folders t
-        vm-move-after-deleting nil
-        emacspeak-vm-voice-lock-messages t)
+        vm-move-after-deleting nil)
   t)
 
+(defun emacspeak-vm-toggle-html-mime-demotion ()
+  "Toggle state of HTML Mime Demotion."
+  (interactive)
+  (cl-declare (special emacspeak-vm-demote-html-attachments
+                       emacspeak-vm-promote-html-attachments
+                       vm-mime-alternative-select-method))
+  (cond
+   ((eq vm-mime-alternative-select-method emacspeak-vm-demote-html-attachments)
+    (setq vm-mime-alternative-select-method
+          emacspeak-vm-promote-html-attachments)
+    (message "Prefering HTML Mime alternative."))
+   ((eq vm-mime-alternative-select-method emacspeak-vm-promote-html-attachments)
+    (setq vm-mime-alternative-select-method
+          emacspeak-vm-demote-html-attachments)
+    (message "Prefering Text/Plain Mime alternative."))
+   (t (message "Resetting state to HTML Mime demotion.")
+      (setq vm-mime-alternative-select-method
+            emacspeak-vm-demote-html-attachments))))
+
 (when emacspeak-vm-use-raman-settings
   (emacspeak-vm-use-raman-settings))
 
@@ -666,57 +709,58 @@ text using wvText."
   :type 'string
   :group 'emacspeak-vm)
 
-(defsubst emacspeak-vm-add-mime-convertor (convertor)
-  "Helper to add a convertor specification."
-  (declare (special vm-mime-type-converter-alist))
+(defun emacspeak-vm-add-mime-converter (converter)
+  "Helper to add a converter specification."
+  (cl-declare (special vm-mime-type-converter-alist))
   (unless
-      (find-if
+      (cl-find-if
        #'(lambda  (i)
-           (string-equal (car i) (car convertor)))
+           (string-equal (car i) (car converter)))
        vm-mime-type-converter-alist)
-    (push   convertor
+    (push   converter
             vm-mime-type-converter-alist)))
 
 (defun emacspeak-vm-customize-mime-settings ()
   "Customize VM mime settings."
-  (declare (special vm-preview-lines
-                    vm-infer-mime-types
-                    vm-mime-decode-for-preview
-                    vm-auto-decode-mime-messages
-                    vm-auto-displayed-mime-content-type-exceptions
-                    vm-mime-attachment-save-directory
-                    vm-mime-base64-encoder-program
-                    vm-mime-base64-decoder-program
-                    vm-mime-attachment-auto-type-alist
-                    vm-mime-type-converter-alist
-                    emacspeak-vm-pdf2text
-                    emacspeak-vm-ppt2html
-                    emacspeak-vm-xls2html
-                    emacspeak-vm-doc2text
-                    emacspeak-vm-cal2text))
-  (emacspeak-vm-add-mime-convertor
+  (cl-declare (special vm-preview-lines
+                       vm-infer-mime-types
+                       vm-mime-decode-for-preview
+                       vm-auto-decode-mime-messages
+                       vm-auto-displayed-mime-content-type-exceptions
+                       vm-mime-attachment-save-directory
+                       vm-mime-base64-encoder-program
+                       vm-mime-base64-decoder-program
+                       vm-mime-attachment-auto-type-alist
+                       vm-mime-type-converter-alist
+                       emacspeak-vm-pdf2text
+                       emacspeak-vm-ppt2html
+                       emacspeak-vm-xls2html
+                       emacspeak-vm-doc2text
+                       emacspeak-vm-cal2text))
+  (emacspeak-vm-add-mime-converter
    (list "text/calendar" "text/plain" emacspeak-vm-cal2text))
-  (emacspeak-vm-add-mime-convertor
+  (emacspeak-vm-add-mime-converter
    (list "application/pdf" "text/plain"
          emacspeak-vm-pdf2text))
-  (emacspeak-vm-add-mime-convertor
+  (emacspeak-vm-add-mime-converter
    (list "application/vnd.ms-excel" "text/html"
          emacspeak-vm-xls2html))
-  (emacspeak-vm-add-mime-convertor
+  (emacspeak-vm-add-mime-converter
    (list "application/vnd.ms-powerpoint" "text/html" emacspeak-vm-ppt2html))
-  (emacspeak-vm-add-mime-convertor
+  (emacspeak-vm-add-mime-converter
    (list "application/msword" "text/plain" emacspeak-vm-doc2text))
   (setq vm-preview-lines nil
         vm-infer-mime-types t
         vm-mime-decode-for-preview nil
         vm-auto-decode-mime-messages t
         vm-auto-displayed-mime-content-type-exceptions '("text/html")
-        vm-mime-attachment-save-directory (expand-file-name "~/Mail/attachments/")
+        vm-mime-attachment-save-directory
+        (expand-file-name "~/Mail/attachments/")
         vm-mime-base64-encoder-program "base64-encode"
         vm-mime-base64-decoder-program "base64-decode")
   t)
 
-(when emacspeak-vm-customize-mime-settings
+(when (and (featurep 'vm)emacspeak-vm-customize-mime-settings)
   (emacspeak-vm-customize-mime-settings))
 
 ;;}}}
@@ -725,7 +769,7 @@ text using wvText."
 
 ;;; local variables:
 ;;; folded-file: t
-;;; byte-compile-dynamic: nil
+;;; byte-compile-dynamic: t
 ;;; end:
 
 ;;}}}
