@@ -1,5 +1,5 @@
-;;; emacspeak-we.el --- Transform Web Pages Using XSLT
-;;; $Id: emacspeak-we.el 7484 2011-12-20 16:48:38Z tv.raman.tv $
+;;; emacspeak-we.el --- Transform Web Pages Using XSLT  -*- lexical-binding: t; -*-
+;;; $Id$
 ;;; $Author: tv.raman.tv $
 ;;; Description:  Edit/Transform Web Pages using XSLT
 ;;; Keywords: Emacspeak,  Audio Desktop Web, XSLT
@@ -15,7 +15,7 @@
 
 ;;}}}
 ;;{{{  Copyright:
-;;;Copyright (C) 1995 -- 2011, T. V. Raman
+;;;Copyright (C) 1995 -- 2018, T. V. Raman
 ;;; Copyright (c) 1994, 1995 by Digital Equipment Corporation.
 ;;; All Rights Reserved.
 ;;;
@@ -45,7 +45,7 @@
 ;;; Invoke XSLT to edit/transform Web pages before they get
 ;;; rendered.
 ;;; we makes emacspeak's webedit layer independent of a given
-;;; Emacs web browser like W3 or W3M
+;;; Emacs web browser  EWW
 ;;; This module will use the abstraction provided by browse-url
 ;;; to handle Web pages.
 ;;; Module emacspeak-webutils provides the needed additional
@@ -56,8 +56,8 @@
 ;;}}}
 ;;{{{  Required modules
 
-(require 'cl)
-(declaim  (optimize  (safety 0) (speed 3)))
+(require 'cl-lib)
+(cl-declaim  (optimize  (safety 0) (speed 3)))
 (require 'emacspeak-preamble)
 (require 'emacspeak-xslt)
 (require 'emacspeak-webutils)
@@ -77,7 +77,7 @@ useful in jumping directly to the printer friendly version of an
 article for example.  Optional interactive prefix arg prompts for
 a rewrite rule even if one is already defined."
   (interactive "P")
-  (declare (special emacspeak-we-url-rewrite-rule))
+  (cl-declare (special emacspeak-we-url-rewrite-rule))
   (emacspeak-webutils-browser-check)
   (let ((url (funcall emacspeak-webutils-url-at-point))
         (redirect nil))
@@ -87,8 +87,8 @@ a rewrite rule even if one is already defined."
             (read-minibuffer  "Specify rewrite rule: " "(")))
     (setq redirect
           (replace-regexp-in-string
-           (first emacspeak-we-url-rewrite-rule)
-           (second emacspeak-we-url-rewrite-rule)
+           (cl-first emacspeak-we-url-rewrite-rule)
+           (cl-second emacspeak-we-url-rewrite-rule)
            url))
     (emacspeak-auditory-icon 'select-object)
     (browse-url (or redirect url))))
@@ -101,24 +101,26 @@ a rewrite rule even if one is already defined."
 
 (make-variable-buffer-local 'emacspeak-we-url-executor)
 
-(defun emacspeak-we-url-expand-and-execute ()
+(defun emacspeak-we-url-expand-and-execute (&optional prefix)
   "Applies buffer-specific URL expander/executor function."
-  (interactive)
-  (declare (special emacspeak-we-url-executor))
+  (interactive "P")
+  (cl-declare (special emacspeak-we-url-executor))
   (emacspeak-webutils-browser-check)
   (let ((url (funcall emacspeak-webutils-url-at-point)))
     (unless url (error "Not on a link."))
     (cond
      ((and (boundp 'emacspeak-we-url-executor)
            (fboundp emacspeak-we-url-executor))
-      (funcall emacspeak-we-url-executor url))
+      (if prefix
+          (funcall emacspeak-we-url-executor url prefix)
+        (funcall emacspeak-we-url-executor url)))
      (t
       (setq emacspeak-we-url-executor
             (intern
              (completing-read
               "Executor function: "
               obarray 'fboundp t
-              "emacspeak-" nil )))
+              "emacspeak-" nil)))
       (if (and (boundp 'emacspeak-we-url-executor)
                (fboundp emacspeak-we-url-executor))
           (funcall emacspeak-we-url-executor url)
@@ -128,7 +130,7 @@ a rewrite rule even if one is already defined."
 ;;}}}
 ;;{{{ applying XSL transforms before displaying
 
-(define-prefix-command 'emacspeak-we-xsl-map )
+(define-prefix-command 'emacspeak-we-xsl-map)
 
 (defvar emacspeak-we-xsl-filter
   (emacspeak-xslt-get "xpath-filter.xsl")
@@ -147,10 +149,10 @@ a rewrite rule even if one is already defined."
   :type 'boolean
   :group 'emacspeak-we)
 
-;;;###autoload
-(defcustom emacspeak-we-xsl-transform nil
+(defcustom emacspeak-we-xsl-transform
+  (emacspeak-xslt-get "sort-tables.xsl")
   "Specifies transform to use before displaying a page.
-Nil means no transform is used. "
+Default is to apply sort-tables."
   :type  '(choice
            (file :tag "XSL")
            (const :tag "none" nil))
@@ -176,19 +178,18 @@ Nil means no transform is used. "
   "Apply specified transformation to current Web page."
   (interactive (list (emacspeak-xslt-read)))
   (emacspeak-webutils-browser-check)
-  (emacspeak-webutils-with-xsl-environment
-   xsl
-   nil
-   emacspeak-xslt-options
-   (browse-url (funcall emacspeak-webutils-current-url))))
+  (add-hook
+   'emacspeak-web-pre-process-hook
+   (emacspeak-webutils-make-xsl-transformer  xsl))
+  (browse-url (funcall emacspeak-webutils-current-url)))
 
 ;;;###autoload
 (defun emacspeak-we-xslt-select (xsl)
   "Select XSL transformation applied to Web pages before they are displayed ."
   (interactive (list (emacspeak-xslt-read)))
-  (declare (special emacspeak-we-xsl-transform))
+  (cl-declare (special emacspeak-we-xsl-transform))
   (setq emacspeak-we-xsl-transform xsl)
-  (when (interactive-p)
+  (when (called-interactively-p 'interactive)
     (emacspeak-auditory-icon 'select-object)
     (message "Will apply %s before displaying HTML pages."
              (file-name-sans-extension
@@ -198,9 +199,9 @@ Nil means no transform is used. "
 (defun emacspeak-we-xsl-toggle ()
   "Toggle  application of XSL transformations."
   (interactive)
-  (declare (special emacspeak-we-xsl-p))
+  (cl-declare (special emacspeak-we-xsl-p))
   (setq emacspeak-we-xsl-p (not emacspeak-we-xsl-p))
-  (when (interactive-p)
+  (when (called-interactively-p 'interactive)
     (emacspeak-auditory-icon
      (if emacspeak-we-xsl-p 'on 'off))
     (message "Turned %s XSL"
@@ -240,13 +241,11 @@ Nil means no transform is used. "
 (defun emacspeak-we-toggle-xsl-keep-result ()
   "Toggle xsl keep result flag."
   (interactive)
-  (declare (special emacspeak-we-xsl-keep-result))
+  (cl-declare (special emacspeak-we-xsl-keep-result))
   (setq emacspeak-we-xsl-keep-result
         (not emacspeak-we-xsl-keep-result))
-  (when (interactive-p)
-    (emacspeak-auditory-icon
-     (if emacspeak-we-xsl-keep-result
-         'on 'off))
+  (when (called-interactively-p 'interactive)
+    (emacspeak-auditory-icon (if emacspeak-we-xsl-keep-result 'on 'off))
     (message "Turned %s xslt keep results."
              (if emacspeak-we-xsl-keep-result
                  'on 'off))))
@@ -254,9 +253,9 @@ Nil means no transform is used. "
   "Set to T  if you want the buffer name to contain the applied filter."
   :type  'boolean
   :group 'emacspeak-we)
-
+(declare-function emacspeak-eww-reading-settings "emacspeak-eww")
 ;;;###autoload
-(defun emacspeak-we-xslt-filter (path    url  &optional speak)
+(defun emacspeak-we-xslt-filter (path    url  &optional _speak)
   "Extract elements matching specified XPath path locator
 from Web page -- default is the current page being viewed."
   (interactive
@@ -264,16 +263,32 @@ from Web page -- default is the current page being viewed."
     (read-from-minibuffer "XPath: ")
     (emacspeak-webutils-read-url)
     current-prefix-arg))
-  (declare (special emacspeak-we-xsl-filter
-                    emacspeak-we-filters-rename-buffer))
+  (cl-declare (special emacspeak-we-xsl-filter
+                       emacspeak-we-filters-rename-buffer))
   (let ((params (emacspeak-xslt-params-from-xpath  path url)))
     (when emacspeak-we-filters-rename-buffer(emacspeak-webutils-rename-buffer (format "Filtered %s" path)))
-    (when speak (emacspeak-webutils-autospeak))
-    (emacspeak-webutils-with-xsl-environment
-     emacspeak-we-xsl-filter
-     params
-     emacspeak-xslt-options             ;options
-     (browse-url url))))
+    (add-hook
+     'emacspeak-web-pre-process-hook
+     (emacspeak-webutils-make-xsl-transformer emacspeak-we-xsl-filter params))
+    (add-hook
+     'emacspeak-web-post-process-hook
+     #'emacspeak-eww-reading-settings 'at-end)
+    (browse-url url)))
+
+(defun emacspeak-we-xslt-pipeline-filter (specs    url  &optional _speak)
+  "Apply a pipeline of filters specified in `specs', a list.
+Each filter is a list of the form
+ `(xsl-stylesheet-name xpath)'."
+  (cl-declare (special emacspeak-we-filters-rename-buffer))
+  (when emacspeak-we-filters-rename-buffer
+    (emacspeak-webutils-rename-buffer (format "Pipeline filtered ")))
+  (add-hook
+   'emacspeak-web-pre-process-hook
+   (emacspeak-webutils-make-xsl-transformer-pipeline specs url))
+  (add-hook
+   'emacspeak-web-post-process-hook
+   #'emacspeak-eww-reading-settings 'at-end)
+  (browse-url url))
 
 ;;;###autoload
 (defun emacspeak-we-xslt-junk (path    url &optional speak)
@@ -282,17 +297,15 @@ from Web page -- default is the current page being viewed."
    (list
     (read-from-minibuffer "XPath: ")
     (emacspeak-webutils-read-url)
-    (interactive-p)))
-  (declare (special emacspeak-we-xsl-junk ))
+    (called-interactively-p 'interactive)))
+  (cl-declare (special emacspeak-we-xsl-junk))
   (let ((params (emacspeak-xslt-params-from-xpath  path url)))
-    (emacspeak-webutils-rename-buffer
-     (format "Filtered %s" path))
+    (emacspeak-webutils-rename-buffer (format "Filtered %s" path))
     (when speak (emacspeak-webutils-autospeak))
-    (emacspeak-webutils-with-xsl-environment
-     emacspeak-we-xsl-junk
-     params
-     emacspeak-xslt-options
-     (browse-url url))))
+    (add-hook
+     'emacspeak-web-pre-process-hook
+     (emacspeak-webutils-make-xsl-transformer emacspeak-we-xsl-junk params))
+    (browse-url url)))
 
 (defcustom emacspeak-we-media-stream-suffixes
   (list
@@ -323,8 +336,8 @@ operate on current web page when in a browser buffer; otherwise
   (interactive
    (list
     (emacspeak-webutils-read-url)
-    (interactive-p)))
-  (declare (special emacspeak-we-media-stream-suffixes))
+    (called-interactively-p 'interactive)))
+  (cl-declare (special emacspeak-we-media-stream-suffixes))
   (let ((filter "//a[%s]")
         (predicate
          (mapconcat
@@ -334,7 +347,7 @@ operate on current web page when in a browser buffer; otherwise
           emacspeak-we-media-stream-suffixes
           " or ")))
     (emacspeak-we-xslt-filter
-     (format filter predicate )
+     (format filter predicate)
      url speak)))
 
 ;;;###autoload
@@ -346,10 +359,19 @@ operate on current web page when in a browser buffer; otherwise
   (interactive
    (list
     (emacspeak-webutils-read-url)
-    (interactive-p)))
+    (called-interactively-p 'interactive)))
   (let ((filter "//a[contains(@href,\"print\")]"))
     (emacspeak-we-xslt-filter filter url speak)))
 
+;;;###autoload
+(defun emacspeak-we-follow-and-extract-main (&optional speak)
+  "Follow URL, then extract role=main."
+  (interactive
+   (list
+    (called-interactively-p 'interactive)))
+  (emacspeak-we-extract-by-role "main"
+                                (funcall emacspeak-webutils-url-at-point) speak))
+
 ;;;###autoload
 (defun emacspeak-we-extract-media-streams-under-point ()
   "In browser buffers, extract media streams from url under point."
@@ -366,7 +388,7 @@ operate on current web page when in a browser buffer; otherwise
    (list
     (read-from-minibuffer "Pattern: ")
     (emacspeak-webutils-read-url)
-    (interactive-p)))
+    (called-interactively-p 'interactive)))
   (let ((filter
          (format
           "//a[contains(@href,\"%s\")]"
@@ -383,12 +405,12 @@ spoken automatically."
    (list
     (read-from-minibuffer "Table Index: ")
     (emacspeak-webutils-read-url)
-    (interactive-p)))
+    (called-interactively-p 'interactive)))
   (emacspeak-we-xslt-filter
    (format "(//table//table)[%s]" index)
    url speak))
 
-(defsubst  emacspeak-we-get-table-list (&optional bound)
+(defun  emacspeak-we-get-table-list (&optional bound)
   "Collect a list of numbers less than bound
  by prompting repeatedly in the
 minibuffer.
@@ -408,7 +430,7 @@ Empty value finishes the list."
         (setq done t)))
     result))
 
-(defsubst  emacspeak-we-get-table-match-list ()
+(defun  emacspeak-we-get-table-match-list ()
   "Collect a list of matches by prompting repeatedly in the
 minibuffer.
 Empty value finishes the list."
@@ -430,7 +452,7 @@ Empty value finishes the list."
    (list
     (emacspeak-we-get-table-list)
     (emacspeak-webutils-read-url)
-    (interactive-p)))
+    (called-interactively-p 'interactive)))
   (let ((filter
          (mapconcat
           #'(lambda  (i)
@@ -440,9 +462,9 @@ Empty value finishes the list."
     (emacspeak-we-xslt-filter filter url speak)))
 
 ;;;###autoload
-(defun emacspeak-we-extract-table-by-position (position   url
-                                                          &optional speak)
-  "Extract table at specified position.
+(defun emacspeak-we-extract-table-by-position (pos   url
+                                                     &optional speak)
+  "Extract table at specified pos.
 Default is to extract from current page."
   (interactive
    (list
@@ -451,7 +473,7 @@ Default is to extract from current page."
     current-prefix-arg))
   (emacspeak-we-xslt-filter
    (format "/descendant::table[%s]"
-           position)
+           pos)
    url
    speak))
 
@@ -474,7 +496,7 @@ Tables are specified by their position in the list
     (emacspeak-we-xslt-filter
      filter
      url
-     (or (interactive-p)
+     (or (called-interactively-p 'interactive)
          speak))))
 
 ;;;###autoload
@@ -490,7 +512,7 @@ Tables are specified by their position in the list
    (format "(/descendant::table[contains(., \"%s\")])[last()]"
            match)
    url
-   (or (interactive-p)
+   (or (called-interactively-p 'interactive)
        speak)))
 
 ;;;###autoload
@@ -513,7 +535,7 @@ Tables are specified by containing  match pattern
     (emacspeak-we-xslt-filter
      filter
      url
-     (or (interactive-p)
+     (or (called-interactively-p 'interactive)
          speak))))
 
 (defvar emacspeak-we-buffer-class-cache nil
@@ -521,7 +543,7 @@ Tables are specified by containing  match pattern
 
 (make-variable-buffer-local 'emacspeak-we-buffer-class-cache)
 
-(defsubst emacspeak-we-build-class-cache ()
+(defun emacspeak-we-build-class-cache ()
   "Build class cache and forward it to rendered page."
   (let ((values nil)
         (content (clone-buffer)))
@@ -538,11 +560,10 @@ Tables are specified by containing  match pattern
     (add-hook
      'emacspeak-web-post-process-hook
      (eval
-      `(function
-        (lambda nil
-          (declare (special  emacspeak-we-buffer-class-cache))
-          (setq emacspeak-we-buffer-class-cache
-                ',(copy-sequence values))))))
+      `#'(lambda nil
+           (cl-declare (special  emacspeak-we-buffer-class-cache))
+           (setq emacspeak-we-buffer-class-cache
+                 ',(copy-sequence values)))))
     (kill-buffer content)))
 
 (defvar emacspeak-we-buffer-id-cache nil
@@ -550,7 +571,7 @@ Tables are specified by containing  match pattern
 
 (make-variable-buffer-local 'emacspeak-we-buffer-id-cache)
 
-(defsubst emacspeak-we-build-id-cache ()
+(defun emacspeak-we-build-id-cache ()
   "Build id cache and forward it to rendered page."
   (let ((values nil)
         (content (clone-buffer)))
@@ -564,11 +585,10 @@ Tables are specified by containing  match pattern
     (add-hook
      'emacspeak-web-post-process-hook
      (eval
-      `(function
-        (lambda nil
-          (declare (special  emacspeak-we-buffer-id-cache))
-          (setq emacspeak-we-buffer-id-cache
-                ',(copy-sequence values))))))
+      `#'(lambda nil
+           (cl-declare (special  emacspeak-we-buffer-id-cache))
+           (setq emacspeak-we-buffer-id-cache
+                 ',(copy-sequence values)))))
     (kill-buffer content)))
 
 (defvar emacspeak-we-buffer-role-cache nil
@@ -576,7 +596,7 @@ Tables are specified by containing  match pattern
 
 (make-variable-buffer-local 'emacspeak-we-buffer-role-cache)
 
-(defsubst emacspeak-we-build-role-cache ()
+(defun emacspeak-we-build-role-cache ()
   "Build role cache and forward it to rendered page."
   (let ((values nil)
         (content (clone-buffer)))
@@ -590,11 +610,10 @@ Tables are specified by containing  match pattern
     (add-hook
      'emacspeak-web-post-process-hook
      (eval
-      `(function
-        (lambda nil
-          (declare (special  emacspeak-we-buffer-role-cache))
-          (setq emacspeak-we-buffer-role-cache
-                ',(copy-sequence values))))))
+      `#'(lambda nil
+           (cl-declare (special  emacspeak-we-buffer-role-cache))
+           (setq emacspeak-we-buffer-role-cache
+                 ',(copy-sequence values)))))
     (kill-buffer content)))
 
 ;;;###autoload
@@ -611,8 +630,25 @@ buffer. Interactive use provides list of class values as completion."
   (let ((filter (format "//*[contains(@class,\"%s\")]" class)))
     (emacspeak-we-xslt-filter filter
                               url
-                              (or (interactive-p)
+                              (or (called-interactively-p 'interactive)
                                   speak))))
+
+(defun emacspeak-we-extract-by-role (role    url &optional speak)
+  "Extract elements having specified role attribute from HTML. Extracts
+specified elements from current WWW page and displays it in a separate
+buffer. Interactive use provides list of role values as completion."
+  (interactive
+   (list
+    (completing-read "Role: "
+                     emacspeak-we-buffer-role-cache)
+    (emacspeak-webutils-read-url)
+    current-prefix-arg))
+  (let ((filter (format "//*[contains(@role,\"%s\")]" role)))
+    (emacspeak-we-xslt-filter filter
+                              url
+                              (or (called-interactively-p 'interactive)
+                                  speak))))
+
 ;;;###autoload
 (defun emacspeak-we-junk-by-class (class    url &optional speak)
   "Extract elements not having specified class attribute from HTML. Extracts
@@ -627,10 +663,10 @@ buffer. Interactive use provides list of class values as completion."
   (let ((filter (format "//*[contains(@class,\"%s\")]" class)))
     (emacspeak-we-xslt-junk filter
                             url
-                            (or (interactive-p)
+                            (or (called-interactively-p 'interactive)
                                 speak))))
 
-(defsubst  emacspeak-we-get-id-list ()
+(defun  emacspeak-we-get-id-list ()
   "Collect a list of ids by prompting repeatedly in the
 minibuffer.
 Empty value finishes the list."
@@ -648,7 +684,7 @@ Empty value finishes the list."
         (setq done t)))
     result))
 
-(defsubst  emacspeak-we-css-get-class-list ()
+(defun  emacspeak-we-css-get-class-list ()
   "Collect a list of classes by prompting repeatedly in the
 minibuffer.
 Empty value finishes the list."
@@ -688,7 +724,7 @@ values as completion. "
     (emacspeak-we-xslt-filter
      (format "//*[%s]" filter)
      url
-     (or (interactive-p) speak))))
+     (or (called-interactively-p 'interactive) speak))))
 ;;;###autoload
 (defun emacspeak-we-junk-by-class-list(classes   url &optional
                                                  speak)
@@ -711,7 +747,7 @@ values as completion. "
     (emacspeak-we-xslt-junk
      (format "//*[%s]" filter)
      url
-     (or (interactive-p) speak))))
+     (or (called-interactively-p 'interactive) speak))))
 
 ;;;###autoload
 (defun emacspeak-we-extract-by-id (id   url &optional speak)
@@ -751,7 +787,7 @@ separate buffer. Interactive use provides list of id values as completion. "
     (emacspeak-we-xslt-filter
      (format "//*[%s]" filter)
      url
-     (or (interactive-p)
+     (or (called-interactively-p 'interactive)
          speak))))
 
 ;;;###autoload
@@ -792,7 +828,7 @@ separate buffer. Interactive use provides list of id values as completion. "
     (emacspeak-we-xslt-filter
      (format "//*[%s]//text()" filter)
      url
-     (or (interactive-p)
+     (or (called-interactively-p 'interactive)
          speak))))
 
 ;;;###autoload
@@ -807,28 +843,29 @@ separate buffer. Interactive use provides list of id values as completion. "
 (make-variable-buffer-local 'emacspeak-we-class-filter)
 
 ;;;###autoload
-(defun emacspeak-we-class-filter-and-follow (class url)
+(defun emacspeak-we-class-filter-and-follow (class url &optional _prompt)
   "Follow url and point, and filter the result by specified class.
 Class can be set locally for a buffer, and overridden with an
 interactive prefix arg. If there is a known rewrite url rule, that is
 used as well."
   (interactive
    (list
-    (or emacspeak-we-class-filter
-        (setq emacspeak-we-class-filter
-              (read-from-minibuffer "Class: ")))
-    (emacspeak-webutils-read-this-url)))
-  (declare (special emacspeak-we-class-filter
-                    emacspeak-we-url-rewrite-rule))
+    (cond
+     ((and (not current-prefix-arg)emacspeak-we-class-filter) emacspeak-we-class-filter)
+     (t (setq emacspeak-we-class-filter
+              (read-from-minibuffer "Class: "))))
+    (emacspeak-webutils-read-this-url)
+    current-prefix-arg))
+  (cl-declare (special emacspeak-we-class-filter emacspeak-we-url-rewrite-rule))
   (let ((redirect nil))
     (when emacspeak-we-url-rewrite-rule
       (setq redirect
             (replace-regexp-in-string
-             (first emacspeak-we-url-rewrite-rule)
-             (second emacspeak-we-url-rewrite-rule)
+             (cl-first emacspeak-we-url-rewrite-rule)
+             (cl-second emacspeak-we-url-rewrite-rule)
              url)))
     (emacspeak-we-extract-by-class
-     emacspeak-we-class-filter
+     class
      (or redirect url)
      'speak)
     (emacspeak-auditory-icon 'open-object)))
@@ -839,18 +876,21 @@ used as well."
 (make-variable-buffer-local 'emacspeak-we-id-filter)
 
 ;;;###autoload
-(defun emacspeak-we-follow-and-filter-by-id (id)
+(defun emacspeak-we-follow-and-filter-by-id (id _prompt)
   "Follow url and point, and filter the result by specified id.
 Id can be set locally for a buffer, and overridden with an
 interactive prefix arg. If there is a known rewrite url rule, that is
 used as well."
   (interactive
    (list
-    (or emacspeak-we-id-filter
-        (setq emacspeak-we-id-filter
-              (read-from-minibuffer "Id: ")))))
-  (declare (special emacspeak-we-id-filter
-                    emacspeak-we-url-rewrite-rule))
+    (cond
+     ((and (not current-prefix-arg)emacspeak-we-id-filter) emacspeak-we-id-filter)
+     (t
+      (setq emacspeak-we-id-filter
+            (read-from-minibuffer "Id: "))))
+    current-prefix-arg))
+  (cl-declare (special emacspeak-we-id-filter
+                       emacspeak-we-url-rewrite-rule))
   (emacspeak-webutils-browser-check)
   (let ((url (funcall emacspeak-webutils-url-at-point))
         (redirect nil))
@@ -859,16 +899,16 @@ used as well."
     (when emacspeak-we-url-rewrite-rule
       (setq redirect
             (replace-regexp-in-string
-             (first emacspeak-we-url-rewrite-rule)
-             (second emacspeak-we-url-rewrite-rule)
+             (cl-first emacspeak-we-url-rewrite-rule)
+             (cl-second emacspeak-we-url-rewrite-rule)
              url)))
     (emacspeak-we-extract-by-id
-     emacspeak-we-id-filter
+     id
      (or redirect url)
      'speak)))
 
 ;;;###autoload
-(defun emacspeak-we-style-filter (style   url &optional speak )
+(defun emacspeak-we-style-filter (style   url &optional speak)
   "Extract elements matching specified style
 from HTML.  Extracts specified elements from current WWW
 page and displays it in a separate buffer.  Optional arg url
@@ -881,10 +921,14 @@ specifies the page to extract contents  from."
   (emacspeak-we-xslt-filter
    (format "//*[contains(@style,  \"%s\")]" style)
    url
-   (or (interactive-p) speak)))
+   (or (called-interactively-p 'interactive) speak)))
 
 ;;}}}
 ;;{{{ xpath  filter
+;;;###autoload
+(defvar emacspeak-we-recent-xpath-filter
+  "//p"
+  "Caches most recently used xpath filter.")
 
 (defvar emacspeak-we-xpath-filter-history 
   (list
@@ -900,19 +944,11 @@ specifies the page to extract contents  from."
 urls.")
 
 (make-variable-buffer-local 'emacspeak-we-xpath-filter)
+
 ;;;###autoload
-(defcustom emacspeak-we-recent-xpath-filter
-  "//p|//ol|//ul|//dl|//h1|//h2|//h3|//h4|//h5|//h6|//blockquote|//div"
-  "Caches most recently used xpath filter.
-Can be customized to set up initial default."
-  :type 'string
-  :group 'emacspeak-we)
-;;;###autoload
-(defcustom emacspeak-we-paragraphs-xpath-filter
+(defvar emacspeak-we-paragraphs-xpath-filter
   "//p"
-  "Filter paragraphs."
-  :type 'string
-  :group 'emacspeak-we)
+  "Filter paragraphs.")
 
 ;;;###autoload
 (defun emacspeak-we-xpath-filter-and-follow (&optional prompt)
@@ -921,9 +957,9 @@ XPath can be set locally for a buffer, and overridden with an
 interactive prefix arg. If there is a known rewrite url rule, that is
 used as well."
   (interactive "P")
-  (declare (special emacspeak-we-xpath-filter
-                    emacspeak-we-recent-xpath-filter emacspeak-we-xpath-filter-history
-                    emacspeak-we-url-rewrite-rule))
+  (cl-declare (special emacspeak-we-xpath-filter
+                       emacspeak-we-recent-xpath-filter emacspeak-we-xpath-filter-history
+                       emacspeak-we-url-rewrite-rule))
   (emacspeak-webutils-browser-check)
   (let ((url (funcall emacspeak-webutils-url-at-point))
         (redirect nil))
@@ -931,19 +967,19 @@ used as well."
     (when emacspeak-we-url-rewrite-rule
       (setq redirect
             (replace-regexp-in-string
-             (first emacspeak-we-url-rewrite-rule)
-             (second emacspeak-we-url-rewrite-rule)
+             (cl-first emacspeak-we-url-rewrite-rule)
+             (cl-second emacspeak-we-url-rewrite-rule)
              url)))
-    (when (or prompt (null emacspeak-we-xpath-filter))
+    (when (or prompt (null emacspeak-we-xpath-filter)
+              (= 0 (length emacspeak-we-xpath-filter)))
       (setq emacspeak-we-xpath-filter
             (read-from-minibuffer
              "Specify XPath: "
              nil nil nil
              'emacspeak-we-xpath-filter-history
              emacspeak-we-recent-xpath-filter))
-      (pushnew emacspeak-we-xpath-filter emacspeak-we-xpath-filter-history)
-      (setq emacspeak-we-recent-xpath-filter
-            emacspeak-we-xpath-filter))
+      (cl-pushnew emacspeak-we-xpath-filter emacspeak-we-xpath-filter-history :test #'string=)
+      (setq emacspeak-we-recent-xpath-filter emacspeak-we-xpath-filter))
     (emacspeak-we-xslt-filter emacspeak-we-xpath-filter
                               (or redirect url)
                               'speak)))
@@ -959,12 +995,10 @@ used as well."
 urls.")
 
 (make-variable-buffer-local 'emacspeak-we-class-filter)
-(defcustom emacspeak-we-recent-class-filter
+(defvar emacspeak-we-recent-class-filter
   nil
-  "Caches most recently used class filter.
-Can be customized to set up initial default."
-  :type 'string
-  :group 'emacspeak-we)
+  "Caches most recently used class filter.")
+
 ;;;###autoload
 (defun emacspeak-we-class-filter-and-follow-link (&optional prompt)
   "Follow url and point, and filter the result by specified class.
@@ -972,9 +1006,9 @@ Class can be set locally for a buffer, and overridden with an
 interactive prefix arg. If there is a known rewrite url rule, that is
 used as well."
   (interactive "P")
-  (declare (special emacspeak-we-class-filter
-                    emacspeak-we-recent-class-filter emacspeak-we-class-filter-history
-                    emacspeak-we-url-rewrite-rule))
+  (cl-declare (special emacspeak-we-class-filter
+                       emacspeak-we-recent-class-filter emacspeak-we-class-filter-history
+                       emacspeak-we-url-rewrite-rule))
   (emacspeak-webutils-browser-check)
   (let ((url (funcall emacspeak-webutils-url-at-point))
         (redirect nil))
@@ -982,8 +1016,8 @@ used as well."
     (when emacspeak-we-url-rewrite-rule
       (setq redirect
             (replace-regexp-in-string
-             (first emacspeak-we-url-rewrite-rule)
-             (second emacspeak-we-url-rewrite-rule)
+             (cl-first emacspeak-we-url-rewrite-rule)
+             (cl-second emacspeak-we-url-rewrite-rule)
              url)))
     (when (or prompt (null emacspeak-we-class-filter))
       (setq emacspeak-we-class-filter
@@ -992,7 +1026,7 @@ used as well."
              nil nil nil
              'emacspeak-we-class-filter-history
              emacspeak-we-recent-class-filter))
-      (pushnew emacspeak-we-class-filter emacspeak-we-class-filter-history)
+      (cl-pushnew emacspeak-we-class-filter emacspeak-we-class-filter-history :test #'string=)
       (setq emacspeak-we-recent-class-filter
             emacspeak-we-class-filter))
     (emacspeak-we-xslt-filter
@@ -1016,10 +1050,10 @@ XPath can be set locally for a buffer, and overridden with an
 interactive prefix arg. If there is a known rewrite url rule, that is
 used as well."
   (interactive "P")
-  (declare (special emacspeak-we-xpath-junk
-                    emacspeak-we-xsl-junk
-                    emacspeak-we-recent-xpath-junk
-                    emacspeak-we-url-rewrite-rule))
+  (cl-declare (special emacspeak-we-xpath-junk
+                       emacspeak-we-xsl-junk
+                       emacspeak-we-recent-xpath-junk
+                       emacspeak-we-url-rewrite-rule))
   (emacspeak-webutils-browser-check)
   (let ((url (funcall emacspeak-webutils-url-at-point))
         (redirect nil))
@@ -1028,8 +1062,8 @@ used as well."
     (when emacspeak-we-url-rewrite-rule
       (setq redirect
             (replace-regexp-in-string
-             (first emacspeak-we-url-rewrite-rule)
-             (second emacspeak-we-url-rewrite-rule)
+             (cl-first emacspeak-we-url-rewrite-rule)
+             (cl-second emacspeak-we-url-rewrite-rule)
              url)))
     (when (or prompt
               (null emacspeak-we-xpath-junk))
@@ -1045,81 +1079,68 @@ used as well."
 
 ;;}}}
 ;;{{{ Property filter
-
 ;;;###autoload
-(defun emacspeak-we-extract-by-property (url &optional speak)
-  "Interactively prompt for an HTML property, e.g. id or class,
-and provide a completion list of applicable  property values. Filter document by property that is specified."
-  (interactive
-   (list
-    (emacspeak-webutils-read-url)
-    current-prefix-arg))
-  (let* ((completion-ignore-case t)
-         (choices
-          (mapcar 'symbol-name (intersection
-                                '(id class style role)
-                                (emacspeak-webutils-property-names-from-html-stack (emacspeak-w3-html-stack)))))
-         (property
-          (read
-           (completing-read "Property: "
-                            choices)))
-         (values (emacspeak-webutils-get-property-from-html-stack
-                  (emacspeak-w3-html-stack)
-                  property))
-         (v (completing-read "Having value: " values))
-         (filter
-          (if (eq property 'class)
-              (format "//*[contains(@%s, \"%s\")]"
-                      property v)
-            (format "//*[@%s=\"%s\"]"
-                    property v))))
-    (emacspeak-we-xslt-filter filter url
-                              (or (interactive-p) speak))))
 
 ;;}}}
 ;;{{{  xsl keymap
 
-(declaim (special emacspeak-we-xsl-map))
+(cl-declaim (special emacspeak-we-xsl-map))
 
-(loop for binding in
-      '(
-        ("C" emacspeak-we-extract-by-class-list)
-        ("D" emacspeak-we-junk-by-class-list)
-        ("w" emacspeak-we-extract-by-property)
-        ("M" emacspeak-we-extract-tables-by-match-list)
-        ("P" emacspeak-we-extract-print-streams)
-        ("R" emacspeak-we-extract-media-streams-under-point)
-        ("T" emacspeak-we-extract-tables-by-position-list)
-        ("X" emacspeak-we-extract-nested-table-list)
-        ("\C-c" emacspeak-we-junk-by-class-list)
-        ("\C-f" emacspeak-we-count-matches)
-        ("\C-p" emacspeak-we-xpath-junk-and-follow)
-        ("\C-t" emacspeak-we-count-tables)
-        ("\C-x" emacspeak-we-count-nested-tables)
-        ("a" emacspeak-we-xslt-apply)
-        ("c" emacspeak-we-extract-by-class)
-        ("d" emacspeak-we-junk-by-class)
-        ("e" emacspeak-we-url-expand-and-execute)
-        ("f" emacspeak-we-xslt-filter)
-        ("i" emacspeak-we-extract-by-id)
-        ("I" emacspeak-we-extract-by-id-list)
-        ("j" emacspeak-we-xslt-junk)
-        ("k" emacspeak-we-toggle-xsl-keep-result)
-        ("m" emacspeak-we-extract-table-by-match)
-        ("o" emacspeak-we-xsl-toggle)
-        ("p" emacspeak-we-xpath-filter-and-follow)
-        ("v" emacspeak-we-class-filter-and-follow-link)
-        ("r" emacspeak-we-extract-media-streams)
-        ("S" emacspeak-we-style-filter)
-        ("s" emacspeak-we-xslt-select)
-        ("t" emacspeak-we-extract-table-by-position)
-        ("u" emacspeak-we-extract-matching-urls)
-        ("x" emacspeak-we-extract-nested-table)
-        ("b" emacspeak-we-follow-and-filter-by-id)
-        ("y" emacspeak-we-class-filter-and-follow)
-        )
-      do
-      (emacspeak-keymap-update emacspeak-we-xsl-map binding))
+(cl-loop for binding in
+         '(
+           ("C" emacspeak-we-extract-by-class-list)
+           ("C-c" emacspeak-we-junk-by-class-list)
+           ("C-f" emacspeak-we-count-matches)
+           ("C-p" emacspeak-we-xpath-junk-and-follow)
+           ("C-t" emacspeak-we-count-tables)
+           ("C-x" emacspeak-we-count-nested-tables)
+           ("D" emacspeak-we-junk-by-class-list)
+           ("I" emacspeak-we-extract-by-id-list)
+           ("M" emacspeak-we-extract-tables-by-match-list)
+           ("P" emacspeak-we-follow-and-extract-main)
+           ("S" emacspeak-we-style-filter)
+           ("T" emacspeak-we-extract-tables-by-position-list)
+           ("X" emacspeak-we-extract-nested-table-list)
+           ("a" emacspeak-we-xslt-apply)
+           ("b" emacspeak-we-follow-and-filter-by-id)
+           ("c" emacspeak-we-extract-by-class)
+           ("d" emacspeak-we-junk-by-class)
+           ("e" emacspeak-we-url-expand-and-execute)
+           ("f" emacspeak-we-xslt-filter)
+           ("i" emacspeak-we-extract-by-id)
+           ("j" emacspeak-we-xslt-junk)
+           ("k" emacspeak-we-toggle-xsl-keep-result)
+           ("m" emacspeak-we-extract-table-by-match)
+           ("o" emacspeak-we-xsl-toggle)
+           ("p" emacspeak-we-xpath-filter-and-follow)
+           ("r" emacspeak-we-extract-by-role)
+           ("s" emacspeak-we-xslt-select)
+           ("t" emacspeak-we-extract-table-by-position)
+           ("u" emacspeak-we-extract-matching-urls)
+           ("v" emacspeak-we-class-filter-and-follow-link)
+                                        ;("w" emacspeak-we-extract-by-property)
+           ("x" emacspeak-we-extract-nested-table)
+           ("y" emacspeak-we-class-filter-and-follow)
+           )
+         do
+         (emacspeak-keymap-update emacspeak-we-xsl-map binding))
+
+;;}}}
+;;{{{  URL Advice: 
+
+(cl-loop
+ for f in
+ '(
+   url-write-global-history url-history-save-history
+   url-http-chunked-encoding-after-change-function url-cookie-handle-set-cookie
+   url-retrieve-internal
+   url-lazy-message url-cookie-write-file)
+ do
+ (eval
+  `(defadvice   ,f (around emacspeak pre act comp)
+     "Silence messages while this function executes"
+     (let ((url-show-status nil))
+       (ems-with-messages-silenced ad-do-it)))))
 
 ;;}}}
 (provide 'emacspeak-we)
