@@ -1,5 +1,5 @@
-;;; emacspeak-widget.el --- Speech enable Emacs' native GUI widget library
-;;; $Id: emacspeak-widget.el 6921 2011-03-09 16:09:36Z tv.raman.tv $
+;;; emacspeak-widget.el --- Speech enable Emacs' native GUI widget library  -*- lexical-binding: t; -*-
+;;; $Id$
 ;;; $Author: tv.raman.tv $ 
 ;;; Description: Emacspeak extensions to widgets
 ;;; Keywords:emacspeak, audio interface to emacs customized widgets
@@ -15,7 +15,7 @@
 
 ;;}}}
 ;;{{{  Copyright:
-;;;Copyright (C) 1995 -- 2011, T. V. Raman 
+;;;Copyright (C) 1995 -- 2018, T. V. Raman 
 ;;; Copyright (c) 1995 by T. V. Raman  
 ;;; All Rights Reserved. 
 ;;;
@@ -49,8 +49,8 @@
 
 ;;; Code:
 
-(require 'cl)
-(declaim  (optimize  (safety 0) (speed 3)))
+(require 'cl-lib)
+(cl-declaim  (optimize  (safety 0) (speed 3)))
 (require 'custom)
 (require 'voice-setup)
 (require 'dtk-speak)
@@ -58,7 +58,7 @@
 (require 'wid-edit)
 (require 'emacspeak-speak)
 (require 'emacspeak-sounds)
-
+(require 'emacspeak-webutils)
 ;;}}}
 ;;{{{  Customize global behavior
 
@@ -78,13 +78,13 @@
    (widget-inactive voice-lighten)
    ))
 
-(declaim (special widget-menu-minibuffer-flag))
+(cl-declaim (special widget-menu-minibuffer-flag))
 (setq  widget-menu-minibuffer-flag t)
 
 ;;}}}
 ;;{{{  helpers 
 
-(defsubst emacspeak-widget-label (w)
+(defun emacspeak-widget-label (w)
   "Construct a label for a widget.
 Returns a string with appropriate personality."
   (let ((inhibit-read-only t)
@@ -97,9 +97,9 @@ Returns a string with appropriate personality."
                        'personality 'emacspeak-widget-button-personality tag)
     tag))
 
-(defsubst emacspeak-widget-help-echo (w)
+(defun emacspeak-widget-help-echo (w)
   "Return help-echo with appropriate personality."
-  (declare (special voice-animate))
+  (cl-declare (special voice-animate))
   (let ((inhibit-read-only t)
         (h (widget-get w :help-echo))
         (help nil))
@@ -149,13 +149,13 @@ Returns a string with appropriate personality."
 ;;;###autoload
 (defun emacspeak-widget-summarize(widget)
   "Summarize specified widget."
-  (let ((emacspeak-help (widget-get widget :emacspeak-help))
-        (emacspeak-speak-messages nil))
-    (cond
-     ((and emacspeak-help
-           (fboundp emacspeak-help))
-      (dtk-speak  (funcall emacspeak-help widget)))
-     (t (dtk-speak (current-message))))))
+  (ems-with-messages-silenced
+   (let ((emacspeak-help (widget-get widget :emacspeak-help)))
+     (cond
+      ((and emacspeak-help
+            (fboundp emacspeak-help))
+       (dtk-speak  (funcall emacspeak-help widget)))
+      (t (dtk-speak (current-message)))))))
 
 ;;}}}
 ;;{{{ advice activators 
@@ -169,7 +169,7 @@ Returns a string with appropriate personality."
   (let* ((inhibit-read-only t)
          (label (emacspeak-widget-label widget))
          (help-echo (emacspeak-widget-help-echo widget))
-         (v  (widget-value widget ))
+         (v  (widget-value widget))
          (value
           (cond
            ((null v) nil)
@@ -249,7 +249,7 @@ Returns a string with appropriate personality."
                           :emacspeak-help))))
     (concat label
             help-echo
-            context )))
+            context)))
 
 (widget-put (get 'push-button 'widget-type)
             :emacspeak-help 'emacspeak-widget-help-push-button)
@@ -362,8 +362,7 @@ Returns a string with appropriate personality."
 
 (defun emacspeak-widget-help-menu-choice  (widget)
   "Summarize a pull down list"
-  (let* ((help-echo (emacspeak-widget-help-echo widget))(label (emacspeak-widget-label widget))
-         (value (format " %s " (widget-get widget :value)))
+  (let* ((label (emacspeak-widget-label widget)) (value (format " %s " (widget-get widget :value)))
          (child (car (widget-get widget :children))))
     (concat label
             " is "
@@ -516,14 +515,14 @@ Returns a string with appropriate personality."
 
 ;;; avoid redundant message speech output
 (defadvice widget-echo-help (around emacspeak pre act comp)
-  (let ((emacspeak-speak-messages nil))
-    ad-do-it
-    ad-return-value))
+  (ems-with-messages-silenced
+   ad-do-it
+   ad-return-value))
 (defadvice widget-beginning-of-line (after emacspeak pre act comp)
   "Provide auditory feedback"
   (cond
-   ((interactive-p)
-    (let ((widget (widget-at (point ))))
+   ((ems-interactive-p)
+    (let ((widget (widget-at (point))))
       ad-do-it
       (emacspeak-auditory-icon 'select-object)
       (message "Moved to start of text field %s"
@@ -536,8 +535,8 @@ Returns a string with appropriate personality."
 (defadvice widget-end-of-line (around emacspeak pre act comp)
   "Provide auditory feedback"
   (cond
-   ((interactive-p)
-    (let ((widget (widget-at (point ))))
+   ((ems-interactive-p)
+    (let ((widget (widget-at (point))))
       ad-do-it
       (emacspeak-auditory-icon 'select-object)
       (message "Moved to end of text field %s"
@@ -549,28 +548,31 @@ Returns a string with appropriate personality."
 
 (defadvice widget-forward (after emacspeak pre act comp)
   "Provide auditory feedback"
-  (when (interactive-p)
-    (emacspeak-auditory-icon 'large-movement)
-    (emacspeak-widget-summarize (widget-at  (point )))))
+  (when (ems-interactive-p)
+    (emacspeak-auditory-icon 'item)
+    (emacspeak-widget-summarize (widget-at  (point)))))
 
 (defadvice widget-backward (after emacspeak pre act comp)
   "Provide auditory feedback"
-  (when (interactive-p)
-    (emacspeak-auditory-icon 'large-movement)
+  (when (ems-interactive-p)
+    (emacspeak-auditory-icon 'item)
     (emacspeak-widget-summarize (widget-at (point)))))
 
 (defadvice widget-kill-line (after emacspeak pre act comp)
   "Provide auditory feedback"
-  (when (interactive-p)
+  (when (ems-interactive-p)
     (emacspeak-auditory-icon 'delete-object)
     (emacspeak-speak-current-kill 0)
-    (dtk-tone 500 30)))
+    (dtk-tone-deletion)))
 
 ;;}}}
 ;;{{{  activating widgets:
-
+;;; forward declaration:
+(defvar emacspeak-we-url-executor)
 (defadvice widget-button-press (around emacspeak pre act comp)
   "Provide auditory feedback"
+  (cl-declare (special emacspeak-webutils-url-at-point
+                       emacspeak-we-url-executor))
   (let ((inhibit-read-only t)
         (widget (widget-at (ad-get-arg 0))))
     (cond
@@ -578,18 +580,17 @@ Returns a string with appropriate personality."
       (let ((pos (ad-get-arg 0))
             (old-position (point)))
         (cond
-         ((and
-           (or (eq major-mode 'w3-mode) (eq major-mode 'w3m-mode))
-           emacspeak-webutils-url-at-point
-           (funcall emacspeak-webutils-url-at-point)
-           emacspeak-we-url-executor
-           (boundp 'emacspeak-we-url-executor)
-           (fboundp emacspeak-we-url-executor))
+         ((and (eq major-mode 'eww-mode)
+               emacspeak-webutils-url-at-point
+               (funcall emacspeak-webutils-url-at-point)
+               emacspeak-we-url-executor
+               (boundp 'emacspeak-we-url-executor)
+               (fboundp emacspeak-we-url-executor))
           (emacspeak-auditory-icon 'button)
           (call-interactively 'emacspeak-we-url-expand-and-execute))
          (t ad-do-it
             (cond
-             ((= old-position (point )) ;did not move
+             ((= old-position (point)) ;did not move
               (emacspeak-auditory-icon 'button)
               (emacspeak-widget-summarize (widget-at pos)))
              (t  (emacspeak-auditory-icon 'large-movement)
@@ -606,19 +607,19 @@ Returns a string with appropriate personality."
 Optional interactive prefix specifies how many levels to go up from current
 widget before summarizing."
   (interactive "P")
-  (let ((widget (widget-at (point ))))
+  (let ((widget (widget-at (point))))
     (when(and widget  level)
-      (loop for i from 1 to level
-            do
-            (setq widget (widget-get  widget :parent))))
+      (cl-loop for i from 1 to level
+               do
+               (setq widget (widget-get  widget :parent))))
     (cond
-     (widget (emacspeak-widget-summarize widget ))
+     (widget (emacspeak-widget-summarize widget))
      (t (message "No widget under point")))))
 
 (defun emacspeak-widget-browse-widget-interactively ()
   "Allows you to browse a widget"
   (interactive)
-  (let ((level nil )
+  (let ((level nil)
         (key nil)
         (continue t))
     (emacspeak-widget-summarize-widget-under-point)
@@ -630,11 +631,11 @@ widget before summarizing."
        ((= key ?.) nil)
        ((= key ?u)
         (if (numberp level)
-            (incf level)
+            (cl-incf level)
           (setq level 1)))
        ((= key ?d)
         (if (> level  0)
-            (decf level)
+            (cl-decf level)
           (message "Leaf widget")))
        (t (read-key-sequence "Press any key to continue")))
       (emacspeak-widget-summarize-widget-under-point level))))
@@ -647,12 +648,12 @@ widget before summarizing."
   (let ((inhibit-read-only t)
         (start (ad-get-arg 1))
         (end (ad-get-arg 2))
-        (orig nil ))
+        (orig nil))
     (setq orig (get-text-property start 'personality))
     ad-do-it 
     (and orig 
          (put-text-property start end 
-                            'personality orig ))))
+                            'personality orig))))
 
 ;;}}}
 ;;{{{ update widget related keymaps so we dont loose the
@@ -660,28 +661,26 @@ widget before summarizing."
 
 (defadvice widget-setup (after emacspeak pre act comp)
   "Update widget keymaps."
-  (declare (special emacspeak-prefix
-                    widget-keymap widget-field-keymap widget-text-keymap))
-  (loop for map in
-        (list widget-keymap
-              widget-field-keymap
-              widget-text-keymap
-              )
-        do
-        (define-key map  emacspeak-prefix 'emacspeak-prefix-command)
-                                        ;(define-key map  "\C-e\C-e" 'widget-end-of-line)
-        (define-key map "\M-h" 'emacspeak-widget-help)
-        (define-key map "\M-p" 'emacspeak-widget-summarize-parent)
-        (define-key map "\M-\C-m"
-          'emacspeak-widget-update-from-minibuffer)))
+  (cl-declare (special emacspeak-prefix
+                       widget-field-keymap widget-text-keymap))
+  (cl-loop
+   for map in
+   '(widget-field-keymap widget-text-keymap)
+   do
+   (when  (keymapp map)
+     (define-key map  emacspeak-prefix 'emacspeak-prefix-command)
+     (define-key map  (concat emacspeak-prefix emacspeak-prefix) 'widget-end-of-line)
+     (define-key map "\M-h" 'emacspeak-widget-help)
+     (define-key map "\M-p" 'emacspeak-widget-summarize-parent)
+     (define-key map "\M-\C-m" 'emacspeak-widget-update-from-minibuffer))))
 
 ;;}}}
 ;;{{{ augment widgets 
 
-(defun emacspeak-widget-update-from-minibuffer (point)
-  "Sets widget at point by invoking its prompter."
+(defun emacspeak-widget-update-from-minibuffer (pos)
+  "Sets widget at `pos' by invoking its prompter."
   (interactive "d")
-  (let ((w (widget-at (point))))
+  (let ((w (widget-at pos)))
     (widget-value-set w
                       (widget-apply w
                                     :prompt-value
@@ -692,9 +691,9 @@ widget before summarizing."
     (widget-apply w :notify)
     (emacspeak-widget-summarize w)))
 
-(declaim (special widget-keymap
-                  widget-field-keymap
-                  widget-text-keymap))
+(cl-declaim (special widget-keymap
+                     widget-field-keymap
+                     widget-text-keymap))
 
 ;;}}}
 
@@ -702,10 +701,11 @@ widget before summarizing."
 ;;{{{ voice widgets 
 
 (define-widget 'voice  'menu-choice
-  :help-echo "Voice selector"
   "Widget for selecting a voice.")
+
 (define-widget 'personality 'item
   "Individual voice in a voice selector.")
+
 ;;; We rely on dectalk-voice-table as our default voice table.
 ;;; Names defined in this --- and other voice tables --- are
 ;;; generic --and  not device specific.
@@ -713,14 +713,14 @@ widget before summarizing."
 
 (defun emacspeak-widget-create-voice-selector ()
   "Create a suitable voice selector widget."
-  (declare (special dectalk-voice-table))
+  (cl-declare (special dectalk-voice-table))
   (let ((w
          (widget-create 'voice
                         :tag "voices")))
     (widget-put w :args 
-                (loop for key being the hash-keys of dectalk-voice-table 
-                      collect
-                      (list 'personality :value key )))
+                (cl-loop for key being the hash-keys of dectalk-voice-table 
+                         collect
+                         (list 'personality :value key)))
     w))
 
 ;;}}}
