@@ -1,5 +1,5 @@
-;;; emacspeak-xslt.el --- Implements Emacspeak  xslt transform engine
-;;; $Id: emacspeak-xslt.el 8276 2013-03-30 15:19:30Z tv.raman.tv $
+;;; emacspeak-xslt.el --- Implements Emacspeak  xslt transform engine  -*- lexical-binding: t; -*-
+;;; $Id$
 ;;; $Author: tv.raman.tv $
 ;;; Description:  xslt transformation routines
 ;;; Keywords: Emacspeak,  Audio Desktop XSLT
@@ -15,7 +15,7 @@
 
 ;;}}}
 ;;{{{  Copyright:
-;;;Copyright (C) 1995 -- 2011, T. V. Raman
+;;;Copyright (C) 1995 -- 2018, T. V. Raman
 ;;; Copyright (c) 1994, 1995 by Digital Equipment Corporation.
 ;;; All Rights Reserved.
 ;;;
@@ -39,26 +39,29 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 ;;{{{  introduction
-
+;;; Commentary:
 ;;; libxml and libxsl are XML libraries for GNOME.
 ;;; xsltproc is a  xslt processor using libxsl
 ;;; this module defines routines for applying xsl transformations
 ;;; using xsltproc
-
+;;; Code:
 ;;}}}
 ;;{{{  Required modules
 
+(cl-declaim  (optimize  (safety 0) (speed 3)))
 (require 'emacspeak-preamble)
 (require 'emacspeak-webutils)
-
 ;;}}}
 ;;{{{  xslt Environment:
-
+;;;###autoload
+(defun emacspeak-xslt-get (style)
+  "Return fully qualified stylesheet path."
+  (expand-file-name style emacspeak-xslt-directory))
 (defgroup emacspeak-xslt nil
   "XSL transformation group."
   :group 'emacspeak)
 
-(defsubst emacspeak-xslt-params-from-xpath (path base)
+(defun emacspeak-xslt-params-from-xpath (path base)
   "Return params suitable for passing to  emacspeak-xslt-region"
   (list
    (cons "path"
@@ -71,15 +74,9 @@
          (format "\"'%s'\""
                  base))))
 
-;;;###autoload
-(defsubst emacspeak-xslt-get (style)
-  "Return fully qualified stylesheet path."
-  (declare (special emacspeak-xslt-directory))
-  (expand-file-name style emacspeak-xslt-directory))
-
-(defsubst emacspeak-xslt-read ()
+(defun emacspeak-xslt-read ()
   "Read XSLT transformation name from minibuffer."
-  (declare (special emacspeak-xslt-directory))
+  (cl-declare (special emacspeak-xslt-directory emacspeak-we-xsl-transform))
   (expand-file-name
    (read-file-name "XSL Transformation: "
                    emacspeak-xslt-directory
@@ -116,50 +113,51 @@ This is useful when handling bad HTML."
   "Apply XSLT transformation to region and replace it with
 the result.  This uses XSLT processor xsltproc available as
 part of the libxslt package."
-  (declare (special emacspeak-xslt-program emacspeak-xslt-options
-                    emacspeak-xslt-keep-errors modification-flag ))
-  (let ((command nil)
-        (parameters (when params
-                      (mapconcat
-                       #'(lambda (pair)
-                           (format "--param %s %s "
-                                   (car pair)
-                                   (cdr pair)))
-                       params
-                       " ")))
-        (coding-system-for-write 'utf-8)
-        (coding-system-for-read 'utf-8)
-        (buffer-file-coding-system 'utf-8))
-    (setq command
-          (format
-           "%s %s  %s  %s - %s"
-           emacspeak-xslt-program
-           (or emacspeak-xslt-options "")
-           (or parameters "")
-           xsl
-           (unless  emacspeak-xslt-keep-errors " 2>/dev/null ")))
-    (shell-command-on-region
-     start end
-     command
-     (current-buffer)
-     'replace
-     (when emacspeak-xslt-keep-errors "*xslt errors*"))
-    (when (get-buffer  "*xslt errors*")
-      (bury-buffer "*xslt errors*"))
-    (unless no-comment
-      (goto-char (point-max))
-      (insert
-       (format "<!--\n %s \n-->\n"
-               command)))
-    (setq modification-flag nil)
-    (set-buffer-multibyte t)
-    (current-buffer)))
+  (cl-declare (special emacspeak-xslt-program emacspeak-xslt-options
+                       emacspeak-xslt-keep-errors modification-flag))
+  (save-excursion
+    (with-silent-modifications
+      (let ((command nil)
+            (parameters (when params
+                          (mapconcat
+                           #'(lambda (pair)
+                               (format "--param %s %s "
+                                       (car pair)
+                                       (cdr pair)))
+                           params
+                           " ")))
+            (coding-system-for-write 'utf-8)
+            (coding-system-for-read 'utf-8)
+            (buffer-file-coding-system 'utf-8))
+        (setq command
+              (format
+               "%s %s  %s  %s - %s"
+               emacspeak-xslt-program
+               (or emacspeak-xslt-options "")
+               (or parameters "")
+               xsl
+               (unless  emacspeak-xslt-keep-errors " 2>/dev/null ")))
+        (shell-command-on-region
+         start end
+         command
+         (current-buffer)
+         'replace
+         (when emacspeak-xslt-keep-errors "*xslt errors*"))
+        (when (get-buffer  "*xslt errors*")
+          (bury-buffer "*xslt errors*"))
+        (unless no-comment
+          (goto-char (point-max))
+          (insert
+           (format "<!--\n %s \n-->\n"
+                   command)))
+        (set-buffer-multibyte t)
+        (current-buffer)))))
 
 ;;;###autoload
-(defsubst emacspeak-xslt-run (xsl &optional start end)
+(defun emacspeak-xslt-run (xsl &optional start end)
   "Run xslt on region, and return output filtered by sort -u.
 Region defaults to entire buffer."
-  (declare (special emacspeak-xslt-program emacspeak-xslt-options))
+  (cl-declare (special emacspeak-xslt-program emacspeak-xslt-options))
   (or start (setq start (point-min)))
   (or end (setq end (point-max)))
   (let ((coding-system-for-read 'utf-8)
@@ -173,25 +171,14 @@ Region defaults to entire buffer."
     (set-buffer-multibyte t)
     (current-buffer)))
 
-;;; uses wget in a pipeline to avoid libxml2 bug:
-;;;###autoload
-(defcustom  emacspeak-xslt-use-wget-to-download nil
-  "Set to T if you want to avoid URL downloader bugs in libxml2.
-There is a bug that bites when using Yahoo Maps that wget can
-work around."
-  :group 'emacspeak-xslt
-  :type 'boolean)
-
 ;;;###autoload
 (defun emacspeak-xslt-url (xsl url &optional params no-comment)
   "Apply XSLT transformation to url
 and return the results in a newly created buffer.
   This uses XSLT processor xsltproc available as
 part of the libxslt package."
-  (declare (special emacspeak-xslt-program
-                    emacspeak-xslt-use-wget-to-download
-                    modification-flag
-                    emacspeak-xslt-keep-errors))
+  (cl-declare (special emacspeak-xslt-program
+                       emacspeak-xslt-keep-errors))
   (let ((result (get-buffer-create " *xslt result*"))
         (command nil)
         (parameters (when params
@@ -202,23 +189,14 @@ part of the libxslt package."
                                    (cdr pair)))
                        params
                        " "))))
-    (if emacspeak-xslt-use-wget-to-download
-        (setq command (format
-                       "wget -U mozilla -q -O - '%s' | %s %s    --html --novalid %s '%s' %s"
-                       url
-                       emacspeak-xslt-program
-                       (or parameters "")
-                       xsl "-"
-                       (unless emacspeak-xslt-keep-errors " 2>/dev/null ")))
-      (setq command
-            (format
-             "%s %s    --html --novalid %s '%s' %s"
-             emacspeak-xslt-program
-             (or parameters "")
-             xsl url
-             (unless emacspeak-xslt-keep-errors " 2>/dev/null "))))
-    (save-excursion
-      (set-buffer result)
+    (setq command
+          (format
+           "%s %s    --html --novalid %s '%s' %s"
+           emacspeak-xslt-program
+           (or parameters "")
+           xsl url
+           (unless emacspeak-xslt-keep-errors " 2>/dev/null ")))
+    (with-current-buffer result 
       (kill-all-local-variables)
       (erase-buffer)
       (setq buffer-undo-list t)
@@ -231,7 +209,7 @@ part of the libxslt package."
         (when emacspeak-xslt-nuke-null-char
           (goto-char (point-min))
           (while (search-forward
-                  ( format "%c" 0)
+                  (format "%c" 0)
                   nil  t)
             (replace-match " "))))
       (when (get-buffer  "*xslt errors*")
@@ -246,43 +224,82 @@ part of the libxslt package."
       (goto-char (point-min))
       result)))
 
+(defun emacspeak-xslt-pipeline-url (specs url &optional  no-comment)
+  "Apply XSLT transformation to url
+and browse the results.
+Argument `specs' is a list of elements of the form
+`(xsl xpath)'.
+  This uses XSLT processor xsltproc available as
+part of the libxslt package."
+  (cl-declare (special emacspeak-xslt-program
+                       emacspeak-xslt-keep-errors))
+  (let ((result (url-retrieve-synchronously url))
+        (command ""))
+    (setq command
+          (apply
+           #'concat
+           (cl-loop
+            for s in specs
+            and i from 0 collect 
+            (format
+             "%s %s %s %s %s - 2>/dev/null  "
+             (if (= i 0)  "" "|")
+             emacspeak-xslt-program
+             (or emacspeak-xslt-options "")
+             (mapconcat
+              #'(lambda (pair)
+                  (format "--param %s %s "
+                          (car pair) (cdr pair)))
+              (emacspeak-xslt-params-from-xpath (cl-second s) url)
+              "")
+             (cl-first s)))))
+    (with-silent-modifications
+      (with-current-buffer result 
+        (let ((coding-system-for-write 'utf-8)
+              (coding-system-for-read 'utf-8)
+              (buffer-file-coding-system 'utf-8))
+          (goto-char (point-min))
+          (search-forward "\n\n")
+          (delete-region (point-min) (point))
+          (shell-command-on-region
+           (point-min) (point-max)
+           command (current-buffer) 'replace
+           (when emacspeak-xslt-keep-errors "*xslt errors*")))
+        (when (get-buffer  "*xslt errors*")
+          (bury-buffer "*xslt errors*"))
+        (goto-char (point-max))
+        (unless no-comment
+          (insert
+           (format "<!--\n %s \n-->\n"
+                   command)))
+        (set-buffer-multibyte t)
+        (goto-char (point-min))
+        (browse-url-of-buffer)))))
+
 ;;;###autoload
 (defun emacspeak-xslt-xml-url (xsl url &optional params)
   "Apply XSLT transformation to XML url
 and return the results in a newly created buffer.
   This uses XSLT processor xsltproc available as
 part of the libxslt package."
-  (declare (special emacspeak-xslt-program
-                    emacspeak-xslt-use-wget-to-download
-                    modification-flag
-                    emacspeak-xslt-keep-errors))
+  (cl-declare (special emacspeak-xslt-program
+                       emacspeak-xslt-keep-errors))
   (let ((result (get-buffer-create " *xslt result*"))
         (command nil)
-        (parameters (when params
-                      (mapconcat
-                       #'(lambda (pair)
-                           (format "--param %s %s "
-                                   (car pair)
-                                   (cdr pair)))
-                       params
-                       " "))))
-    (if emacspeak-xslt-use-wget-to-download
-        (setq command
-              (format
-               "wget -q -O - '%s' | %s %s --novalid %s %s %s"
-               url
-               emacspeak-xslt-program
-               (or parameters "")
-               xsl "-"
-               (unless emacspeak-xslt-keep-errors " 2>/dev/null ")))
-      (setq command
-            (format
-             "%s %s --novalid %s '%s' %s"
-             emacspeak-xslt-program
-             (or parameters "")
-             xsl url
-             (unless emacspeak-xslt-keep-errors " 2>/dev/null "))))
-    (save-excursion
+        (parameters
+         (when params
+           (mapconcat
+            #'(lambda (pair)
+                (format "--param %s %s " (car pair) (cdr pair)))
+            params " "))))
+    (setq command
+          (format
+           "%s %s --novalid %s '%s' %s"
+           emacspeak-xslt-program
+           (or parameters "")
+           xsl url
+           (unless emacspeak-xslt-keep-errors " 2>/dev/null ")))
+    (save-current-buffer
       (set-buffer result)
       (kill-all-local-variables)
       (erase-buffer)
@@ -315,13 +332,15 @@ part of the libxslt package."
     (read-file-name "Style File: "
                     emacspeak-xslt-directory)
     (read-file-name "File:" default-directory)))
+  (cl-declare (special emacspeak-xslt-directory))
   (with-temp-buffer
-    (let ((coding-system-for-read 'utf-8)
+    (let ((browse-url-browser-function  'eww-browse-url)
+          (coding-system-for-read 'utf-8)
           (coding-system-for-write 'utf-8)
           (buffer-file-coding-system 'utf-8))
       (insert-file file)
       (shell-command
-       (format "%s   --novalid --nonet --param base %s  %s  %s  2>/dev/null"
+       (format "%s   --novalid --nonet --param base %s  %s  \"%s\"  2>/dev/null"
                emacspeak-xslt-program 
                (format "\"'file://%s'\"" (expand-file-name file))
                (expand-file-name style)
@@ -336,15 +355,13 @@ part of the libxslt package."
   (interactive
    (list
     (expand-file-name
-     (read-file-name "XSL Transformation: "
-                     emacspeak-xslt-directory))
+     (read-file-name "XSL Transformation: "))
     (read-string "URL: " (browse-url-url-at-point))))
-  (declare (special emacspeak-xslt-options))
-  (emacspeak-webutils-with-xsl-environment
-   style
-   nil
-   emacspeak-xslt-options
-   (browse-url url)))
+  (cl-declare (special emacspeak-xslt-options))
+  (add-hook
+   'emacspeak-web-pre-process-hook
+   (emacspeak-webutils-make-xsl-transformer style))
+  (browse-url url))
 
 ;;;###autoload
 (defun emacspeak-xslt-view-xml (style url &optional unescape-charent)
@@ -354,7 +371,8 @@ part of the libxslt package."
     (emacspeak-xslt-read)
     (emacspeak-webutils-read-this-url)
     current-prefix-arg))
-  (let ((src-buffer
+  (let ((browse-url-browser-function  'eww-browse-url)
+        (src-buffer
          (emacspeak-xslt-xml-url
           style
           url
@@ -362,8 +380,8 @@ part of the libxslt package."
            (cons "base"
                  (format "\"'%s'\""
                          url))))))
-    (when (ems-interactive-p ) (emacspeak-webutils-autospeak))
-    (save-excursion
+    (when (ems-interactive-p) (emacspeak-webutils-autospeak))
+    (save-current-buffer
       (set-buffer src-buffer)
       (when unescape-charent
         (emacspeak-webutils-unescape-charent (point-min) (point-max)))
@@ -380,10 +398,11 @@ part of the libxslt package."
     (point)
     (mark)
     current-prefix-arg))
-  (let ((src-buffer
-         (ems-modify-buffer-safely
-          (emacspeak-xslt-region style start end))))
-    (save-excursion
+  (let ((browse-url-browser-function  'eww-browse-url)
+        (src-buffer
+         (with-silent-modifications
+           (emacspeak-xslt-region style start end))))
+    (save-current-buffer
       (set-buffer src-buffer)
       (when unescape-charent
         (emacspeak-webutils-unescape-charent (point-min) (point-max)))
@@ -397,7 +416,7 @@ part of the libxslt package."
 
 ;;; local variables:
 ;;; folded-file: t
-;;; byte-compile-dynamic: nil
+;;; byte-compile-dynamic: t
 ;;; end:
 
 ;;}}}
