@@ -1,11 +1,11 @@
-;;; espeak-voices.el --- Define various device independent voices in terms of Espeak tags
+;;; espeak-voices.el --- Define  Espeak tags  -*- lexical-binding: t; -*-
 ;;; Description:  Module to set up Espeak voices and personalities
 ;;; Keywords: Voice, Personality, Espeak
 ;;{{{  LCD Archive entry:
 
 ;;}}}
 ;;{{{  Copyright:
-;;;Copyright (C) 1995 -- 2011, T. V. Raman 
+;;;Copyright (C) 1995 -- 2018, T. V. Raman 
 ;;; Copyright (c) 1994, 1995 by Digital Equipment Corporation.
 ;;; All Rights Reserved.
 ;;;
@@ -31,16 +31,31 @@
 ;;{{{  Introduction:
 
 ;;; Commentary:
-;;; This module defines the various voices used in voice-lock mode.
-;;; This module is Espeak specific.
 
+;;; This module defines the various voices used in voice-lock mode by
+;;; the ESpeak TTS engine.
+
+;;; Code:
 ;;}}}
 ;;{{{ Required modules
 
-;;; Code:
-(require 'cl)
-(declaim  (optimize  (safety 0) (speed 3)))
+(require 'cl-lib)
+(cl-declaim  (optimize  (safety 0) (speed 3)))
 (require 'acss-structure)
+(require 'tts)
+
+;;}}}
+;;{{{ Customizations:
+;;;###autoload
+(defcustom espeak-default-speech-rate 175
+  "Default speech rate for eSpeak."
+  :group 'tts
+  :type 'integer
+  :set #'(lambda(sym val)
+           (set-default sym val)
+           (when (and (getenv "DTK_PROGRAM")
+                      (string-match "espeak$" (getenv "DTK_PROGRAM")))
+             (setq-default dt-speech-rate val))))
 
 ;;}}}
 ;;{{{ Top-Level TTS Call
@@ -55,7 +70,8 @@
 ;;}}}
 ;;{{{  voice table
 
-(defvar tts-default-voice nil
+(defvar tts-default-voice
+  "<voice xml:lang=\"$la\" gender=\"male\" variant=\"1\">"
   "Default voice used. ")
 
 (defvar espeak-default-voice-string ""
@@ -65,26 +81,26 @@
   "Association between symbols and strings to set Espeak  voices.
 The string can set any voice parameter.")
 
-(defsubst espeak-define-voice (name command-string)
+(defun espeak-define-voice (name command-string)
   "Define an Espeak  voice named NAME.
 This voice will be set   by sending the string
 COMMAND-STRING to the TTS engine."
-  (declare (special espeak-voice-table ))
+  (cl-declare (special espeak-voice-table))
   (puthash name command-string espeak-voice-table))
 
-(defsubst espeak-get-voice-command  (name)
+(defun espeak-get-voice-command  (name)
   "Retrieve command string for  voice NAME."
-  (declare (special espeak-voice-table))
+  (cl-declare (special espeak-voice-table))
   (cond
    ((listp name)
     (mapconcat #'espeak-get-voice-command name " "))
    (t (or  (gethash name espeak-voice-table)
            espeak-default-voice-string))))
 
-(defsubst espeak-voice-defined-p (name)
+(defun espeak-voice-defined-p (name)
   "Check if there is a voice named NAME defined."
-  (declare (special espeak-voice-table ))
-  (gethash name espeak-voice-table ))
+  (cl-declare (special espeak-voice-table))
+  (gethash name espeak-voice-table))
 
 ;;}}}
 ;;{{{ voice definitions
@@ -113,7 +129,7 @@ COMMAND-STRING to the TTS engine."
 
 ;;{{{ voice family codes
 
-(defsubst espeak-get-family-code (name)
+(defun espeak-get-family-code (_name)
   "Get control code for voice family NAME."
   "")
 
@@ -125,17 +141,17 @@ COMMAND-STRING to the TTS engine."
 Keys are symbols of the form <FamilyName-Dimension>.
 Values are vectors holding the control codes for the 10 settings.")
 
-(defsubst espeak-css-set-code-table (family dimension table)
+(defun espeak-css-set-code-table (family dimension table)
   "Set up voice FAMILY.
 Argument DIMENSION is the dimension being set,
 and TABLE gives the values along that dimension."
-  (declare (special espeak-css-code-tables))
+  (cl-declare (special espeak-css-code-tables))
   (let ((key (intern (format "%s-%s" family dimension))))
-    (puthash key table espeak-css-code-tables )))
+    (puthash key table espeak-css-code-tables)))
 
-(defsubst espeak-css-get-code-table (family dimension)
+(defun espeak-css-get-code-table (family dimension)
   "Retrieve table of values for specified FAMILY and DIMENSION."
-  (declare (special espeak-css-code-tables))
+  (cl-declare (special espeak-css-code-tables))
   (let ((key (intern (format "%s-%s" family dimension))))
     (gethash key espeak-css-code-tables)))
 
@@ -155,13 +171,12 @@ and TABLE gives the values along that dimension."
 ;;{{{  paul average pitch
 
 (let ((table (make-vector 10 "")))
-  (mapcar
-   (function
-    (lambda (setting)
-      (aset table
-            (first setting)
-            (format "<prosody pitch=\"%s\">"
-                    (second setting)))))
+  (mapc
+   #'(lambda (setting)
+       (aset table
+             (cl-first setting)
+             (format "<prosody pitch=\"%s\">"
+                     (cl-second setting))))
    '(
      (0 0)
      (1 10)
@@ -173,43 +188,41 @@ and TABLE gives the values along that dimension."
      (7 70)
      (8 80)
      (9 90)))
-  (espeak-css-set-code-table 'paul 'average-pitch table ))
+  (espeak-css-set-code-table 'paul 'average-pitch table))
 
 ;;}}}
 ;;{{{  harry average pitch
 
 (let ((table (make-vector 10 "")))
-  (mapcar
-   (function
-    (lambda (setting)
-      (aset table
-            (first setting)
-            (format "<prosody pitch=\"%s\">"
-                    (second setting)))))
+  (mapc
+   #'(lambda (setting)
+       (aset table
+             (cl-first setting)
+             (format "<prosody pitch=\"%s\">"
+                     (cl-second setting))))
    '(
      (0 26)
      (1 32.5)
      (2 39)
      (3 45.5)
-     (4 52 )
+     (4 52)
      (5 58.5)
      (6 65)
      (7 71.5)
      (8 78)
      (9 84.5)))
-  (espeak-css-set-code-table 'harry 'average-pitch table ))
+  (espeak-css-set-code-table 'harry 'average-pitch table))
 
 ;;}}}
 ;;{{{  betty average pitch
 
 (let ((table (make-vector 10 "")))
-  (mapcar
-   (function
-    (lambda (setting)
-      (aset table
-            (first setting)
-            (format "<prosody pitch=\"%s\">"
-                    (second setting)))))
+  (mapc
+   #'(lambda (setting)
+       (aset table
+             (cl-first setting)
+             (format "<prosody pitch=\"%s\">"
+                     (cl-second setting))))
    '(
      (0 40)
      (1 48)
@@ -221,11 +234,11 @@ and TABLE gives the values along that dimension."
      (7 96)
      (8 104)
      (9 112)))
-  (espeak-css-set-code-table 'betty 'average-pitch table ))
+  (espeak-css-set-code-table 'betty 'average-pitch table))
 
 ;;}}}
 
-(defsubst espeak-get-average-pitch-code (value family)
+(defun espeak-get-average-pitch-code (value family)
   "Get  AVERAGE-PITCH for specified VALUE and  FAMILY."
   (or family (setq family 'paul))
   (if value
@@ -241,13 +254,12 @@ and TABLE gives the values along that dimension."
 ;;{{{  paul pitch range
 
 (let ((table (make-vector 10 "")))
-  (mapcar
-   (function
-    (lambda (setting)
-      (aset table
-            (first setting)
-            (format "<prosody range=\"%s\">"
-                    (second setting)))))
+  (mapc
+   #'(lambda (setting)
+       (aset table
+             (cl-first setting)
+             (format "<prosody range=\"%s\">"
+                     (cl-second setting))))
    '(
      (0 0)
      (1 10)
@@ -259,19 +271,18 @@ and TABLE gives the values along that dimension."
      (7 80)
      (8 90)
      (9 100)))
-  (espeak-css-set-code-table 'paul 'pitch-range table ))
+  (espeak-css-set-code-table 'paul 'pitch-range table))
 
 ;;}}}
 ;;{{{  harry pitch range
 
 (let ((table (make-vector 10 "")))
-  (mapcar
-   (function
-    (lambda (setting)
-      (aset table
-            (first setting)
-            (format "<prosody range=\"%s\">"
-                    (second setting)))))
+  (mapc
+   #'(lambda (setting)
+       (aset table
+             (cl-first setting)
+             (format "<prosody range=\"%s\">"
+                     (cl-second setting))))
    '(
      (0 0)
      (1 10)
@@ -283,19 +294,18 @@ and TABLE gives the values along that dimension."
      (7 80)
      (8 90)
      (9 100)))
-  (espeak-css-set-code-table 'harry 'pitch-range table ))
+  (espeak-css-set-code-table 'harry 'pitch-range table))
 
 ;;}}}
 ;;{{{  betty pitch range
 
 (let ((table (make-vector 10 "")))
-  (mapcar
-   (function
-    (lambda (setting)
-      (aset table
-            (first setting)
-            (format "<prosody range=\"%s\">"
-                    (second setting)))))
+  (mapc
+   #'(lambda (setting)
+       (aset table
+             (cl-first setting)
+             (format "<prosody range=\"%s\">"
+                     (cl-second setting))))
    '(
      (0 0)
      (1 10)
@@ -307,10 +317,10 @@ and TABLE gives the values along that dimension."
      (7 80)
      (8 90)
      (9 100)))
-  (espeak-css-set-code-table 'betty 'pitch-range table ))
+  (espeak-css-set-code-table 'betty 'pitch-range table))
 
 ;;}}}
-(defsubst espeak-get-pitch-range-code (value family)
+(defun espeak-get-pitch-range-code (value family)
   "Get pitch-range code for specified VALUE and FAMILY."
   (or family (setq family 'paul))
   (if value
@@ -323,7 +333,7 @@ and TABLE gives the values along that dimension."
 
 ;;;  Not implemented fo Espeak now.
 
-(defsubst espeak-get-stress-code (value family)
+(defun espeak-get-stress-code (_value _family)
   "Just a dummy."
   "")
 
@@ -337,15 +347,14 @@ and TABLE gives the values along that dimension."
 ;;{{{  paul richness
 
 (let ((table (make-vector 10 "")))
-  (mapcar
-   (function
-    (lambda (setting)
-      (aset table
-            (first setting)
-            (format "<prosody volume=\"%s\">"
-                    (second setting)))))
+  (mapc
+   #'(lambda (setting)
+       (aset table
+             (cl-first setting)
+             (format "<prosody volume=\"%s\">"
+                     (cl-second setting))))
    ;;            (format " ri:%s sm:%s "
-   ;;                    (third setting)))))
+   ;;                    (cl-third setting)))))
    '(
      (0 10 100)
      (1 20 80)
@@ -363,15 +372,14 @@ and TABLE gives the values along that dimension."
 ;;{{{  harry richness
 
 (let ((table (make-vector 10 "")))
-  (mapcar
-   (function
-    (lambda (setting)
-      (aset table (first setting)
-            (format "<prosody volume=\"%s\">"
-                    (second setting)))))
+  (mapc
+   #'(lambda (setting)
+       (aset table (cl-first setting)
+             (format "<prosody volume=\"%s\">"
+                     (cl-second setting))))
    ;;            (format " ri:%s sm:%s "
-   ;;                     (second setting)
-   ;;                     (third setting)))))
+   ;;                     (cl-second setting)
+   ;;                     (cl-third setting)))))
    '(
      (0 10 100)
      (1 20 80)
@@ -389,15 +397,14 @@ and TABLE gives the values along that dimension."
 ;;{{{  betty richness
 
 (let ((table (make-vector 10 "")))
-  (mapcar
-   (function
-    (lambda (setting)
-      (aset table (first setting)
-            (format "<prosody volume=\"%s\">"
-                    (second setting)))))
+  (mapc
+   #'(lambda (setting)
+       (aset table (cl-first setting)
+             (format "<prosody volume=\"%s\">"
+                     (cl-second setting))))
    ;;            (format " ri:%s sm:%s "
-   ;;                     (second setting)
-   ;;                     (third setting)))))
+   ;;                     (cl-second setting)
+   ;;                     (cl-third setting)))))
    '(
      (0 10 100)
      (1 20 80)
@@ -413,7 +420,7 @@ and TABLE gives the values along that dimension."
 
 ;;}}}
 
-(defsubst espeak-get-richness-code (value family)
+(defun espeak-get-richness-code (value family)
   (or family (setq family 'paul))
   (if value 
       (aref (espeak-css-get-code-table family 'richness)
@@ -423,7 +430,7 @@ and TABLE gives the values along that dimension."
 ;;}}}
 ;;{{{  punctuations
 
-(defsubst espeak-get-punctuations-code (value)
+(defun espeak-get-punctuations-code (_value)
   "Return string needed to set specified punctuations mode."
   "")
 
@@ -442,7 +449,7 @@ and TABLE gives the values along that dimension."
            (espeak-get-punctuations-code (acss-punctuations style))
            (espeak-get-average-pitch-code (acss-average-pitch style) family)
            (espeak-get-pitch-range-code (acss-pitch-range style) family)
-           (espeak-get-stress-code (acss-stress style ) family)
+           (espeak-get-stress-code (acss-stress style) family)
            (espeak-get-richness-code (acss-richness style) family)
            )))
     ;;                  "]")))
@@ -453,9 +460,9 @@ and TABLE gives the values along that dimension."
 
 (defun espeak-list-voices ()
   "List defined voices."
-  (declare (special espeak-voice-table))
-  (loop for k being the hash-keys of espeak-voice-table 
-        collect   k))
+  (cl-declare (special espeak-voice-table))
+  (cl-loop for k being the hash-keys of espeak-voice-table 
+           collect   k))
 
 ;;}}}
 ;;{{{ Configurater 
@@ -468,22 +475,23 @@ and TABLE gives the values along that dimension."
              (boundp 'dtk-character-to-speech-table)
              (vectorp dtk-character-to-speech-table))
     (setq espeak-character-to-speech-table
-          (let ((table (copy-seq dtk-character-to-speech-table)))
-            (loop for entry across-ref table 
-                  when   (string-match "\\(\\[\\*\\]\\)"  entry) do
-                  (setf entry (replace-match " " nil nil  entry 1)))
+          (let ((table (cl-copy-seq  dtk-character-to-speech-table)))
+            (cl-loop for entry across-ref table 
+                     when   (string-match "\\(\\[\\*\\]\\)"  entry) do
+                     (setf entry (replace-match " " nil nil  entry 1)))
             table))))
-
+;;;###autoload
 (defun espeak-configure-tts ()
   "Configure TTS environment to use eSpeak."
-  (declare (special tts-default-speech-rate
-                    espeak-default-speech-rate
-                    dtk-speaker-process
-                    emacspeak-unspeakable-rule))
+  (cl-declare (special tts-default-speech-rate
+                       espeak-default-speech-rate
+                       dtk-speaker-process
+                       emacspeak-unspeakable-rule))
   (fset 'tts-list-voices'espeak-list-voices)
   (fset 'tts-voice-defined-p 'espeak-voice-defined-p)
   (fset 'tts-get-voice-command 'espeak-get-voice-command)
-  (fset 'tts-define-voice-from-speech-style 'espeak-define-voice-from-speech-style)
+  (fset
+   'tts-define-voice-from-speech-style 'espeak-define-voice-from-speech-style)
   (setq tts-default-voice nil)
   (setq tts-default-speech-rate espeak-default-speech-rate)
   (set-default 'tts-default-speech-rate espeak-default-speech-rate)
@@ -491,6 +499,23 @@ and TABLE gives the values along that dimension."
   (dtk-unicode-update-untouched-charsets '(ascii latin-iso8859-1)))
 
 ;;}}}
+;;{{{ tts-env for Espeak:
+;;;###autoload
+(defun espeak-make-tts-env  ()
+  "Constructs a TTS environment for Espeak."
+  (cl-declare (special espeak-default-speech-rate))
+  (make-tts-env
+   :name :espeak :default-voice 'paul
+   :default-speech-rate espeak-default-speech-rate
+   :list-voices #'espeak-list-voices
+   :acss-voice-defined-p #'espeak-voice-defined-p
+   :get-acss-voice-command #'espeak-get-voice-command
+   :define-voice-from-acss #'espeak-define-voice-from-speech-style
+   :speech-rate-base 100 :speech-rate-step 10))
+
+(tts-env-set :espeak  (espeak-make-tts-env))
+
+;;}}}  
 (provide 'espeak-voices)
 ;;{{{  emacs local variables
 
