@@ -1,5 +1,5 @@
-;;; outloud-voices.el --- Define various device independent voices in terms of OutLoud tags
-;;; $Id: outloud-voices.el 7511 2012-02-13 21:55:37Z tv.raman.tv $
+;;; outloud-voices.el --- Define  OutLoud tags  -*- lexical-binding: t; -*-
+;;; $Id$
 ;;; $Author: tv.raman.tv $
 ;;; Description:  Module to set up Eloquent voices and personalities
 ;;; Keywords: Voice, Personality, IBM ViaVoice Outloud
@@ -16,7 +16,7 @@
 ;;}}}
 ;;{{{  Copyright:
 
-;;;Copyright (C) 1995 -- 2011, T. V. Raman 
+;;;Copyright (C) 1995 -- 2018, T. V. Raman
 ;;; All Rights Reserved.
 ;;;
 ;;; This file is not part of GNU Emacs, but the same permissions apply.
@@ -48,29 +48,41 @@
 ;;}}}
 ;;{{{ Required modules
 
-(require 'cl)
-(declaim  (optimize  (safety 0) (speed 3)))
+(require 'cl-lib)
+(cl-declaim  (optimize  (safety 0) (speed 3)))
 (require 'acss-structure)
 (require 'dtk-unicode)
+(require 'tts)
+
+;;}}}
+;;{{{ Customizations:
+;;;###autoload
+(defcustom outloud-default-speech-rate 50
+  "Default speech rate for outloud."
+  :group 'tts
+  :type 'integer
+  :set #'(lambda(sym val)
+           (set-default sym val)
+           (when (and (getenv "DTK_PROGRAM")
+                      (string-match "outloud" (getenv "DTK_PROGRAM")))
+             (setq-default dtk-speech-rate val))))
 
 ;;}}}
 ;;{{{ Forward declarations:
 
 ;;; From dtkk-speak.el:
 (defvar dtk-speech-rate)
-(defvar tts-default-speech-rate )
-(defvar dectalk-default-speech-rate )
-(defvar dtk-speech-rate-step )
-(defvar dtk-speech-rate-base )
-(defvar outloud-default-speech-rate)
+(defvar tts-default-speech-rate)
+(defvar dtk-speech-rate-step)
+(defvar dtk-speech-rate-base)
 ;;}}}
 ;;{{{ Top level TTS  switcher
 
 ;;;### autoload
-(defun outloud ()
+(defun outloud (&optional device)
   "Select Outloud server."
-  (interactive)
-  (dtk-select-server "outloud")
+  (interactive "P")
+  (funcall-interactively #'dtk-select-server "outloud" device)
   (dtk-initialize))
 
 (defun outloud-32()
@@ -89,33 +101,33 @@
   "Association between symbols and strings to set Outloud  voices.
 The string can set any voice parameter.")
 
-(defsubst outloud-define-voice (name command-string)
+(defun outloud-define-voice (name command-string)
   "Define a Outloud  voice named NAME.
 This voice will be set   by sending the string
 COMMAND-STRING to the TTS engine."
-  (declare (special outloud-voice-table ))
+  (cl-declare (special outloud-voice-table))
   (puthash name command-string outloud-voice-table))
 
-(defsubst outloud-get-voice-command-internal  (name)
+(defun outloud-get-voice-command-internal  (name)
   "Retrieve command string for  voice NAME."
-  (declare (special outloud-voice-table))
+  (cl-declare (special outloud-voice-table))
   (cond
    ((listp name)
-    (mapconcat #'outloud-get-voice-command name " "))
+    (mapconcat #'outloud-get-voice-command-internal name " "))
    (t (or  (gethash name outloud-voice-table)
            outloud-default-voice-string))))
 
-(defsubst outloud-get-voice-command (name)
+(defun outloud-get-voice-command (name)
   "Retrieve command string for  voice NAME."
-  (declare (special dtk-speech-rate))
-  (concat 
+  (cl-declare (special dtk-speech-rate))
+  (concat
    (outloud-get-voice-command-internal name)
-   (format " `vs%s " dtk-speech-rate )))
+   (format "`vs%s" dtk-speech-rate)))
 
-(defsubst outloud-voice-defined-p (name)
+(defun outloud-voice-defined-p (name)
   "Check if there is a voice named NAME defined."
-  (declare (special outloud-voice-table ))
-  (gethash name outloud-voice-table ))
+  (cl-declare (special outloud-voice-table))
+  (gethash name outloud-voice-table))
 
 ;;}}}
 ;;{{{ voice definitions
@@ -143,7 +155,7 @@ COMMAND-STRING to the TTS engine."
 
 ;;{{{ voice family codes
 
-(defsubst outloud-get-family-code (name)
+(defun outloud-get-family-code (name)
   "Get control code for voice family NAME."
   (outloud-get-voice-command-internal name))
 
@@ -155,17 +167,17 @@ COMMAND-STRING to the TTS engine."
 Keys are symbols of the form <FamilyName-Dimension>.
 Values are vectors holding the control codes for the 10 settings.")
 
-(defsubst outloud-css-set-code-table (family dimension table)
+(defun outloud-css-set-code-table (family dimension table)
   "Set up voice FAMILY.
 Argument DIMENSION is the dimension being set,
 and TABLE gives the values along that dimension."
-  (declare (special outloud-css-code-tables))
+  (cl-declare (special outloud-css-code-tables))
   (let ((key (intern (format "%s-%s" family dimension))))
-    (puthash key table outloud-css-code-tables )))
+    (puthash key table outloud-css-code-tables)))
 
-(defsubst outloud-css-get-code-table (family dimension)
+(defun outloud-css-get-code-table (family dimension)
   "Retrieve table of values for specified FAMILY and DIMENSION."
-  (declare (special outloud-css-code-tables))
+  (cl-declare (special outloud-css-code-tables))
   (let ((key (intern (format "%s-%s" family dimension))))
     (gethash key outloud-css-code-tables)))
 
@@ -180,94 +192,92 @@ and TABLE gives the values along that dimension."
 
 ;;; Average pitch for standard male voice is 65 --this is mapped to
 ;;; a setting of 5.
+;;; head-size for default male is 50.
 ;;; Average pitch varies inversely with speaker head size --a child
 ;;; has a small head and a higher pitched voice.
 ;;; We change parameter head-size in conjunction with average pitch to
 ;;; produce a more natural change on the TTS engine.
 
 ;;{{{  paul average pitch
-
+;;; median: pitch: 65  head-size 50
 (let ((table (make-vector 10 "")))
-  (mapcar
-   (function
-    (lambda (setting)
-      (aset table
-            (first setting)
-            (format " `vb%s `vh%s "
-                    (second setting)
-                    (third setting)))))
+  (mapc
+   #'(lambda (setting)
+       (aset table
+             (cl-first setting)
+             (format
+              " `vb%s `vh%s "
+              (cl-second setting) (cl-third setting))))
    '(
-     (0 44 52)
-     (1 50 58 )
-     (2 56 56)
-     (3 58 54)
-     (4 62 52  )
+     (0 40 75) ; pitch, head-size
+     (1 45 70)
+     (2 50 65)
+     (3 55 60)
+     (4 60 55)
      (5 65 50)
-     (6 69 48)
-     (7 73 46 )
-     (8 77 44)
-     (9 82 40)))
-  (outloud-css-set-code-table 'paul 'average-pitch table ))
+     (6 70 45)
+     (7 75 40)
+     (8 80 35)
+     (9 85 30)))
+  (outloud-css-set-code-table 'paul 'average-pitch table))
 
 ;;}}}
 ;;{{{  harry average pitch
 
 (let ((table (make-vector 10 "")))
 
-  (mapcar
-   (function
-    (lambda (setting)
-      (aset table
-            (first setting)
-            (format " `vb%s `vh% s"
-                    (second setting)
-                    (third setting)))))
+  (mapc
+   #'(lambda (setting)
+       (aset table
+             (cl-first setting)
+             (format " `vb%s `vh% s"
+                     (cl-second setting)
+                     (cl-third setting))))
    '(
      (0 0 90)
-     (1 10 85 )
+     (1 10 85)
      (2 20 80)
      (3 30 70)
      (4 40 60)
      (5 50 60)
      (6 60 50)
-     (7 70 40 )
+     (7 70 40)
      (8 80 30)
      (9 90 20)))
-  (outloud-css-set-code-table 'harry 'average-pitch table ))
+  (outloud-css-set-code-table 'harry 'average-pitch table))
 
 ;;}}}
 ;;{{{  betty average pitch
 
-;;;defalt baseline is average pitch of 81 
+;;;defalt baseline is average pitch of 81
 
 (let ((table (make-vector 10 "")))
-  (mapcar
-   (function
-    (lambda (setting)
-      (aset table
-            (first setting)
-            (format " `vb%s `vh% s"
-                    (second setting)
-                    (third setting)))))
+  (mapc
+   #'(lambda (setting)
+       (aset table
+             (cl-first setting)
+             (format " `vb%s `vh% s"
+                     (cl-second setting)
+                     (cl-third setting))))
    '(
      (0 5 70)
      (1 17 66)
      (2 33 62)
      (3 49 58)
-     (4 65 54 )
+     (4 65 54)
      (5 81  50)
      (6 85 55)
      (7 89  60)
      (8 93 65)
      (9 97 69)))
-  (outloud-css-set-code-table 'betty 'average-pitch table ))
+  (outloud-css-set-code-table 'betty 'average-pitch table))
 
 ;;}}}
 
-(defsubst outloud-get-average-pitch-code (value family)
+(defun outloud-get-average-pitch-code (value family)
   "Get  AVERAGE-PITCH for specified VALUE and  FAMILY."
   (or family (setq family 'paul))
-  (if value 
+  (if value
       (aref (outloud-css-get-code-table family 'average-pitch)
             value)
     ""))
@@ -283,79 +293,76 @@ and TABLE gives the values along that dimension."
 ;;{{{  paul pitch range
 
 (let ((table (make-vector 10 "")))
-  (mapcar
-   (function
-    (lambda (setting)
-      (aset table
-            (first setting)
-            (format " `vf%s  "
-                    (second setting)))))
+  (mapc
+   #'(lambda (setting)
+       (aset table
+             (cl-first setting)
+             (format
+              " `vf%s  " (cl-second setting))))
    '(
-     (0 0 )
-     (1 5 )
+     (0 0)
+     (1 5)
      (2  15)
      (3  20)
-     (4  25 )
-     (5  30 )
+     (4  25)
+     (5  30)
      (6  47)
      (7  64)
      (8  81)
      (9  100)))
-  (outloud-css-set-code-table 'paul 'pitch-range table ))
+  (outloud-css-set-code-table 'paul 'pitch-range table))
 
 ;;}}}
 ;;{{{  harry pitch range
 
 (let ((table (make-vector 10 "")))
-  (mapcar
-   (function
-    (lambda (setting)
-      (aset table
-            (first setting)
-            (format " `vf%s  "
-                    (second setting)))))
+  (mapc
+   #'(lambda (setting)
+       (aset table
+             (cl-first setting)
+             (format " `vf%s  "
+                     (cl-second setting))))
    '(
-     (0 0 )
-     (1 5 )
+     (0 0)
+     (1 5)
      (2  15)
      (3  20)
-     (4  25 )
-     (5  30 )
+     (4  25)
+     (5  30)
      (6  47)
      (7  64)
      (8  81)
      (9  100)))
-  (outloud-css-set-code-table 'harry 'pitch-range table ))
+  (outloud-css-set-code-table 'harry 'pitch-range table))
 
 ;;}}}
 ;;{{{  betty pitch range
 
 (let ((table (make-vector 10 "")))
-  (mapcar
-   (function
-    (lambda (setting)
-      (aset table
-            (first setting)
-            (format " `vf%s  "
-                    (second setting)))))
+  (mapc
+   #'(lambda (setting)
+       (aset table
+             (cl-first setting)
+             (format " `vf%s  "
+                     (cl-second setting))))
    '(
-     (0 0 )
-     (1 5 )
+     (0 0)
+     (1 5)
      (2  15)
      (3  20)
-     (4  25 )
-     (5  30 )
+     (4  25)
+     (5  30)
      (6  47)
      (7  64)
      (8  81)
      (9  100)))
-  (outloud-css-set-code-table 'betty 'pitch-range table ))
+  (outloud-css-set-code-table 'betty 'pitch-range table))
 
 ;;}}}
-(defsubst outloud-get-pitch-range-code (value family)
+(defun outloud-get-pitch-range-code (value family)
   "Get pitch-range code for specified VALUE and FAMILY."
   (or family (setq family 'paul))
-  (if value 
+  (if value
       (aref (outloud-css-get-code-table family 'pitch-range)
             value)
     ""))
@@ -364,23 +371,23 @@ and TABLE gives the values along that dimension."
 ;;{{{  stress
 
 ;;; On the outloud we map stress to roughness
-;;; we also use stress markers `00 .. `4 
+;;; we also use stress markers `00 .. `4  (disabled after experimentation)
 ;;{{{  paul stress
 
 (let ((table (make-vector 10 "")))
-  (mapcar
+  (mapc
    #'(lambda (setting)
-       (aset table (first setting)
+       (aset table (cl-first setting)
              (format " `vr%s  "
-                     (second setting))))
+                     (cl-second setting))))
 ;;; stress markers not used for now.
    '(
      (0 0 "`00")
      (1 5 "`00")
      (2  10 "`0")
      (3  15 "`0")
-     (4  20 "`1" )
-     (5  25 "`1" )
+     (4  20 "`1")
+     (5  25 "`1")
      (6  30 "`v2")
      (7  35 "`v2")
      (8  40 "`v3")
@@ -390,9 +397,9 @@ and TABLE gives the values along that dimension."
   (outloud-css-set-code-table 'betty  'stress table))
 
 ;;}}}
-(defsubst outloud-get-stress-code (value family)
-  (or family (setq family 'paul ))
-  (if value 
+(defun outloud-get-stress-code (value family)
+  (or family (setq family 'paul))
+  (if value
       (aref (outloud-css-get-code-table family 'stress)
             value)
     ""))
@@ -403,15 +410,14 @@ and TABLE gives the values along that dimension."
 ;;{{{  paul richness
 
 (let ((table (make-vector 10 "")))
-  (mapcar
-   (function
-    (lambda (setting)
-      (aset table
-            (first setting)
-            (format " `vy%s  `vv%s "
-                    (second setting)
-                    (third setting)))))
-   '(
+  (mapc
+   #'(lambda (setting)
+       (aset table
+             (cl-first setting)
+             (format
+              " `vy%s  `vv%s "
+              (cl-second setting) (cl-third setting))))
+   '(; whisper, volume
      (0 0 60)
      (1 4 78)
      (2 8 80)
@@ -420,7 +426,7 @@ and TABLE gives the values along that dimension."
      (5 20 92)
      (6 24 93)
      (7 28 95)
-     (8 32 97 )
+     (8 32 97)
      (9 36 100)))
   (outloud-css-set-code-table 'paul 'richness table)
   (outloud-css-set-code-table 'harry 'richness table)
@@ -428,9 +434,9 @@ and TABLE gives the values along that dimension."
 
 ;;}}}
 
-(defsubst outloud-get-richness-code (value family)
+(defun outloud-get-richness-code (value family)
   (or family (setq family 'paul))
-  (if value 
+  (if value
       (aref (outloud-css-get-code-table family 'richness)
             value)
     ""))
@@ -438,7 +444,7 @@ and TABLE gives the values along that dimension."
 ;;}}}
 ;;{{{  punctuations
 
-(defsubst outloud-get-punctuations-code (value)
+(defun outloud-get-punctuations-code (_value)
   "Return string needed to set specified punctuations mode."
   "")
 
@@ -450,47 +456,65 @@ and TABLE gives the values along that dimension."
   "Define NAME to be a outloud voice as specified by settings in STYLE."
   (let* ((family(acss-family style))
          (command
-          (concat 
+          (concat
            (outloud-get-family-code family)
            (outloud-get-punctuations-code (acss-punctuations style))
            (outloud-get-average-pitch-code (acss-average-pitch style) family)
            (outloud-get-pitch-range-code (acss-pitch-range style) family)
-           (outloud-get-stress-code (acss-stress style ) family)
+           (outloud-get-stress-code (acss-stress style) family)
            (outloud-get-richness-code (acss-richness style) family))))
     (outloud-define-voice name command)))
 
 ;;}}}
-;;{{{ list voices 
+;;{{{ list voices
 
 (defun outloud-list-voices ()
   "List defined voices."
-  (declare (special outloud-voice-table))
-  (loop for k being the hash-keys of outloud-voice-table 
-        collect   k))
+  (cl-declare (special outloud-voice-table))
+  (cl-loop for k being the hash-keys of outloud-voice-table collect   k))
 
 ;;}}}
-;;{{{ Configurater 
+;;{{{ Configurater
 ;;;###autoload
 (defun outloud-configure-tts ()
   "Configure TTS environment to use ViaVoice  family of synthesizers."
-  (declare (special tts-default-speech-rate
-                    outloud-default-speech-rate
-                    dtk-speech-rate-step
-                    dtk-speech-rate-base
-                    dtk-speaker-process))
+  (cl-declare (special tts-default-speech-rate tts-default-voice
+                       outloud-default-speech-rate
+                       dtk-speech-rate-step dtk-speech-rate-base))
   (fset 'tts-list-voices'outloud-list-voices)
   (fset 'tts-voice-defined-p 'outloud-voice-defined-p)
   (fset 'tts-get-voice-command 'outloud-get-voice-command)
-  (fset 'tts-define-voice-from-speech-style 'outloud-define-voice-from-speech-style)
+  (fset
+   'tts-define-voice-from-speech-style 'outloud-define-voice-from-speech-style)
+  (setq tts-default-voice 'paul)
   (setq tts-default-speech-rate outloud-default-speech-rate)
   (set-default 'tts-default-speech-rate outloud-default-speech-rate)
-  (setq dtk-speech-rate-step 8
+  (setq dtk-speech-rate-step 10
         dtk-speech-rate-base 50
         dtk-speech-rate outloud-default-speech-rate)
-  (setq-default dtk-speech-rate-step 8
+  (setq-default dtk-speech-rate-step 10
                 dtk-speech-rate outloud-default-speech-rate
                 dtk-speech-rate-base 50)
-  (dtk-unicode-update-untouched-charsets '(ascii latin-iso8859-1 latin-iso8859-15 latin-iso8859-9 eight-bit-graphic)))
+  (dtk-unicode-update-untouched-charsets
+   '(ascii latin-iso8859-1 latin-iso8859-15 latin-iso8859-9 eight-bit-graphic)))
+
+;;}}}
+;;{{{  tts-env for Outloud:
+
+;;;###autoload
+(defun outloud-make-tts-env  ()
+  "Constructs a TTS environment for Outloud."
+  (cl-declare (special outloud-default-speech-rate))
+  (make-tts-env
+   :name :outloud :default-voice 'paul
+   :default-speech-rate outloud-default-speech-rate
+   :list-voices #'outloud-list-voices
+   :acss-voice-defined-p #'outloud-voice-defined-p
+   :get-acss-voice-command #'outloud-get-voice-command
+   :define-voice-from-acss #'outloud-define-voice-from-speech-style
+   :speech-rate-base 50 :speech-rate-step 10))
+
+(tts-env-set :outloud  (outloud-make-tts-env))
 
 ;;}}}
 (provide 'outloud-voices)
